#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#Bookmarks Dialog

import os.path
import wx
import drScrolledMessageDialog
import drFileDialog
from drProperty import *

def CheckBranch(tree, branch):
	t = tree.GetItemText(branch)
	isfolder = (t[0] == '>')
	if isfolder:
		ccount = tree.GetChildrenCount(branch, 0)
		if (ccount < 1):
			tree.SetBadBranch(branch)
			return 0
		else:
			if (wx.MAJOR_VERSION >= 2) and (wx.MINOR_VERSION >= 5):
				b, cookie = tree.GetFirstChild(branch)
			else:
				b, cookie = tree.GetFirstChild(branch, 1)
			rvalue = CheckBranch(tree, b)
			x = 1
			while (x < ccount):
				b, cookie = tree.GetNextChild(branch, cookie)
				rvalue = rvalue and CheckBranch(tree, b)
				x = x + 1
			return rvalue
	return 1

def CreateBranchString(tree, branch, tablevel):	
	t = tree.GetItemText(branch)
	isfolder = (t[0] == '>')
	x = 0
	y = ""
	while (x < tablevel):
		y = y + '\t'
		x = x + 1
	y = y + t + "\n"
	s = y
	if isfolder:
		ccount = tree.GetChildrenCount(branch, 0)
		if (ccount > 0):
			if (wx.MAJOR_VERSION >= 2) and (wx.MINOR_VERSION >= 5):
				b, cookie = tree.GetFirstChild(branch)
			else:
				b, cookie = tree.GetFirstChild(branch, 1)
			s = s + CreateBranchString(tree, b, (tablevel + 1))
			x = 1
			while (x < ccount):
				b, cookie = tree.GetNextChild(branch, cookie)
				s = s + CreateBranchString(tree, b, (tablevel + 1))
				x = x + 1
	return s

def BuildTreeFromString(tree, branch, thestring):
	line = " "
	roots = [branch]
	rootindex = 0
	lastCount = 1
	i = 0
	lastI = 0
	#Skip the First Line
	i = thestring.find('\n')
	if (i > -1):
		line = thestring[0:(i + 1)]
		lastI = i + 1
	thestring = thestring[lastI:]
	#Get On With It!
	while (len(line) > 0):
		i = thestring.find('\n')
		if (i > -1):
			line = thestring[0:(i + 1)]
			lastI = i + 1
			thestring = thestring[lastI:]
			c = line.count('\t')
			line = line[c:].rstrip()
			while (lastCount > c):
				roots.pop()
				rootindex = rootindex - 1
				lastCount = lastCount - 1
			currentItem = tree.AppendItem(roots[rootindex], line)
			if (line[0] == '>'):
				tree.SetItemImage(currentItem, 0, wx.TreeItemIcon_Normal)
				tree.SetItemImage(currentItem, 1, wx.TreeItemIcon_Expanded)
				roots.append(currentItem)
				rootindex = rootindex + 1
				lastCount = c + 1
			else:
				tree.SetItemImage(currentItem, 2, wx.TreeItemIcon_Normal)
				tree.SetItemImage(currentItem, 2, wx.TreeItemIcon_Selected)			
		else:
			line = ""

def WriteBranch(tree, branch, filehandle, tablevel):
	t = tree.GetItemText(branch)
	isfolder = (t[0] == '>')
	x = 0
	y = ""
	while (x < tablevel):
		y = y + '\t'
		x = x + 1
	y = y + t + "\n"
	filehandle.write(y)
	if isfolder:
		ccount = tree.GetChildrenCount(branch, 0)
		if (ccount > 0):
			if (wx.MAJOR_VERSION >= 2) and (wx.MINOR_VERSION >= 5):
				b, cookie = tree.GetFirstChild(branch)
			else:
				b, cookie = tree.GetFirstChild(branch, 1)
			WriteBranch(tree, b, filehandle, (tablevel + 1))
			x = 1
			while (x < ccount):
				b, cookie = tree.GetNextChild(branch, cookie)
				WriteBranch(tree, b, filehandle, (tablevel + 1))
				x = x + 1

class drTree(wx.TreeCtrl):
	def __init__(self, parent, id, point, size, style):
		wx.TreeCtrl.__init__(self, parent, id, point, size, style)
		
		self.grandparent = self.GetGrandParent()
		
		style = self.grandparent.prefs.bookmarksstyle
		yarrr = convertStyleStringToWXFontArray(style)
		
		imagesize = (16,16)
		
		self.imagelist = wx.ImageList(imagesize[0], imagesize[1])
		
		self.images = [wx.BitmapFromImage(wx.Image(self.grandparent.programdirectory + "/bitmaps/16/folder.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(self.grandparent.programdirectory + "/bitmaps/16/folder open.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(self.grandparent.programdirectory + "/bitmaps/16/bookmark.png", wx.BITMAP_TYPE_PNG))]
		
		map(self.imagelist.Add, self.images)
		
		self.AssignImageList(self.imagelist)
		
		w = wx.Font(yarrr[1], wx.NORMAL, wx.NORMAL, wx.NORMAL, yarrr[2])
		
		w.SetFaceName(yarrr[0])
		
		if (yarrr[3]):
			w.SetWeight(wx.BOLD)
		else:
			w.SetWeight(wx.NORMAL)
		if (yarrr[4]):
			w.SetStyle(wx.ITALIC)
		else:
			w.SetStyle(wx.NORMAL)
		
		self.SetFont(w)
					
		f = convertColorPropertyToColorArray(getStyleProperty("fore", style))
		b = convertColorPropertyToColorArray(getStyleProperty("back", style))
				
		self.TextColor = wx.Colour(f[0], f[1], f[2])
		
		self.SetForegroundColour(self.TextColor)
							
		self.badbranch = 0
		self.dragging = 0
		self.draggingId = self.GetRootItem()
		self.dragString = ""
		
		self.SetBackgroundColour(wx.Colour(b[0], b[1], b[2]))

		self.edittext = ""

		self.Bind(wx.EVT_TREE_BEGIN_DRAG,  self.OnBeginDrag, id=id)
		self.Bind(wx.EVT_TREE_ITEM_ACTIVATED,  self.OnItemActivated, id=id)
		self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
		self.Bind(wx.EVT_MOTION, self.OnMotion)
		self.Bind(wx.EVT_TREE_END_LABEL_EDIT, self.OnLabelEdited)
		self.Bind(wx.EVT_LEFT_DOWN, self.OnLabelEdited)
		
	def GetBadBranch(self):
		return self.badbranch		
				
	def OnBeginDrag(self, event):				
		self.draggingId = self.GetSelection()
		if (self.draggingId == self.GetRootItem()) or (not self.draggingId.IsOk()):
			return
				
		t = self.GetItemText(self.draggingId)
		if (t[0] == '>'):
			self.dragString = CreateBranchString(self, self.draggingId, 0)
		
		self.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
		self.dragging = 1
					
	def OnItemActivated(self, event):
		sel = self.GetSelection()
		if (not sel.IsOk()):
			return
		self.edittext = self.GetItemText(sel)
		self.EditLabel(sel)
	
	def OnLabelEdited(self, event):		
		if len(self.edittext) > 0:
			if self.edittext[0] == '>':
				sel = self.GetSelection()
				text = self.GetItemText(sel)
				if not text[0] == '>':
					self.SetItemText(sel, '>' + text)
		event.Skip()
		
	def OnLeftUp(self, event):
		self.SetCursor(wx.STANDARD_CURSOR)
		if (self.dragging):
			self.dragging = 0
			item, flags = self.HitTest(event.GetPosition())
			if (item.IsOk()):
				if (item == self.draggingId):
					return
				t = self.GetItemText(item)
				olditem = self.GetItemParent(item)
				if not (t[0] == '>'):
					olditem = item
					item = self.GetItemParent(item)				
				icon = [wx.TreeItemIcon_Normal, wx.TreeItemIcon_Selected]
				iconNum = [2, 2]
				if len(self.dragString) > 0:
					if self.dragString[0] == '>':
						icon = [wx.TreeItemIcon_Normal, wx.TreeItemIcon_Expanded]
						iconNum = [0, 1]
				if t[0] == '>':
					newbranch = self.InsertItemBefore(item, 0, self.GetItemText(self.draggingId))
				else:
					newbranch = self.InsertItem(item, olditem, self.GetItemText(self.draggingId))
				self.SetItemImage(newbranch, iconNum[0], icon[0])
				self.SetItemImage(newbranch, iconNum[1], icon[1])				
				if len(self.dragString) > 0:
					BuildTreeFromString(self, newbranch, self.dragString)
				self.Delete(self.draggingId)
				self.SelectItem(newbranch)
			self.dragString = ""
	
	def OnMotion(self, event):
		if (self.dragging):		
			sel, flags = self.HitTest(event.GetPosition())			
			if (sel.IsOk()):
				self.SelectItem(sel)						
		event.Skip()
		
	def SetBadBranch(self, branch):
		self.badbranch = branch	

class drBookmarksDialog(wx.Dialog):

	def __init__(self, parent, bookmarksfile):
		wx.Dialog.__init__(self, parent, -1, ("Edit Bookmarks"), wx.Point(50, 50), wx.Size(-1, -1), wx.DEFAULT_DIALOG_STYLE | wx.MAXIMIZE_BOX | wx.THICK_FRAME | wx.RESIZE_BORDER)
		
		self.ID_ADD = 1001
		self.ID_ADD_FOLDER = 1002
		self.ID_REMOVE = 1003
		self.ID_SAVE = 1005
		
		self.theSizer = wx.BoxSizer(wx.VERTICAL)
		
		self.cmdSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.homedirectory = parent.homedirectory
		self.wildcard = parent.wildcard
		self.bookmarksfile = bookmarksfile
		
		self.btnAdd = wx.Button(self, self.ID_ADD, "&Add")
		self.btnAddFolder = wx.Button(self, self.ID_ADD_FOLDER, "Add &Folder")
		self.btnRemove = wx.Button(self, self.ID_REMOVE, "&Remove")
		self.btnSave = wx.Button(self, self.ID_SAVE, "&Save")
		
		self.cmdSizer.Add(self.btnAdd, 0, wx.SHAPED)
		self.cmdSizer.Add(self.btnAddFolder, 0, wx.SHAPED)
		self.cmdSizer.Add(self.btnRemove, 0, wx.SHAPED)
		self.cmdSizer.Add(self.btnSave, 0, wx.SHAPED)

		self.theSizer.Add(self.cmdSizer, 0, wx.SHAPED | wx.ALIGN_CENTER)
		
		self.parent = parent
		
		self.bookmarkstree = drTree(self, -1, wx.Point(0, 0), wx.Size(400, 200), wx.TR_DEFAULT_STYLE | wx.TR_EDIT_LABELS)
		self.theSizer.Add(self.bookmarkstree, 9, wx.EXPAND)
	
		root = self.bookmarkstree.AddRoot(">Bookmarks")

		self.bookmarkstree.SetItemImage(root, 0, wx.TreeItemIcon_Normal)
		self.bookmarkstree.SetItemImage(root, 1, wx.TreeItemIcon_Expanded)
		
		self.btnClose = wx.Button(self, 101, "&Close")
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.btnClose, 0, wx.SHAPED | wx.ALIGN_CENTER)
		self.btnClose.SetDefault()
		
		self.SetAutoLayout(True)
		self.SetSizerAndFit(self.theSizer)
		
		self.initialize()
		
		self.bookmarkstree.Expand(root)

		self.Bind(wx.EVT_BUTTON,  self.OnbtnAdd, id=self.ID_ADD)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnAddFolder, id=self.ID_ADD_FOLDER)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnRemove, id=self.ID_REMOVE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnSave, id=self.ID_SAVE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnClose, id=101)

		self.parent.LoadDialogSizeAndPosition(self, 'bookmarksdialog.sizeandposition.dat')

	def OnCloseW(self, event):
		self.parent.SaveDialogSizeAndPosition(self, 'bookmarksdialog.sizeandposition.dat')
		if event is not None:
			event.Skip()
	
	def initialize(self):
		if (os.path.exists(self.bookmarksfile)):
			try:
				f = open(self.bookmarksfile, 'r')
				BuildTreeFromString(self.bookmarkstree, self.bookmarkstree.GetRootItem(), f.read())
				f.close()
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Your bookmarks file is a tad messed up.\n"), "Error")
				d.ShowModal()
				d.Destroy()
				
	def OnbtnAdd(self, event):
		sel = self.bookmarkstree.GetSelection()
		if (not sel.IsOk()):
			if self.bookmarkstree.GetCount() < 2:
				sel = self.bookmarkstree.GetRootItem()
			else:
				return				
		if (self.bookmarkstree.GetItemText(sel)[0] == '>'):
			d = wx.SingleChoiceDialog(self, "Add Bookmark:", "Add Bookmark", ["Type It In", "Select File", "Select Directory"], wx.OK|wx.CANCEL)
			d.SetSize(wx.Size(250, 200))
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_OK):
				s = d.GetStringSelection()
				currentItem = None
				if (s == "Type It In"):
					d = wx.TextEntryDialog(self, 'Enter Bookmark:', 'Add Bookmark', '')
					if (d.ShowModal() == wx.ID_OK):
						v = d.GetValue()
						currentItem = self.bookmarkstree.AppendItem(sel, v)
					d.Destroy()
				elif (s == "Select Directory"):
					d = wx.DirDialog(self, "Select Directory:", style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON|wx.MAXIMIZE_BOX|wx.THICK_FRAME)
					if (len(self.parent.ddirectory) > 0):
						try:
							d.SetPath(self.parent.ddirectory)			
						except:
							de = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Setting Default Directory To: " + self.parent.ddirectory), "DrPython Error")
							de.ShowModal()
							de.Destroy()
					if d.ShowModal() == wx.ID_OK:
						currentItem = self.bookmarkstree.AppendItem(sel, d.GetPath())
					d.Destroy()
				else:
					dlg = drFileDialog.FileDialog(self.parent, "Select File", self.wildcard)
					if (len(self.parent.ddirectory) > 0):
						try:
							dlg.SetDirectory(self.parent.ddirectory)			
						except:
							d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Setting Default Directory To: " + self.parent.ddirectory), "DrPython Error")
							d.ShowModal()
							d.Destroy()
					if (dlg.ShowModal() == wx.ID_OK):
						currentItem = self.bookmarkstree.AppendItem(sel, dlg.GetPath().replace("\\", "/"))
					dlg.Destroy()
					
				if currentItem is not None:
					self.bookmarkstree.SetItemImage(currentItem, 2, wx.TreeItemIcon_Normal)
					self.bookmarkstree.SetItemImage(currentItem, 2, wx.TreeItemIcon_Selected)
		else:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "You can only add a bookmark to a folder.\nSelect either \"Bookmarks\", or a folder.", "Bad Bookmark Folder")
			d.ShowModal()
			d.Destroy()
		
	def OnbtnAddFolder(self, event):
		sel = self.bookmarkstree.GetSelection()
		if (not sel.IsOk()):
			if self.bookmarkstree.GetCount() < 2:
				sel = self.bookmarkstree.GetRootItem()
			else:
				return
		if (self.bookmarkstree.GetItemText(sel)[0] == '>'):
			d = wx.TextEntryDialog(self, 'Enter Bookmark Folder:', 'Add Bookmark Folder', '')
			if (d.ShowModal() == wx.ID_OK):
				v = d.GetValue()
				item = self.bookmarkstree.AppendItem(self.bookmarkstree.GetSelection(), ">" + v)
				self.bookmarkstree.SetItemImage(item, 0, wx.TreeItemIcon_Normal)
				self.bookmarkstree.SetItemImage(item, 1, wx.TreeItemIcon_Expanded)
			d.Destroy()
		else:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "You can only add a folder to another folder.\nSelect either \"Bookmarks\", or a folder.", "Bad Bookmark Folder")
			d.ShowModal()
			d.Destroy()
		
	def OnbtnClose(self, event):
		self.Close(1)
		
	def OnbtnRemove(self, event):
		sel = self.bookmarkstree.GetSelection()
		if (not sel.IsOk()):
			return
		if (sel == self.bookmarkstree.GetRootItem()):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "You can't remove the Root Item.\n", "Mistake")
			d.ShowModal()
			d.Destroy()
			return			
		t = self.bookmarkstree.GetItemText(sel)
		message = "This will remove the bookmark: " + t
		if (self.bookmarkstree.GetItemText(sel)[0] == '>'):
			message = "This will remove all bookmarks in folder: " + t + "!"
		d = wx.MessageDialog(self, (message + "\nAre you sure you want to do this?"), "DrPython", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_NO):
			return
		try:	
			self.bookmarkstree.Delete(sel)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Something went wrong trying to remove that item....\nWhat's it called....\nOh yes: " + t + "\n"), "Error")
			d.ShowModal()
			d.Destroy()
			return	
	
	def OnbtnSave(self, event):
		if (not os.path.exists(self.homedirectory)):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Dude, you've got some problems...\nYour homedirectory (" + self.homedirectory + ") does not exist!\nLet's not bother speculating about how or why.\nRead the help file for this truly screwed up situation.\nDrPython will now politely ignore your request to save.\nTry again when you have fixed this problem."), "Huge Error")
			d.ShowModal()
			d.Destroy()
			return
		try:
			root = self.bookmarkstree.GetRootItem()
			f = open(self.bookmarksfile, 'w')
			WriteBranch(self.bookmarkstree, root, f, 0)
			f.close()			
		except IOError:			
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("There were some problems writing to:\n"  + self.bookmarksfile + "\nEither the file is having metaphysical issues, or you do not have permission to write.\nFor metaphysical issues, consult the documentation.\nFor permission issues, change the permissions on the directory to allow yourself write access.\nDrPython will now politely ignore your request to save.\nTry again when you have fixed the problem."), "Write Error")
			d.ShowModal()
			d.Destroy()
			return
		if self.parent.prefs.enablefeedback:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Succesfully wrote to:\n"  + self.bookmarksfile), "Saved Bookmarks")
			d.ShowModal()
			d.Destroy()	