#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#File Dialog

import os, os.path, re, stat, time, cStringIO
import wx
import drScrolledMessageDialog

class FileListCtrl(wx.ListView):
	def __init__(self, parent, id, fsize, fstyle):
		wx.ListView.__init__(self, parent, id, size=fsize, style=fstyle)
		
		bDir = parent.parent.programdirectory + '/bitmaps/16/'
			
		self.parent = parent	
		
		self.IsDetailed = False
		
		self.pywfilter = re.compile('.*\.pyw$')
		self.pyfilter = re.compile('.*\.py$')
				
		self.imagelist = wx.ImageList(16, 16)
		self.images = [wx.BitmapFromImage(wx.Image(bDir+'folder.png', wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+'py.png', wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+'pyw.png', wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+'unknown.png', wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+'up.png', wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+'down.png', wx.BITMAP_TYPE_PNG))]
		
		map(self.imagelist.Add, self.images)
		
		self.AssignImageList(self.imagelist, wx.IMAGE_LIST_SMALL)
		
		self.directories = ['']
		self.dirposition = -1
		
		self.ShowHiddenFiles = False
		
		self.DataArray = []
		self.currentcolumn = 0
		self.direction = [0, -1, -1, -1, -1]
				
		self.Bind(wx.EVT_LIST_COL_CLICK, self.OnColumnClick)
		self.Bind(wx.EVT_SIZE, self.OnSize)

	def GetDirectory(self):
		try:
			return self.directories[self.dirposition]
		except:
			return self.directories[0]

	def GoBack(self, filter):
		self.dirposition = self.dirposition - 1		
		if self.dirposition < 0:			
			self.dirposition = 0
		self.SetDirectoryTo(self.directories[self.dirposition], filter, False)

	def GoForward(self, filter):
		self.dirposition = self.dirposition + 1		
		if self.dirposition >= len(self.directories):
			self.dirposition = len(self.directories) - 1
		self.SetDirectoryTo(self.directories[self.dirposition], filter, False) 

	def IsDirectory(self, index):
		return self.DataArray[index][2] == '<Dir>'
	
	def OnColumnClick(self, event):
		oldcol = self.currentcolumn
		self.currentcolumn = event.GetColumn()
		self.ClearColumnImage(oldcol)
		if oldcol != self.currentcolumn:
			self.direction[oldcol] = -1
		if self.direction[self.currentcolumn] == -1:
			self.direction[self.currentcolumn] = 0
			self.SetColumnImage(self.currentcolumn, 5)
		else:
			self.direction[self.currentcolumn] = not self.direction[self.currentcolumn]
			if self.direction[self.currentcolumn]:
				self.SetColumnImage(self.currentcolumn, 4)
			else:
				self.SetColumnImage(self.currentcolumn, 5)
		self.SortItems(self.SortData)	
	
	def OnSize(self, event):
		if self.IsDetailed:
			width = self.GetSizeTuple()[0]
			self.SetColumnWidth(0, int(width * .4))
			self.SetColumnWidth(1, int(width * .15))
			self.SetColumnWidth(2, int(width * .15))
			self.SetColumnWidth(3, int(width * .15))
			self.SetColumnWidth(4, int(width * .15))
		if event is not None:
			event.Skip()
					
	def SetDirectoryTo(self, directory, filter="*", updatedirectories=True):
		if not os.path.exists(directory):
			return
		self.parent.txtDirectory.SetValue(directory)
		thelist = os.listdir(directory)
		if updatedirectories:
			self.directories = self.directories[:self.dirposition+1]
			self.directories.append(directory)
			self.dirposition = self.dirposition + 1
		dirs = []
		l = len(thelist)
		x = 0
		while x < l:
			if os.path.isdir(directory + '/' + thelist[x]):			
				dirs.append(thelist.pop(x))
				x = x - 1
				l = l - 1
			x = x + 1
		self.DeleteAllItems()
		thelist.sort()
		dirs.sort()
			
		filter = filter.replace('.', '\.').replace('*', '.*').replace(';', '$|') + '$'		
		refilter = re.compile(filter)

		dirs.insert(0, '..')
				
		for item in dirs:
			additem = True
			if not self.ShowHiddenFiles:
				if item[0] == '.':
					if item != '..':
						additem = False
			if additem:
				i = self.GetItemCount()
				self.InsertStringItem(i, item)
				self.SetItemImage(i, 0, 0)
				if self.IsDetailed:
					self.SetItemData(i, i)
					st = os.stat(directory + '/' + item)
					
					mtime = time.strftime('%m/%d/%Y %H:%M:%S' ,time.localtime(st.st_mtime))
					mmode = str(oct(stat.S_IMODE(st.st_mode)))
					
					self.DataArray.append((item, '', '<Dir>', mtime, mmode))
					
					self.SetStringItem(i, 1, '')
					self.SetStringItem(i, 2, '<Dir>')
					self.SetStringItem(i, 3, mtime)
					self.SetStringItem(i, 4, mmode)
		for item in thelist:
			additem = True
			if not self.ShowHiddenFiles:
				if item[0] == '.':
					additem = False
			if additem:
				if refilter.match(item):
					i = self.GetItemCount()
					self.InsertStringItem(i, item)
					if self.pywfilter.match(item):
						self.SetItemImage(i, 2, 2)
					elif self.pyfilter.match(item):
						self.SetItemImage(i, 1, 1)
					else:
						self.SetItemImage(i, 3, 3)
					if self.IsDetailed:
						self.SetItemData(i, i)
						st = os.stat(directory + '/' + item)	
						size = str(st.st_size)				
						mtime = time.strftime('%m/%d/%Y %H:%M:%S' ,time.localtime(st.st_mtime))
						mmode = str(oct(stat.S_IMODE(st.st_mode)))
						type = item.rfind('.')
						if type > -1:
							type = item[type+1:]
						else:
							type = ''
							
						self.DataArray.append((item, size, type, mtime, mmode))	
						
						self.SetStringItem(i, 1, str(size))
						self.SetStringItem(i, 2, type)
						self.SetStringItem(i, 3, mtime)
						self.SetStringItem(i, 4, mmode)

	def SetShowHiddenFiles(self, showhidden):
		self.ShowHiddenFiles = showhidden

	def SetViewDetailed(self, fstyle = 0):
		self.SetWindowStyleFlag(fstyle|wx.LC_REPORT)
		self.IsDetailed = True
		self.InsertColumn(0, 'Name')
		self.InsertColumn(1, 'Size')
		self.InsertColumn(2, 'Type')
		self.InsertColumn(3, 'Modified')
		self.InsertColumn(4, 'Permissions')
		
		self.SetColumnImage(0, 5)
		
		self.OnSize(None)		
		
	def SetViewList(self, fstyle = 0):
		self.SetWindowStyleFlag(fstyle|wx.LC_LIST)
		self.IsDetailed = False

	def SortData(self, item1, item2):
		i1 = self.DataArray[item1][self.currentcolumn]
		i2 = self.DataArray[item2][self.currentcolumn]
		i1isdir = self.IsDirectory(item1)
		i2isdir = self.IsDirectory(item2)
		if i1isdir and not i2isdir:
			if self.direction[self.currentcolumn]:
				return 1
			else:
				return -1
		elif not i1isdir and i2isdir:
			if self.direction[self.currentcolumn]:
				return -1
			else:
				return 1
		elif i1isdir and i2isdir:
			if self.currentcolumn == 1:
				i1 = self.DataArray[item1][0]
				i2 = self.DataArray[item2][0]
			if self.direction[self.currentcolumn]:
				if i1 < i2:
					return 1
				if i1 > i2:
					return -1
			else:
				if i1 < i2:
					return -1
				if i1 > i2:
					return 1
		if self.currentcolumn == 1:
			try:
				i1 = int(i1)
			except:
				i1 = 0
			try:
				i2 = int(i2)
			except:
				i1 = 0
		if self.direction[self.currentcolumn]:
			if i1 < i2:
				return -1
			if i1 > i2:
				return 1
		else:
			if i1 < i2:
				return 1
			if i1 > i2:
				return -1			
		return 0

class FileDialog(wx.Dialog):
	def __init__(self, parent, title, wildcard, point=(0, 0), size=wx.DefaultSize, IsASaveDialog=0, MultipleSelection=0):
		wx.Dialog.__init__(self, parent, -1, title, point, size, wx.DEFAULT_DIALOG_STYLE | wx.MAXIMIZE_BOX | wx.THICK_FRAME | wx.RESIZE_BORDER)
		
		self.parent = parent					
	
		self.IsASaveDialog = IsASaveDialog
		self.MultipleSelection = MultipleSelection
		if IsASaveDialog:
			self.MultipleSelection = False
		
		#Constants:
		
		self.ID_BOOKMARKS = 3001
		self.ID_DISPLAY_LIST = 3002
		self.ID_DISPLAY_DETAILED = 3003
		self.ID_BACK = 3004
		self.ID_FORWARD = 3005
		self.ID_UP = 3006
		self.ID_HOME = 3011
		self.ID_REFRESH = 3007
		self.ID_NEW_DIRECTORY = 3008
		self.ID_DELETE_FILE = 3009
		
		self.ID_SHOW_HIDDEN_FILES = 3011
		
		self.ID_LIST_FILES =  3015
		
		self.ID_DIR = 3020
		self.ID_FILE = 3030
		self.ID_EXTENSION = 3031
		self.ID_SHOW_READONLY = 3032
		
		self.ID_OK = 3040
		self.ID_CANCEL = 3041
		
		self.ID_BOOKMARK_MENU = 3100
		
		self.ID_BOOKMARK_BASE = 3150
		
		#/Constants
		
		#Components:

		self.cboFile = wx.ComboBox(self, self.ID_FILE, size=wx.Size(500, -1), choices=self.parent.recentfiles)
		
		imagesize = (24, 24)
		
		bDir = self.parent.programdirectory + '/bitmaps/24/'
		
		self.btnDisplayList = wx.BitmapButton(self, self.ID_DISPLAY_LIST, wx.BitmapFromImage(wx.Image(bDir+'list.png', wx.BITMAP_TYPE_PNG)))
		self.btnDisplayDetailed = wx.BitmapButton(self, self.ID_DISPLAY_DETAILED, wx.BitmapFromImage(wx.Image(bDir+'detailed.png', wx.BITMAP_TYPE_PNG)))
		self.btnBookmarks = wx.BitmapButton(self, self.ID_BOOKMARKS, wx.BitmapFromImage(wx.Image(bDir+'bookmarks.png', wx.BITMAP_TYPE_PNG)))
		self.btnBack = wx.BitmapButton(self, self.ID_BACK, wx.BitmapFromImage(wx.Image(bDir+'back.png', wx.BITMAP_TYPE_PNG)))
		self.btnForward = wx.BitmapButton(self, self.ID_FORWARD, wx.BitmapFromImage(wx.Image(bDir+'forward.png', wx.BITMAP_TYPE_PNG)))
		self.btnUp = wx.BitmapButton(self, self.ID_UP, wx.BitmapFromImage(wx.Image(bDir+'up.png', wx.BITMAP_TYPE_PNG)))
		self.btnHome = wx.BitmapButton(self, self.ID_HOME, wx.BitmapFromImage(wx.Image(bDir+'home.png', wx.BITMAP_TYPE_PNG)))
		self.btnRefresh = wx.BitmapButton(self, self.ID_REFRESH, wx.BitmapFromImage(wx.Image(bDir+'Reload File.png', wx.BITMAP_TYPE_PNG)))
		self.btnNewDirectory = wx.BitmapButton(self, self.ID_NEW_DIRECTORY, wx.BitmapFromImage(wx.Image(bDir+'new directory.png', wx.BITMAP_TYPE_PNG)))
		self.btnDeleteFile = wx.BitmapButton(self, self.ID_DELETE_FILE, wx.BitmapFromImage(wx.Image(bDir+'Delete.png', wx.BITMAP_TYPE_PNG)))
		
		self.btnDisplayList.SetToolTipString('List View')
		self.btnDisplayDetailed.SetToolTipString('Detailed View')
		self.btnBookmarks.SetToolTipString('Bookmarks')
		self.btnBack.SetToolTipString('Back')
		self.btnForward.SetToolTipString('Forward')
		self.btnUp.SetToolTipString('Up')
		self.btnHome.SetToolTipString('Home')
		self.btnRefresh.SetToolTipString('Refresh')
		self.btnNewDirectory.SetToolTipString('New Directory')
		self.btnDeleteFile.SetToolTipString('Delete')
		
		self.txtDirectory = wx.TextCtrl(self, self.ID_DIR, '', size=wx.Size(350, -1), style=wx.TE_READONLY)
		
		if MultipleSelection:
			self.lstFiles = FileListCtrl(self, self.ID_LIST_FILES, wx.Size(500, 230), wx.LC_LIST)
		else:
			self.lstFiles = FileListCtrl(self, self.ID_LIST_FILES, wx.Size(500, 230), wx.LC_LIST | wx.LC_SINGLE_SEL)
		
		self.btnOk = wx.Button(self, self.ID_OK, "  &Ok  ")
		
		self.btnOk.SetDefault()
		self.cboFile.SetFocus()
				
		if not self.parent.PLATFORM_IS_WIN:
			self.cboExtension = wx.ComboBox(self, self.ID_EXTENSION, size=wx.Size(250, -1))
			self.chkShowHiddenFiles = wx.CheckBox(self, self.ID_SHOW_HIDDEN_FILES, '  Show Hidden Files  ')
		else:
			self.cboExtension = wx.ComboBox(self, self.ID_EXTENSION, size=wx.Size(500, -1))
				
		self.btnCancel = wx.Button(self, self.ID_CANCEL, "  &Cancel  ")
		
		#/Components
		
		#Data
		
		swildcards = wildcard.split('|')
		self.wildcards = []
		x = 0
		l = len(swildcards)
		while x < l:
			self.cboExtension.Append(swildcards[x])
			self.wildcards.append(swildcards[x+1])
			x = x + 2		
		
		self.cboExtension.SetValue(swildcards[0])
		
		self.currentextension = 0
				
		self.lstFiles.SetDirectoryTo(self.parent.ddirectory, self.wildcards[self.currentextension])
		
		self.bookmarks = []
		
		#/Data
		
		#Sizer:
						
		self.theSizer = wx.BoxSizer(wx.VERTICAL)
		
		self.commandSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		self.commandSizer.Add(self.btnDisplayList, 0, wx.SHAPED)
		self.commandSizer.Add(self.btnDisplayDetailed, 0, wx.SHAPED)
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		self.commandSizer.Add(self.btnBack, 0, wx.SHAPED)
		self.commandSizer.Add(self.btnForward, 0, wx.SHAPED)
		self.commandSizer.Add(self.btnUp, 0, wx.SHAPED)
		self.commandSizer.Add(self.btnHome, 0, wx.SHAPED)
		self.commandSizer.Add(self.btnRefresh, 0, wx.SHAPED)
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		self.commandSizer.Add(self.btnBookmarks, 0, wx.SHAPED)
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 1, wx.EXPAND)
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)
		
		self.commandSizer.Add(self.btnNewDirectory, 0, wx.SHAPED)
		self.commandSizer.Add(self.btnDeleteFile, 0, wx.SHAPED)
		self.commandSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED)

		self.dirSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.dirSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.dirSizer.Add(self.txtDirectory, 9, wx.EXPAND)
		self.dirSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		
		self.listSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.listSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.listSizer.Add(self.lstFiles, 9, wx.EXPAND)
		self.listSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		
		self.OkSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.OkSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.OkSizer.Add(self.cboFile, 9, wx.EXPAND | wx.ALIGN_LEFT)
		self.OkSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.OkSizer.Add(self.btnOk, 0, wx.ALIGN_RIGHT)
		self.OkSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
				
		self.CancelSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.CancelSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.CancelSizer.Add(self.cboExtension, 2, wx.EXPAND)
		self.CancelSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		if not self.parent.PLATFORM_IS_WIN:		
			self.CancelSizer.Add(self.chkShowHiddenFiles, 1, wx.SHAPED | wx.ALIGN_RIGHT)
			self.CancelSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.SHAPED | wx.ALIGN_RIGHT)
		self.CancelSizer.Add(self.btnCancel, 0, wx.ALIGN_RIGHT)
		self.CancelSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
				
		self.theSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.theSizer.Add(self.commandSizer, 0, wx.EXPAND | wx.ALIGN_CENTER)
		self.theSizer.Add(self.dirSizer, 0, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.theSizer.Add(self.listSizer, 9, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.theSizer.Add(self.OkSizer, 0, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
		self.theSizer.Add(self.CancelSizer, 0, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '   '), 0, wx.EXPAND)
				
		self.SetAutoLayout(True)
		self.SetSizerAndFit(self.theSizer)
		
		#/Sizer
		
		#Events:

		self.Bind(wx.EVT_BUTTON, self.OnbtnUp, id=self.ID_UP)
		self.Bind(wx.EVT_BUTTON, self.OnbtnBack, id=self.ID_BACK)
		self.Bind(wx.EVT_BUTTON, self.OnbtnForward, id=self.ID_FORWARD)
		self.Bind(wx.EVT_BUTTON, self.OnbtnHome, id=self.ID_HOME)
		self.Bind(wx.EVT_BUTTON, self.OnbtnRefresh, id=self.ID_REFRESH)
		self.Bind(wx.EVT_BUTTON, self.OnbtnBookmarks, id=self.ID_BOOKMARKS)
		
		self.Bind(wx.EVT_BUTTON, self.OnbtnNewDirectory, id=self.ID_NEW_DIRECTORY)
		self.Bind(wx.EVT_BUTTON, self.OnbtnDelete, id=self.ID_DELETE_FILE)
		
		self.Bind(wx.EVT_BUTTON, self.OnbtnDisplayList, id=self.ID_DISPLAY_LIST)
		self.Bind(wx.EVT_BUTTON, self.OnbtnDisplayDetailed, id=self.ID_DISPLAY_DETAILED)
		
		self.Bind(wx.EVT_BUTTON, self.OnbtnCancel, id=self.ID_CANCEL)
		self.Bind(wx.EVT_BUTTON, self.OnbtnOk, id=self.ID_OK)
		
		self.Bind(wx.EVT_COMBOBOX, self.OnExtension, id=self.ID_EXTENSION)
		
		if not self.parent.PLATFORM_IS_WIN:
			self.Bind(wx.EVT_CHECKBOX, self.OnShowHiddenFiles, id=self.ID_SHOW_HIDDEN_FILES)
		
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnFileSelected, id=self.ID_LIST_FILES)
		self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnFileActivated, id=self.ID_LIST_FILES)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnbtnOk, id=self.ID_FILE)		
		self.Bind(wx.EVT_CHAR, self.OnChar)
		self.cboFile.Bind(wx.EVT_CHAR, self.OnChar)
		
		#/Events
				
		self.parent.LoadDialogSizeAndPosition(self, 'filedialog.sizeandposition.dat')

	def OnCloseW(self, event):
		self.parent.SaveDialogSizeAndPosition(self, 'filedialog.sizeandposition.dat')
		if event is not None:
			event.Skip()

				
	def EndAndCheck(self, value):
		if self.IsASaveDialog:
			filename = self.lstFiles.GetDirectory().replace('\\', '/') + '/' + os.path.basename(self.cboFile.GetValue())
			if os.path.exists(filename):
				msg = 'File "' + filename + '" already exists.  Overwrite?'
				d = wx.MessageDialog(self, msg, "OverWrite File?", wx.YES_NO | wx.ICON_QUESTION)
				answer = d.ShowModal()
				d.Destroy()
				if answer == wx.ID_NO:
					return
		self.OnCloseW(None)
		self.EndModal(value)
	
	def GetPath(self):
		return self.lstFiles.GetDirectory().replace('\\', '/') + '/' + os.path.basename(self.cboFile.GetValue())
	
	def GetPaths(self):
		i = self.lstFiles.GetFirstSelected()
		if i == -1:
			return [self.cboFile.GetValue()]
		
		dir = self.lstFiles.GetDirectory()
		filelist = [os.path.join(dir, self.lstFiles.GetItemText(i)).replace('\\', '/')]
		
		y = i
		y = self.lstFiles.GetNextSelected(y)
		while y != -1:
			filelist.append(os.path.join(dir, self.lstFiles.GetItemText(y)).replace('\\', '/'))
			y = self.lstFiles.GetNextSelected(y)
		currentfile = self.cboFile.GetValue().replace('\\', '/')
		if not (currentfile in filelist):
			filelist.insert(0, currentfile)
		return filelist
	
	def OnBookmark(self, event):
		bookmarkindex = event.GetId() - self.ID_BOOKMARK_BASE
		if not (os.path.exists(self.bookmarks[bookmarkindex])):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error with: " + self.bookmarks[bookmarkindex] + "\nBookmark does not actually exist.\n"), "Error")
			d.ShowModal()
			d.Destroy()
			return
		if os.path.isdir(self.bookmarks[bookmarkindex]):
			self.lstFiles.SetDirectoryTo(self.bookmarks[bookmarkindex], self.wildcards[self.currentextension])
			return
		self.cboFile.SetValue(self.bookmarks[bookmarkindex])
	
	def OnbtnBack(self, event):
		self.lstFiles.GoBack(self.wildcards[self.currentextension])	
	
	def OnbtnBookmarks(self, event):
		bookfile = self.parent.homedirectory + "/bookmarks.dat"
		bookmarksmenu = wx.Menu()
		self.bookmarks = []
		if (os.path.exists(bookfile)):
			try:
				#Read from the file
				f = open(bookfile, 'r')
				folders = [bookmarksmenu]
				folderindex = 0
				menuTitles = []
				menuTitleindex = -1
				lastCount = 1
				bookmarkcount = 0
				#Skip the First Line
				line = f.readline()
				#Initialize
				line = f.readline()
				while (len(line) > 0):
					c = line.count('\t')
					line = line[c:].rstrip()
					while (lastCount > c):
						folders[(folderindex - 1)].AppendMenu(self.ID_BOOKMARK_MENU, menuTitles.pop(), folders.pop())					
						folderindex = folderindex - 1
						menuTitleindex = menuTitleindex - 1
						lastCount = lastCount - 1
					if (line[0] == '>'):
						folders.append(wx.Menu())
						menuTitles.append(line[1:])
						folderindex = folderindex + 1
						menuTitleindex = menuTitleindex + 1
						c = c + 1
					else:
						self.bookmarks.append(line)
						self.Bind(wx.EVT_MENU, self.OnBookmark, id=(self.ID_BOOKMARK_BASE + bookmarkcount))
						folders[folderindex].Append((self.ID_BOOKMARK_BASE + bookmarkcount), line, line)
						bookmarkcount = bookmarkcount + 1
					lastCount = c
					line = f.readline()
				f.close()
				#Add any menus not yet added:
				c = 1
				while (lastCount > c):
					folders[(folderindex - 1)].AppendMenu(self.ID_BOOKMARK_MENU, menuTitles.pop(), folders.pop())					
					folderindex = folderindex - 1
					menuTitleindex = menuTitleindex - 1
					lastCount = lastCount - 1
				
				posX, posY = self.btnBookmarks.GetPosition()
				posY = posY + self.btnBookmarks.GetSize()[1]			
				self.PopupMenu(bookmarksmenu, (posX, posY))
				bookmarksmenu.Destroy()
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Your bookmarks file is a tad messed up.\n"), "Error")
				d.ShowModal()
				d.Destroy()
				
	def OnbtnCancel(self, event):
		self.SetReturnCode(wx.ID_CANCEL)
		self.EndModal(wx.ID_CANCEL)	
	
	def OnbtnDelete(self, event):
		filename = self.cboFile.GetValue()
		if os.path.exists(filename) and not os.path.isdir(filename):
			msg = 'Delete "' + filename + '"?'
			d = wx.MessageDialog(self, msg, "Delete File?", wx.YES_NO | wx.ICON_QUESTION)
			answer = d.ShowModal()
			d.Destroy()
			if answer == wx.ID_YES:
				try:
					os.remove(filename)
					self.OnbtnRefresh(None)
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self, 'Error Removing "' + filename + '".', "Error")
					d.ShowModal()
					d.Destroy()								
	
	def OnbtnDisplayDetailed(self, event):
		if self.MultipleSelection:
			self.lstFiles.SetViewDetailed()
		else:
			self.lstFiles.SetViewDetailed(wx.LC_SINGLE_SEL)
		self.lstFiles.SetDirectoryTo(self.lstFiles.GetDirectory(), self.wildcards[self.currentextension])		

	def OnbtnDisplayList(self, event):
		if self.MultipleSelection:			
			self.lstFiles.SetViewList()
		else:
			self.lstFiles.SetViewList(wx.LC_SINGLE_SEL)
		self.lstFiles.SetDirectoryTo(self.lstFiles.GetDirectory(), self.wildcards[self.currentextension])		
		
	def OnbtnForward(self, event):
		self.lstFiles.GoForward(self.wildcards[self.currentextension])
	
	def OnbtnHome(self, event):
		try:
			self.lstFiles.SetDirectoryTo(os.path.expanduser('~'), self.wildcards[self.currentextension])
		except:		
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, 'Error Setting Current Directory to Home', "DrPython Error")
			d.ShowModal()
			d.Destroy()
	
	def OnbtnNewDirectory(self, event):
		d = wx.TextEntryDialog(self, "Enter New Directory:", "New Directory", "")
		answer = d.ShowModal()
		v = d.GetValue()
		d.Destroy()
		if (answer == wx.ID_OK):
			if len(v) > 0:
				os.mkdir(self.lstFiles.GetDirectory() + '/' + v)
				self.OnbtnRefresh(None)
		
	def OnbtnOk(self, event):
		filename = self.cboFile.GetValue()
		if os.path.isdir(filename):
			self.lstFiles.SetDirectoryTo(filename, self.wildcards[self.currentextension])
		else:
			self.EndAndCheck(wx.ID_OK)
	
	def OnbtnRefresh(self, event):
		self.lstFiles.SetDirectoryTo(self.lstFiles.GetDirectory(), self.wildcards[self.currentextension])
	
	def OnbtnUp(self, event):
		cdir = os.path.split(self.lstFiles.GetDirectory())[0]
		self.lstFiles.SetDirectoryTo(cdir, self.wildcards[self.currentextension])
	
	def OnChar(self, event):
		if (event.GetKeyCode() == wx.WXK_ESCAPE):
			self.OnbtnCancel(None)
		elif (event.GetKeyCode() == wx.WXK_RETURN):
			self.OnbtnOk(None)
		else:
			event.Skip()
	
	def OnExtension(self, event):
		self.currentextension = self.cboExtension.GetSelection()
		self.lstFiles.SetDirectoryTo(self.lstFiles.GetDirectory(), self.wildcards[self.currentextension])
	
	def OnFileActivated(self, event):
		seltext = event.GetLabel()
		filename = self.lstFiles.GetDirectory() + '/' + seltext
		if os.path.isdir(filename):
			self.lstFiles.SetDirectoryTo(filename, self.wildcards[self.currentextension])
		else:
			self.EndAndCheck(wx.ID_OK)
		
	def OnFileSelected(self, event):
		seltext = event.GetLabel()
		filename = self.lstFiles.GetDirectory() + '/' + seltext
		if not os.path.isdir(filename):
			self.cboFile.SetValue(filename)
	
	def OnShowHiddenFiles(self, event):
		showhidden = self.chkShowHiddenFiles.GetValue()
		self.lstFiles.SetShowHiddenFiles(showhidden)
		self.OnbtnRefresh(None)
			
	def SetDirectory(self, directory):
		self.lstFiles.SetDirectoryTo(directory, self.wildcards[self.currentextension])
