#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#FindReplace Dialog

import wx
import wx.stc
import drScrolledMessageDialog
import re, os.path

class drFinder:
	#copy old finder limodou 2004/04/19
	def __init__(self, parent, oldfinder=None):
	#end limodou
		self.parent = parent
		self.reset()
		#copy old finder limodou 2004/04/19
		self.Copy(oldfinder)
		#end limodou

	#copy old finder limodou 2004/04/19
	def Copy(self, finder):
		if finder:
			self.findtext = finder.findtext
			self.findflags = finder.findflags
			self.backwards = finder.backwards
	#end limodou
		
	def DoFind(self, findtext, findflags, backwards=False):
		self.RE = 0
		doclength = self.parent.txtDocument.GetLength()
		
		self.findtext = findtext
		self.findflags = findflags
		self.backwards = backwards		
		
		prev = self.findpos
		
		if ((self.findpos >= doclength) or (self.findpos < 0)) and (doclength > 0):
			if backwards:
				self.findpos = self.targetEnd
			else:
				self.findpos = self.targetStart			
		
		if (backwards):
			endpos = self.targetStart
			if self.findpos == 0:
				#Go Forwards from 0:
				self.findpos = prev
				endpos = self.targetEnd
		else:
			endpos = self.targetEnd
					
		self.findpos = self.parent.txtDocument.FindText(self.findpos, endpos, findtext, findflags)
		
		if (self.findpos == -1):
			if ((self.findpos >= doclength) or (self.findpos < 0)) and (doclength > 0):
				self.lastposition = -1
				if self.backwards:
					msg = 'Start of document reached: "' + findtext + '".'	
					self.findpos = self.targetEnd
				else:
					msg = 'End of document reached: "' + findtext + '".'
					self.findpos = self.targetStart
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, msg, "DrPython Find")
				d.ShowModal()
				d.Destroy()
				return
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ('Search string "' + findtext + '" not found'), "DrPython Find")
			d.ShowModal()
			d.Destroy()
			return
		
		if self.parent.prefs.docfolding:
			self.parent.txtDocument.EnsureVisible(self.parent.txtDocument.LineFromPosition(self.findpos))
		self.parent.txtDocument.GotoPos(self.findpos)
		self.parent.txtDocument.EnsureCaretVisible()
		self.parent.txtDocument.SetSelectionStart(self.findpos)
		self.parent.txtDocument.SetSelectionEnd(self.findpos + len(findtext))
		self.lastposition = self.findpos + len(findtext)
		
		if (backwards):
			self.findpos = self.findpos - 1
		else:
			self.findpos = self.findpos + 1
				
	def DoREFind(self, findtext, matchcase):
		self.RE = 1
		doclength = self.parent.txtDocument.GetLength()	
		
		self.findtext = findtext
		self.findflags = matchcase
		
		prev = self.findpos
		
		case = 0
		if (not matchcase):
			case = re.IGNORECASE
		
		try:
			regularexpression = re.compile(findtext, case | re.MULTILINE)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, 'Regular Expression Error', "DrPython Find")
			d.ShowModal()
			d.Destroy()	
			return
			
				
		endpos = self.targetEnd
		if (self.findpos >= doclength) or (self.findpos < 0):
			self.findpos = 0
						
		matchedtext = regularexpression.search(self.parent.txtDocument.GetText()[self.findpos:endpos])
				
		if matchedtext is None:
			self.lastposition = -1
			if (prev > self.targetStart):
				self.findpos = self.targetStart
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, 'End of document reached: "' + findtext + '".', "DrPython Find")
				d.ShowModal()
				d.Destroy()	
				return
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ('Regular expression "' + findtext + '" not found'), "DrPython Find")
			d.ShowModal()
			d.Destroy()
			return

		oldpos = self.findpos
		self.findpos = oldpos + matchedtext.start()
		endpos = oldpos + matchedtext.end()
				
		if self.parent.prefs.docfolding:
			self.parent.txtDocument.EnsureVisible(self.parent.txtDocument.LineFromPosition(self.findpos))
		self.parent.txtDocument.GotoPos(self.findpos)
		self.parent.txtDocument.EnsureCaretVisible()
		self.parent.txtDocument.SetSelectionStart(self.findpos)
		self.parent.txtDocument.SetSelectionEnd(endpos)	
		self.lastposition = endpos	
		
		self.findpos = endpos
	
	def DoFindNext(self):
		if self.lastposition != -1:
			self.findpos = self.parent.txtDocument.GetCurrentPos()
		if (not self.RE):
			dosecondfind = self.backwards and self.lastposition == -1
			self.DoFind(self.findtext, self.findflags)
			if dosecondfind:
				self.DoFind(self.findtext, self.findflags)
			return
		self.DoREFind(self.findtext, self.findflags)
		
	def DoFindPrevious(self):
		if self.lastposition != -1:
			self.findpos = self.parent.txtDocument.GetCurrentPos()
		if (not self.RE):	
			moved = False
			if (self.findpos != self.lastposition):
				moved = True
			self.DoFind(self.findtext, self.findflags, True)
			if not moved:
				self.DoFind(self.findtext, self.findflags, True)
		
	def GetFindPos(self):
		return self.findpos
	
	def GetFindText(self):
		return self.findtext

	def ReplaceAll(self, findtext, replacetext, flags, prompt = 0):
		p = self.parent.txtDocument.FindText(self.targetStart, self.targetEnd, findtext, flags)
		diff = len(replacetext) - len(findtext)
		x = 0	
		notfirst = 0
		favpos = wx.Point(5, 5)
		while (p is not -1):
			if self.parent.prefs.docfolding:
				self.parent.txtDocument.EnsureVisible(self.parent.txtDocument.LineFromPosition(p))
			self.parent.txtDocument.GotoPos(p)
			self.parent.txtDocument.EnsureCaretVisible()
			self.parent.txtDocument.SetTargetStart(p)
			self.parent.txtDocument.SetTargetEnd(p + len(findtext))
			if prompt:
				self.parent.txtDocument.SetSelection(p, (p + len(findtext)))
				d = wx.SingleChoiceDialog(self.parent, ("Found \"" + findtext + "\" at Line: " \
				+ str(self.parent.txtDocument.LineFromPosition(p)+1) + \
				" Col: " + str(self.parent.txtDocument.GetColumn(p))
				 + "\n(Hit Cancel to Stop)"), "Replace", ["Replace", "Skip"], wx.CHOICEDLG_STYLE)
				if notfirst:
					d.Move(favpos)
				else:
					notfirst = 1
				answer = d.ShowModal()
				favpos = d.GetPosition()
				d.Destroy()
				if (answer == wx.ID_OK):					
					if (d.GetStringSelection() == "Replace"):
						self.parent.txtDocument.ReplaceTarget(replacetext)
						self.targetEnd = self.targetEnd + diff
					else:
						x = x - 1
					p = self.parent.txtDocument.FindText((p + len(replacetext)), self.targetEnd, findtext, flags)						
				else:
					p = -1
					x = x - 1
			else:
				self.parent.txtDocument.ReplaceTarget(replacetext)
				self.targetEnd = self.targetEnd + diff
				p = self.parent.txtDocument.FindText((p + len(replacetext)), self.targetEnd, findtext, flags)
			x = x + 1						
		return x
	
	def REReplaceAll(self, findtext, replacetext, matchcase, prompt = 0):
		case = 0
		if (not matchcase):
			case = re.IGNORECASE
		
		try:		
			regularexpression = re.compile(findtext, case | re.MULTILINE)		
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, 'Regular Expression Error', "DrPython Replace")
			d.ShowModal()
			d.Destroy()	
			return
		
		oldtext = self.parent.txtDocument.GetText()
		if not prompt:
			newtext, x = regularexpression.subn(replacetext, oldtext[self.targetStart:self.targetEnd])
		
			self.parent.txtDocument.SetText(oldtext[0:self.targetStart] + newtext + oldtext[self.targetEnd:])
		else:
			matchedtext = regularexpression.search(self.parent.txtDocument.GetText()[self.findpos:self.targetEnd])
			
			diff = len(replacetext) - len(findtext)
			
			x = 0
			notfirst = 0
			
			favpos = wx.Point(5, 5)
			while matchedtext is not None:		
			
				oldpos = self.findpos
				self.findpos = oldpos + matchedtext.start()
				endpos = oldpos + matchedtext.end()
				
				if self.parent.prefs.docfolding:
					self.parent.txtDocument.EnsureVisible(self.parent.txtDocument.LineFromPosition(self.findpos))
				self.parent.txtDocument.GotoPos(self.findpos)
				self.parent.txtDocument.EnsureCaretVisible()
				self.parent.txtDocument.SetSelectionStart(self.findpos)
				self.parent.txtDocument.SetSelectionEnd(endpos)
				self.parent.txtDocument.SetTargetStart(self.findpos)
				self.parent.txtDocument.SetTargetEnd(endpos)
				
				d = wx.SingleChoiceDialog(self.parent, ("Found \"" + findtext + "\" at Line: " \
				+ str(self.parent.txtDocument.LineFromPosition(self.findpos)+1) + \
				" Col: " + str(self.parent.txtDocument.GetColumn(self.findpos)) + \
				"\n(Hit Cancel to Stop)"), "Replace", ["Replace", "Skip"], wx.CHOICEDLG_STYLE)
				
				if notfirst:
					d.Move(favpos)
				else:
					notfirst = 1
				answer = d.ShowModal()
				favpos = d.GetPosition()
				d.Destroy()
				if (answer == wx.ID_OK):					
					if (d.GetStringSelection() == "Replace"):
						self.parent.txtDocument.ReplaceTarget(replacetext)
						self.findpos = self.findpos + len(replacetext)
						self.targetEnd = self.targetEnd + diff
					else:
						self.findpos = endpos
						x = x - 1
					x = x + 1
				
					matchedtext = regularexpression.search(self.parent.txtDocument.GetText()[self.findpos:self.targetEnd])
				else:
					matchedtext = None			
					
		return x
	
	def reset(self):
		self.findflags = 0
		self.findpos = 0
		self.lastposition = 0
		self.backwards = 0
		self.findtext = ""
		self.targetStart = 0
		self.targetEnd = 0
		self.RE = 0
		#copy old finder limodou 2004/04/19
		self.SetTargetRange(self.parent.txtDocument.GetCurrentPos(), self.parent.txtDocument.GetTextLength())
		#end limodou
			
	def SetFindPos(self, findpos):
		self.findpos = findpos	
	
	def SetFindText(self, findtext):
		self.findtext = findtext
		
	def SetTargetRange(self, start, end, backwards = 0):
		self.findpos = start
		if (backwards):
			self.findpos = end
		self.targetStart = start
		self.targetEnd = end

class drFindTextCtrl(wx.ComboBox):
	def __init__(self, parent, id, value, pos, size, returnfunction = None, InFiles = False):
		wx.ComboBox.__init__(self, parent, id, value, pos, size)		
		
		#This is a workaround for a bug in how wx.TextCtrl handles
		#carriage returns on windows.
		self.parent = parent
		if InFiles:
			self.ancestor = parent.parent.parent
		else:
			self.ancestor = parent.parent
		if self.ancestor.prefs.platform_is_windows:
			self.doReplace = True
			self.cr = chr(5)
		else:
			self.doReplace = False
			self.cr = '\r'
								
		self.ID_CHAR_BASE = 20
		
		self.ID_MENU_BASE = 50
		
		self.ID_HISTORY_BASE = 420
		
		self.returnfunc = returnfunction
		if self.returnfunc is None:
			self.returnfunc = self.parent.OnbtnFind
				
		self.Bind(wx.EVT_CHAR, self.OnChar)
				
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnPopUp)
		
		#wxPython Bug Work-Around
		if self.ancestor.prefs.platform_is_windows:
			self.insertionpoint = -1
			self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
			#franz07/19: catch kill focus
			self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
			#endfranz07/19
		
	def AppendToHistory(self, targetList):
		text = self.GetText()
		try:
			i = targetList.index(text)
			targetList.pop(i)
		except:
			pass
		
		targetList.append(text)
	
	def GetText(self):
		text = self.GetValue()
		if self.doReplace:
			return text.replace(chr(5), '\r')
		else:
			return text
		
	def OnChar(self, event):
		if (event.GetKeyCode() == wx.WXK_ESCAPE):
			self.parent.OnbtnCancel(event)
		elif (event.GetKeyCode() == wx.WXK_RETURN):
			self.returnfunc(event)
		else:
			event.Skip()

	def OnFocus(self, event):
		self.insertionpoint = -1
		event.Skip()

	#franz07/19: do not steal focus
	#drpython: removed "pass", replaced with function code below.
	def OnKillFocus(self, event):
		self.insertionpoint = self.GetInsertionPoint()
		event.Skip()
	
	#franz07/19
			
	def OnPopUp(self, event):
		self.PopUp(event.GetPosition())
		
	def OnPopUpMenu(self, event):
		eid = event.GetId()
				
		pos = self.GetInsertionPoint()
		#wxPython Bug Work-Around
		if self.ancestor.prefs.platform_is_windows:
			if self.insertionpoint > -1:
				pos = self.insertionpoint
		text = self.GetValue()
		
		if eid == self.ID_CHAR_BASE:
			self.SetValue(text[:pos] + '\t' + text[pos:])
			self.SetInsertionPoint(pos+1)
		elif eid == self.ID_CHAR_BASE+1:
			self.SetValue(text[:pos] + '\n' + text[pos:])
			self.SetInsertionPoint(pos+1)
		elif eid == self.ID_CHAR_BASE+2:
			self.SetValue(text[:pos] + self.cr + text[pos:])
			self.SetInsertionPoint(pos+1)	
		elif eid == self.ID_MENU_BASE+1:
			self.Cut()
		elif eid == self.ID_MENU_BASE+2:
			self.Copy()
		elif eid == self.ID_MENU_BASE+3:
			self.Paste()
		elif eid == self.ID_MENU_BASE+4:
			f, to = self.GetSelection()
			self.Remove(f, to)
		elif eid == self.ID_MENU_BASE+5:
			self.SetValue("")
	
	def PopUp(self, pos):		
		self.PopUpMenu = wx.Menu()
		
		self.CharMenu = wx.Menu()
		
		self.CharMenu.Append(self.ID_CHAR_BASE, "Tab (\\t)")
		self.CharMenu.Append(self.ID_CHAR_BASE+1, "Newline (\\n)")
		self.CharMenu.Append(self.ID_CHAR_BASE+2, "Carraige Return (\\r)")
		
		self.PopUpMenu.Append(self.ID_MENU_BASE+5, "Clear Text")
				
		self.PopUpMenu.AppendSeparator()
		
		self.PopUpMenu.AppendMenu(self.ID_MENU_BASE, "Insert Special Character", self.CharMenu)
		
		self.PopUpMenu.AppendSeparator()
		
		self.PopUpMenu.Append(self.ID_MENU_BASE+1, "Cut")
		self.PopUpMenu.Append(self.ID_MENU_BASE+2, "Copy")
		self.PopUpMenu.Append(self.ID_MENU_BASE+3, "Paste")
		self.PopUpMenu.Append(self.ID_MENU_BASE+4, "Delete")
		
		x = 0
		y = 0
		while x < 6:
			if y < 3:
				self.Bind(wx.EVT_MENU, self.OnPopUpMenu, id=self.ID_CHAR_BASE+y)
				y = y + 1
			self.Bind(wx.EVT_MENU, self.OnPopUpMenu, id=self.ID_MENU_BASE+x)			
			x = x + 1		
		
		self.PopupMenu(self.PopUpMenu, pos)
		
		self.PopUpMenu.Destroy()	
			
	def SetHistory(self, history):
		if self.GetCount() > 0:
			self.Clear()
		l = len(history)
		self.Append('')
		x = l - 1
		while x > -1:
			self.Append(history[x])
			x = x - 1
		if l > 0:
			self.SetMark(0, len(history[l-1]))
		
class drFindReplaceDialog(wx.Dialog):
	def __init__(self, parent, id, title, IsReplace = 0):
		wx.Dialog.__init__(self, parent, id, title, wx.Point(50, 50), wx.Size(-1, -1), wx.DEFAULT_DIALOG_STYLE | wx.MAXIMIZE_BOX | wx.THICK_FRAME | wx.RESIZE_BORDER)		
		
		self.ID_FIND = 1001
		self.ID_CANCEL = 1002
		
		self.ID_CHK_REGEX = 1010
		self.ID_CREATERE = 1011
		
		self.ID_BTNSF = 98
		self.ID_BTNRW = 99
		
		self.parent = parent
				
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)

		self.IsReplace = IsReplace

		#Size Buffer
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)

		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)						
		self.theSizer.Add(wx.StaticText(self, -1, "Search For: "), 1, wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
				
		self.txtSearchFor = drFindTextCtrl(self, -1, "", wx.DefaultPosition, wx.Size(200, -1))	
		self.btnPopUpSearchFor = wx.Button(self, self.ID_BTNSF, " Menu ")	
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtSearchFor, 1 ,wx.SHAPED)
		self.theSizer.Add(self.btnPopUpSearchFor, 1, wx.SHAPED)
		
		self.txtSearchFor.SetHistory(parent.FindHistory)
				
		if (IsReplace):
			#self.SetSize(wx.Size(400, 345))
			self.txtReplaceWith = drFindTextCtrl(self, -1, "", wx.DefaultPosition, wx.Size(200, -1))
			self.btnPopUpReplaceWith = wx.Button(self, self.ID_BTNRW, " Menu ")	
			self.txtReplaceWith.SetHistory(parent.ReplaceHistory)
		
		self.chkRegularExpression = wx.CheckBox(self, self.ID_CHK_REGEX, "RegularExpression")
		self.btnCreateRE = wx.Button(self, self.ID_CREATERE, " &Create ")
		self.chkMatchCase = wx.CheckBox(self, -1, "Match Case")
		self.chkFindBackwards = wx.CheckBox(self, -1, "Find Backwards")
		self.chkWholeWord = wx.CheckBox(self, -1, "Whole Word")
		self.chkInSelection = wx.CheckBox(self, -1, "In Selection")
		self.chkFromCursor = wx.CheckBox(self, -1, "From Cursor")
		
		#Prefs
		self.chkRegularExpression.SetValue(parent.prefs.findreplaceregularexpression)
		self.btnCreateRE.Enable(parent.prefs.findreplaceregularexpression)
		self.chkMatchCase.SetValue(parent.prefs.findreplacematchcase)
		self.chkFindBackwards.SetValue(parent.prefs.findreplacefindbackwards)
		self.chkWholeWord.SetValue(parent.prefs.findreplacewholeword)
		self.chkInSelection.SetValue(parent.prefs.findreplaceinselection)
		self.chkFromCursor.SetValue(parent.prefs.findreplacefromcursor)
		
		self.chkInSelection.Enable(len(parent.txtDocument.GetSelectedText()) > 0)
		
		if (IsReplace):
			self.chkPromptOnReplace  = wx.CheckBox(self, -1, "Prompt on Replace")
			self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)						
			self.theSizer.Add(wx.StaticText(self, -1, "Replace With: "), 1, wx.SHAPED)		
			self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)			
			
			self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
			self.theSizer.Add(self.txtReplaceWith, 1 ,wx.SHAPED)
			self.theSizer.Add(self.btnPopUpReplaceWith, 1, wx.SHAPED)
						
			self.chkFindBackwards.Disable()
						
			#Prefs
			self.chkPromptOnReplace.SetValue(parent.prefs.findreplacepromptonreplace)
			
			self.Bind(wx.EVT_BUTTON,  self.OnbtnPopUp, id=self.ID_BTNRW)
		
		#Size Buffer
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
				
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(self.chkRegularExpression, 1, wx.SHAPED)
		self.theSizer.Add(self.btnCreateRE, 1, wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(self.chkMatchCase, 1, wx.SHAPED)
		self.theSizer.Add(self.chkFindBackwards, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(self.chkWholeWord, 1, wx.SHAPED)
		self.theSizer.Add(self.chkInSelection, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(self.chkFromCursor, 1, wx.SHAPED)
		if (IsReplace):
			self.theSizer.Add(self.chkPromptOnReplace, 1, wx.SHAPED)
		else:	
			self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)		
		
		self.btnCancel = wx.Button(self, self.ID_CANCEL, "&Cancel")
		self.btnFind = wx.Button(self, self.ID_FIND, "&Ok")
		
		self.theSizer.Add(wx.StaticText(self, -1, "    "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnCancel, 1, wx.SHAPED)
		self.theSizer.Add(self.btnFind, 1, wx.SHAPED)
				
		self.SetAutoLayout(True)
		self.SetSizerAndFit(self.theSizer)				
				
		self.btnFind.SetDefault()
		self.txtSearchFor.SetFocus()
				
		self.Bind(wx.EVT_BUTTON,  self.OnbtnCancel, id=self.ID_CANCEL)	
		self.Bind(wx.EVT_BUTTON,  self.OnbtnFind, id=self.ID_FIND)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnCreateRE, id=self.ID_CREATERE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnPopUp, id=self.ID_BTNSF)
	
		self.Bind(wx.EVT_CHECKBOX,  self.OnCheckRegularExpression, id=self.ID_CHK_REGEX)

		self.parent.LoadDialogSizeAndPosition(self, 'findreplacedialog.sizeandposition.dat')

	def OnCloseW(self, event):
		self.parent.SaveDialogSizeAndPosition(self, 'findreplacedialog.sizeandposition.dat')
		if event is not None:
			event.Skip()
				
	def GetOptions(self):
		rarray = [self.chkRegularExpression.GetValue(), self.chkMatchCase.GetValue(), \
		self.chkFindBackwards.GetValue(), self.chkWholeWord.GetValue(), \
		self.chkInSelection.GetValue(), self.chkFromCursor.GetValue()]
				
		if (self.IsReplace):
			rarray.append(self.chkPromptOnReplace.GetValue())			
			
		return rarray
	
	def OnbtnCancel(self, event):
		self.Close(1)
	
	def OnbtnCreateRE(self, event):
		from drRegularExpressionDialog import drRegularExpressionDialog
		d = drRegularExpressionDialog(self, -1, "Create Regular Expression")
		d.Show()
						
	def OnbtnFind(self, event):
		self.Show(0)
		findflags = 0
		findbackwards = self.chkFindBackwards.GetValue()
		
		self.txtSearchFor.AppendToHistory(self.parent.FindHistory)
		if self.IsReplace:
			self.txtReplaceWith.AppendToHistory(self.parent.ReplaceHistory)
				
		#Set Target Range
		if (self.chkInSelection.GetValue()):
			selstart = self.parent.txtDocument.GetSelectionStart()
			selend = self.parent.txtDocument.GetSelectionEnd()
			if (selend - selstart) < 1:
				selstart = 0
				selend = self.parent.txtDocument.GetTextLength()
			self.parent.Finder.SetTargetRange(selstart, selend, findbackwards)
		elif (self.chkFromCursor.GetValue()):
			if (findbackwards):
				self.parent.Finder.SetTargetRange(0, self.parent.txtDocument.GetCurrentPos(), findbackwards)
			else:
				self.parent.Finder.SetTargetRange(self.parent.txtDocument.GetCurrentPos(), self.parent.txtDocument.GetTextLength(), findbackwards)
		else:
			self.parent.Finder.SetTargetRange(0, self.parent.txtDocument.GetTextLength(), findbackwards)
		
		#Do Search
		if (self.chkRegularExpression.GetValue()):
			if (self.IsReplace):
				x = self.parent.Finder.REReplaceAll(self.txtSearchFor.GetText(), self.txtReplaceWith.GetText(), self.chkMatchCase.GetValue(), self.chkPromptOnReplace.GetValue())
				if self.parent.prefs.enablefeedback:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self, (str(x) + " occurances of \"" + self.txtSearchFor.GetText() + "\" replaced with \"" + self.txtReplaceWith.GetText() + "\""), "Replace")
					d.ShowModal()
					d.Destroy()
			else:		
				self.parent.Finder.DoREFind(self.txtSearchFor.GetText(), self.chkMatchCase.GetValue())
		else:
			#Set Flags
			if (self.chkWholeWord.GetValue()):
				findflags = findflags | wx.stc.STC_FIND_WHOLEWORD
			if (self.chkMatchCase.GetValue()):
				findflags = findflags | wx.stc.STC_FIND_MATCHCASE
				
			if (self.IsReplace):
				x = self.parent.Finder.ReplaceAll(self.txtSearchFor.GetText(), self.txtReplaceWith.GetText(), findflags, self.chkPromptOnReplace.GetValue())
				if self.parent.prefs.enablefeedback:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self, (str(x) + " occurances of \"" + self.txtSearchFor.GetText() + "\" replaced with \"" + self.txtReplaceWith.GetText() + "\""), "Replace")
					d.ShowModal()
					d.Destroy()
			else:
				self.parent.Finder.DoFind(self.txtSearchFor.GetText(), findflags, findbackwards)
		
		self.Close(1)
		
		if (self.IsReplace):
			self.parent.ReplaceOptions = self.GetOptions()
		else:
			self.parent.FindOptions = self.GetOptions()
			
	def OnbtnPopUp(self, event):
		eid = event.GetId()
		if eid == self.ID_BTNSF:
			s = self.txtSearchFor.GetPosition()[0]
			x = self.btnPopUpSearchFor.GetPosition()[0]
			self.txtSearchFor.PopUp((x-s, 0))
		elif eid == self.ID_BTNRW:			
			s = self.txtReplaceWith.GetPosition()[0]
			x = self.btnPopUpReplaceWith.GetPosition()[0]
			self.txtReplaceWith.PopUp((x-s, 0))
	
	def OnCheckRegularExpression(self, event):
		usingRegularExpressions = self.chkRegularExpression.GetValue()
		self.btnCreateRE.Enable(usingRegularExpressions)
		self.chkWholeWord.Enable(not usingRegularExpressions)
		if not self.IsReplace:
			self.chkFindBackwards.Enable(not usingRegularExpressions)
	
	def SetFindString(self, findstring):
		self.txtSearchFor.SetValue(findstring)
		self.txtSearchFor.SetMark(0, len(findstring))		

	def SetOptions(self, OptionsArray):
		if len(OptionsArray) > 0:
			self.chkRegularExpression.SetValue(OptionsArray[0])
			self.btnCreateRE.Enable(OptionsArray[0])
			self.chkMatchCase.SetValue(OptionsArray[1])
			self.chkFindBackwards.SetValue(OptionsArray[2])
			self.chkWholeWord.SetValue(OptionsArray[3])
			self.chkInSelection.SetValue(OptionsArray[4])
			self.chkFromCursor.SetValue(OptionsArray[5])
					
			if (self.IsReplace):
				self.chkPromptOnReplace.SetValue(OptionsArray[6])