#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#Plugins Menu

import os, os.path, shutil, sys
import wx
import drScrolledMessageDialog

class drPluginMenu(wx.Menu):
	def __init__(self, parent):
		wx.Menu.__init__(self)
				
		self.ID_LOAD_PLUGIN_BASE = 4505		
		
		self.parent = parent
		self.homedirectory = parent.homedirectory

		self.indexes = []
		self.loadedindexes = []

		self.setupMenu()
	
	def OnLoadPluginsFromIndex(self, event):
		idnum = event.GetId()
		i = idnum - self.ID_LOAD_PLUGIN_BASE
		index = self.indexes[i]
		try:
			f = file(index, 'r')
			pluginstoload = f.read().rstrip().split('\n')
			f.close()
			for plugin in pluginstoload:
				try:				
					self.parent.InitializePlugin(plugin)				
				except:
					errstring = str(sys.exc_info()[0]).lstrip("exceptions.") + ": " + str(sys.exc_info()[1])

					d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error loading plugin: " + plugin + "\n\n" + errstring), "Load Error")
					d.ShowModal()
					d.Destroy()
			self.loadedindexes.append(self.indexes[i])		
			self.reloadMenu()
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error loading plugins from: " + index), "Load Error")
			d.ShowModal()
			d.Destroy()
			
	def setupMenu(self):	
		files = os.listdir(self.homedirectory + "/plugins")
		x = 0
		self.indexes = []
		for f in files:
			if f.find(".idx") > -1:
				index = self.homedirectory + "/plugins/" + f
				if not (index in self.loadedindexes):
					self.indexes.append(index)
					self.Append(self.ID_LOAD_PLUGIN_BASE + x, str(os.path.basename(f)))
					self.parent.Bind(wx.EVT_MENU, self.OnLoadPluginsFromIndex, id=self.ID_LOAD_PLUGIN_BASE + x)
					x = x + 1
				
	def reloadMenu(self):
		mnuitems = self.GetMenuItems()
		num = len(mnuitems)
		x = 0
		while (x < num):
			self.Remove(mnuitems[x].GetId())
			mnuitems[x].Destroy()
			x = x + 1
		self.setupMenu()

class drPluginAboutMenu(wx.Menu):
	def __init__(self, parent):
		wx.Menu.__init__(self)
				
		self.ID_LOAD_PLUGIN_ABOUT_BASE = 4305
		
		self.parent = parent
		self.homedirectory = parent.homedirectory

		self.pluginsArray = []
	
	def OnLoadPluginsAbout(self, event):
		idnum = event.GetId()
		i = idnum - self.ID_LOAD_PLUGIN_ABOUT_BASE
		plugin = self.pluginsArray[i]		
		try:
			exec(compile("import " + plugin, plugin, 'exec'))
			try:
				exec(compile(plugin + ".OnAbout", plugin, 'exec'))
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("No info for this plugin."), "Plugin Help Error")
				d.ShowModal()
				d.Destroy()
				return
			exec(compile(plugin + ".OnAbout(self.parent)", plugin, 'exec'))
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error With Help."), "Plugin Help Error")
			d.ShowModal()
			d.Destroy()
	
	def AddItem(self, plugin):
		try:
			exec(compile("import " + plugin, plugin, 'exec'))
			exec(compile(plugin + ".OnAbout", plugin, 'exec'))
			x = len(self.pluginsArray)
			self.pluginsArray.append(plugin)
			self.Append(self.ID_LOAD_PLUGIN_ABOUT_BASE + x, plugin)
			self.parent.Bind(wx.EVT_MENU, self.OnLoadPluginsAbout, id=self.ID_LOAD_PLUGIN_ABOUT_BASE + x)
		except:
			pass
		
class drPluginHelpMenu(wx.Menu):
	def __init__(self, parent):
		wx.Menu.__init__(self)
				
		self.ID_LOAD_PLUGIN_HELP_BASE = 4705		
		
		self.parent = parent
		self.homedirectory = parent.homedirectory

		self.pluginsArray = []
	
	def OnLoadPluginsHelp(self, event):
		idnum = event.GetId()
		i = idnum - self.ID_LOAD_PLUGIN_HELP_BASE
		plugin = self.pluginsArray[i]		
		try:
			exec(compile("import " + plugin, plugin, 'exec'))
			try:
				exec(compile(plugin + ".OnHelp", plugin, 'exec'))
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("No documentation for this plugin."), "Plugin Help Error")
				d.ShowModal()
				d.Destroy()
				return
			exec(compile(plugin + ".OnHelp(self.parent)", plugin, 'exec'))
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error With Help."), "Plugin Help Error")
			d.ShowModal()
			d.Destroy()
	
	def AddItem(self, plugin):
		try:
			exec(compile("import " + plugin, plugin, 'exec'))
			exec(compile(plugin + ".OnHelp", plugin, 'exec'))
			x = len(self.pluginsArray)
			self.pluginsArray.append(plugin)
			self.Append(self.ID_LOAD_PLUGIN_HELP_BASE + x, plugin)
			self.parent.Bind(wx.EVT_MENU, self.OnLoadPluginsHelp, id=self.ID_LOAD_PLUGIN_HELP_BASE + x)
		except:
			pass

		
class drPluginPreferencesMenu(wx.Menu):
	def __init__(self, parent):
		wx.Menu.__init__(self)
				
		self.ID_LOAD_PLUGIN_PREFS_BASE = 4905		
		
		self.parent = parent
		self.homedirectory = parent.homedirectory

		self.pluginsArray = []
	
	def OnLoadPluginsPreferences(self, event):
		idnum = event.GetId()
		i = idnum - self.ID_LOAD_PLUGIN_PREFS_BASE
		plugin = self.pluginsArray[i]	
		try:
			exec(compile("import " + plugin, plugin, 'exec'))
			try:
				exec(compile(plugin + ".OnPreferences", plugin, 'exec'))
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("No documentation for this plugin."), "Plugin Help Error")
				d.ShowModal()
				d.Destroy()
				return
			exec(compile(plugin + ".OnPreferences(self.parent)", plugin, 'exec'))
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error With Help."), "Plugin Help Error")
			d.ShowModal()
			d.Destroy()
			
	def AddItem(self, plugin):
		try:
			exec(compile("import " + plugin, plugin, 'exec'))
			exec(compile(plugin + ".OnPreferences", plugin, 'exec'))
			x = len(self.pluginsArray)
			self.pluginsArray.append(plugin)
			self.Append(self.ID_LOAD_PLUGIN_PREFS_BASE + x, plugin)
			self.parent.Bind(wx.EVT_MENU, self.OnLoadPluginsPreferences, id=self.ID_LOAD_PLUGIN_PREFS_BASE + x)
		except:
			pass