#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import os, os.path
import wx
import drScrolledMessageDialog
from drProperty import *
import drPrefsFile
import drFileDialog
from drStyleDialog import drStyleDialog, drSimpleStyleDialog
from drPreferences import drPreferences

class BookmarksPanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.btnStyle = wx.Button(self, self.grandparent.ID_BOOKMARKS_STYLE, " Edit Bookmarks Style ")
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnStyle, 1, wx.SHAPED)		

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
					
		self.Bind(wx.EVT_BUTTON,  self.OnbtnStyle, id=self.grandparent.ID_BOOKMARKS_STYLE)
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
					
	def OnbtnStyle(self, event):		
		d = drSimpleStyleDialog(self, -1, ("Style: Bookmarks"), self.grandparent.prefs.bookmarksstyle, self.grandparent.prefs.bookmarksstyle)

		d.ShowModal()	
		if (d.ClickedOk()):
			self.grandparent.prefs.bookmarksstyle = d.GetStyleString()
		d.Destroy()

class DocumentPanel(wx.ScrolledWindow):

	def __init__(self, parent, id):
		wx.ScrolledWindow.__init__(self, parent, id)

		self.grandparent = parent.GetParent()			
				
		self.EnableScrolling(True, True)
		
		self.SetScrollbars(10, 10, 30, 30)
		
		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.txtmarginwidth = wx.TextCtrl(self, self.grandparent.ID_DOC_MARGIN_WIDTH, str(self.grandparent.prefs.docmarginwidth), wx.Point(225, 215), wx.Size(35, -1))
					
		self.raddocautoindent = wx.RadioBox(self, -1, "AutoIndent:", wx.DefaultPosition, wx.DefaultSize, ["None", "Normal", "Context Sensitive"], 1, wx.RA_SPECIFY_COLS)
		
		self.chkdocupdateindentation = wx.CheckBox(self, -1, "")
			
		self.chkdocusetabs = wx.CheckBox(self, -1, "")
		
		self.chkdocwordwrap = wx.CheckBox(self, -1, "")

		self.chkWhitespace = wx.CheckBox(self, -1, "")
		
		self.chkfolding = wx.CheckBox(self, -1, "")	
		
		self.txtcaretwidth = wx.TextCtrl(self, self.grandparent.ID_DOC_CARET_WIDTH, str(self.grandparent.prefs.doccaretwidth), wx.Point(225, 215), wx.Size(35, -1))
		
		self.chkhighlightcurrentline = wx.CheckBox(self, -1, "")
		
		self.chkuseindentationguides = wx.CheckBox(self, -1, "")
		
		#Chris McDonough (I added txt prefix, update for local constants)
		self.txtdoclonglinecol = wx.TextCtrl(self, self.grandparent.ID_LONGLINE_COL, str(self.grandparent.prefs.doclonglinecol), wx.Point(225, 215), wx.Size(35, -1))
		#/Chris McDonough
		
		self.chkscrollextrapage = wx.CheckBox(self, -1, "")
		
		self.radusestyles = wx.RadioBox(self, -1, "Styles:", wx.DefaultPosition, wx.DefaultSize, ["Don't Use Styles", "Use Styles", "Only Use \"Normal\", \"Caret Foreground\""], 1, wx.RA_SPECIFY_COLS)	
		
		self.boxdefaultsyntaxhighlighting = wx.Choice(self, -1, wx.Point(225, 135), wx.Size(-1, -1), ["Python", "C/C++", "HTML"])
		
		self.chkonlyusedefaultsyntaxhighlighting = wx.CheckBox(self, -1, "")
	
		self.btnStyle = wx.Button(self, self.grandparent.ID_FILE_STYLE, " Edit Text Styles ")
		self.btnApplyProperty = wx.Button(self, self.grandparent.ID_APPLY_PROPERTY, " Apply Text Property To All Styles ")
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Line Number Margin Width:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtmarginwidth,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.raddocautoindent,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Update Indentation:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkdocupdateindentation,1,wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Use Tabs:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkdocusetabs,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Word Wrap:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkdocwordwrap,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Whitespace is visible on startup:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkWhitespace ,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Folding:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkfolding ,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Use Indentation guides:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkuseindentationguides ,1,wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		
		#Chris McDonough (I added txt prefix, update for local constants)
		self.theSizer.Add(wx.StaticText(self, -1, "Long Line Indicator Column:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtdoclonglinecol,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		#/Chris McDonough
		
		self.theSizer.Add(wx.StaticText(self, -1, "Scroll Extra Page:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkscrollextrapage,1,wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "Caret Width:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtcaretwidth,1,wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "Highlight Current Line:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkhighlightcurrentline,1,wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		
		self.theSizer.Add(self.radusestyles,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Default Syntax Highlighting:"), 1, wx.SHAPED)
		self.theSizer.Add(self.boxdefaultsyntaxhighlighting,1,wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Only Use Default Syntax Highlighting"), 1, wx.SHAPED)
		self.theSizer.Add(self.chkonlyusedefaultsyntaxhighlighting,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnStyle,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)		
		self.theSizer.Add(self.btnApplyProperty,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		
		#Buffer
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)
		
		self.reset()	
				
		self.Bind(wx.EVT_TEXT, self.Ontxtmarginwidth, id=self.grandparent.ID_DOC_MARGIN_WIDTH)
		self.Bind(wx.EVT_TEXT, self.Ontxtcaretwidth, id=self.grandparent.ID_DOC_CARET_WIDTH)
		self.Bind(wx.EVT_TEXT, self.Ontxtdoclonglinecol, id=self.grandparent.ID_LONGLINE_COL)
		self.Bind(wx.EVT_BUTTON, self.OnbtnStyle, id=self.grandparent.ID_FILE_STYLE)
		self.Bind(wx.EVT_BUTTON, self.OnbtnApplyProperty, id=self.grandparent.ID_APPLY_PROPERTY)
		self.Bind(wx.EVT_BUTTON, self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)

	def reset(self):
			
		self.txtmarginwidth.SetValue(str(self.grandparent.prefs.docmarginwidth))
		
		self.raddocautoindent.SetSelection(self.grandparent.prefs.docautoindent)
	
		self.chkdocupdateindentation.SetValue(self.grandparent.prefs.docupdateindentation)
	
		self.chkdocusetabs.SetValue(self.grandparent.prefs.docusetabs)
		
		self.chkdocwordwrap.SetValue(self.grandparent.prefs.docwordwrap)

		self.chkWhitespace.SetValue(self.grandparent.prefs.docwhitespaceisvisible)	
			
		self.chkfolding.SetValue(self.grandparent.prefs.docfolding)	
	
		self.chkuseindentationguides.SetValue(self.grandparent.prefs.docuseindentationguides)
	
		self.chkscrollextrapage.SetValue(self.grandparent.prefs.docscrollextrapage)

		self.txtcaretwidth.SetValue(str(self.grandparent.prefs.doccaretwidth))

		self.chkhighlightcurrentline.SetValue(self.grandparent.prefs.dochighlightcurrentline)
	
		self.boxdefaultsyntaxhighlighting.SetSelection(self.grandparent.prefs.docdefaultsyntaxhighlighting)
	
		self.chkonlyusedefaultsyntaxhighlighting.SetValue(self.grandparent.prefs.doconlyusedefaultsyntaxhighlighting)
	
		self.radusestyles.SetSelection(self.grandparent.prefs.docusestyles)		
	
	def Ontxtdoclonglinecol(self, event):
		#Chris McDonough (Edited to allow for negative numbers)
		x = self.txtdoclonglinecol.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			if not x == "-":
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Actual Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
				d.ShowModal()
				d.Destroy()
				self.txtdoclonglinecol.SetValue(str(self.grandparent.prefs.doclonglinecol))
				return
			y = 0			
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txtdoclonglinecol.SetValue(str(self.grandparent.prefs.doclonglinecol))
			return
		if (y > 1024):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()	
		
	def Ontxtcaretwidth(self, event):
		x = self.txtcaretwidth.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txtcaretwidth.SetValue(str(self.grandparent.prefs.doccaretwidth))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txtcaretwidth.SetValue(str(self.grandparent.prefs.doccaretwidth))
			return
		if (y > 256):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()
	
	def Ontxtmarginwidth(self, event):
		x = self.txtmarginwidth.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txtmarginwidth.SetValue(str(self.grandparent.prefs.docmarginwidth))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txtmarginwidth.SetValue(str(self.grandparent.prefs.docmarginwidth))
			return
		if (y > 1024):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()			
		
	def OnbtnStyle(self, event):
		d = drStyleDialog(self, -1, ("Edit Styles"))
		d.ShowModal()
		if d.ClickedOk():
			self.grandparent.prefs.PythonStyleArray, self.grandparent.prefs.CPPStyleArray, self.grandparent.prefs.HTMLStyleArray = d.GetArrays()
		d.Destroy()
		
	def OnbtnApplyProperty(self, event):
		d = wx.MessageDialog(self, "This will set a property (either font, size, foreground or background) for all text styles\n(except for line number, caret foreground and selection)\nto the property for normal text.\nAre you sure you want to do this?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
		d = wx.SingleChoiceDialog(self, "Select the property you wish to apply to all styles:", "Apply Property To All Styles", ["Font", "Size", "Background", "Foreground"], wx.OK|wx.CANCEL)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_OK):		
			#ignore caret foreground, selection, folding style: (last three items)
			l = len(self.targetArray) - 3
			#Ignore default text for all, and line number text for bg/fg (first two items)
			x = 2
			s = d.GetStringSelection()
			if (s == "Font"):
				prop = "face"
				dummy = ",face:terminal"
				x = 1
			elif (s == "Size"):
				prop = "size"
				dummy = ",size:10"
				x = 1
			elif (s == "Background"):
				prop = "back"
				dummy = ",back:#FFFFFF"
			elif (s == "Foreground"):
				prop = "fore"
				dummy = ",fore:#000000"		
			bstring = getStyleProperty(prop, self.targetArray[0])
			while (x < l):
				tstring = self.targetArray[x]
				self.targetArray.pop(x)
				try:
					tstring.index(prop)
				except:
					tstring = tstring + dummy
				self.targetArray.insert(x, setStyleProperty(prop, tstring, bstring))
				x = x + 1

class DocumentationPanel(wx.ScrolledWindow):

	def __init__(self, parent, id):

		wx.ScrolledWindow.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
					
		self.EnableScrolling(True, True)
		
		self.SetScrollbars(10, 10, 30, 30)

		self.btnBrowse = wx.Button(self, self.grandparent.ID_DOCUMENTATION_BROWSE, " &Browse ")

		self.btnBrowseP = wx.Button(self, self.grandparent.ID_DOCUMENTATION_BROWSE_P, " &Browse ")
		self.btnBrowseW = wx.Button(self, self.grandparent.ID_DOCUMENTATION_BROWSE_W, " &Browse ")
		self.btnBrowseR = wx.Button(self, self.grandparent.ID_DOCUMENTATION_BROWSE_R, " &Browse ")		

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.txtbrowser = wx.TextCtrl(self, -1, "", wx.Point(225, 215), wx.Size(250, -1))
		
		self.txtpython = wx.TextCtrl(self, -1, "", wx.Point(225, 215), wx.Size(250, -1))
		
		self.txtwxwidgets = wx.TextCtrl(self, -1, "", wx.Point(225, 215), wx.Size(250, -1))
		
		self.txtrehowto = wx.TextCtrl(self, -1, "", wx.Point(225, 215), wx.Size(250, -1))
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Browser:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtbrowser, 1, wx.SHAPED)
		self.theSizer.Add(self.btnBrowse, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Python Docs:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtpython, 1, wx.SHAPED)	
		self.theSizer.Add(self.btnBrowseP, 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "WxWidgets Docs:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtwxwidgets, 1, wx.SHAPED)
		self.theSizer.Add(self.btnBrowseW, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Regular Expression Howto:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtrehowto, 1, wx.SHAPED)
		self.theSizer.Add(self.btnBrowseR, 1, wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()		
		
		if self.grandparent.prefs.platform_is_windows:
			self.wildcard = "Program File (*.exe)|*.exe|All files (*)|*"
		else:
			self.wildcard = "All files (*)|*"			
				
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowse, id=self.grandparent.ID_DOCUMENTATION_BROWSE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowseDoc, id=self.grandparent.ID_DOCUMENTATION_BROWSE_P)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowseDoc, id=self.grandparent.ID_DOCUMENTATION_BROWSE_W)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowseDoc, id=self.grandparent.ID_DOCUMENTATION_BROWSE_R)		
	
	def reset(self):
		self.txtbrowser.SetValue(self.grandparent.prefs.documentationbrowser)		
		self.txtpython.SetValue(self.grandparent.prefs.documentationpythonlocation)		
		self.txtwxwidgets.SetValue(self.grandparent.prefs.documentationwxwidgetslocation)
		self.txtrehowto.SetValue(self.grandparent.prefs.documentationrehowtolocation)
		
	def OnbtnBrowse(self, event):
		dlg = drFileDialog.FileDialog(self.grandparent.parent, "Select A Browser", self.wildcard)
		if (dlg.ShowModal() == wx.ID_OK):
			self.txtbrowser.SetValue(dlg.GetPath())
		dlg.Destroy()
		
	def OnbtnBrowseDoc(self, event):
		eid = event.GetId()

		result = ""

		dlg = drFileDialog.FileDialog(self.grandparent.parent, "Location of Documentation", self.wildcard)
		if (dlg.ShowModal() == wx.ID_OK):
			result = dlg.GetPath()
		dlg.Destroy()
			
		if len(result) > 0:	
			if eid == self.grandparent.ID_DOCUMENTATION_BROWSE_P:
				self.txtpython.SetValue(result)		
			elif eid == self.grandparent.ID_DOCUMENTATION_BROWSE_W:
				self.txtwxwidgets.SetValue(result)				
			elif eid == self.grandparent.ID_DOCUMENTATION_BROWSE_R:
				self.txtrehowto.SetValue(result)		


class DrScriptPanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
		
		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.txtdefaultdirectory = wx.TextCtrl(self, -1, self.grandparent.prefs.drscriptdefaultdirectory,  wx.Point(15, 325), wx.Size(250, -1))	
		
		self.chkloadexamples = wx.CheckBox(self, -1, "")
		
		self.btnBrowse = wx.Button(self, self.grandparent.ID_DRSCRIPT_BROWSE, " &Browse ")
		
		self.btnStyle = wx.Button(self, self.grandparent.ID_DRSCRIPT_STYLE, " Edit DrScript Style ")
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1, wx.SHAPED)
		
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnStyle, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
				
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Default Directory:"), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtdefaultdirectory,1, wx.SHAPED)
		self.theSizer.Add(self.btnBrowse,1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Load Example Scripts:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkloadexamples, 1, wx.SHAPED)
		
		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
		
		self.reset()
					
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowse, id=self.grandparent.ID_DRSCRIPT_BROWSE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnStyle, id=self.grandparent.ID_DRSCRIPT_STYLE)
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
		
	def reset(self):
		self.txtdefaultdirectory.SetValue(self.grandparent.prefs.drscriptdefaultdirectory)	
		self.chkloadexamples.SetValue(self.grandparent.prefs.drscriptloadexamples)

	def OnbtnBrowse(self, event):
		d = wx.DirDialog(self, "Select Default Directory For DrScript:", style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON|wx.MAXIMIZE_BOX|wx.THICK_FRAME)
		if d.ShowModal() == wx.ID_OK:
			self.txtdefaultdirectory.SetValue(d.GetPath())
		d.Destroy()
		
	def OnbtnStyle(self, event):		
		d = drSimpleStyleDialog(self, -1, ("Style: DrScript"), self.grandparent.prefs.drscriptstyle, self.grandparent.prefs.drscriptstyle)

		d.ShowModal()	
		if (d.ClickedOk()):
			self.grandparent.prefs.drscriptstyle = d.GetStyleString()
		d.Destroy()

class FindReplacePanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
				
		self.chkregularexpression = wx.CheckBox(self, -1, "")
		self.chkmatchcase = wx.CheckBox(self, -1, "")
		self.chkfindbackwards = wx.CheckBox(self, -1, "")
		self.chkwholeword = wx.CheckBox(self, -1, "")
		self.chkinselection = wx.CheckBox(self, -1, "")
		self.chkfromcursor = wx.CheckBox(self, -1, "")
		self.chkpromptonreplace = wx.CheckBox(self, -1, "")
		self.chkundercursor = wx.CheckBox(self, -1, "")
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Regular Expression:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkregularexpression, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Match Case:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkmatchcase, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Find Backwards:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkfindbackwards, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Whole Word:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkwholeword, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "In Selection:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkinselection, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "From Cursor:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkfromcursor, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Prompt On Replace:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkpromptonreplace, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Under Cursor:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkundercursor, 1, wx.SHAPED)		

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()
		
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
			
	def reset(self):

		self.chkregularexpression.SetValue(self.grandparent.prefs.findreplaceregularexpression)
		self.chkmatchcase.SetValue(self.grandparent.prefs.findreplacematchcase)
		self.chkfindbackwards.SetValue(self.grandparent.prefs.findreplacefindbackwards)
		self.chkwholeword.SetValue(self.grandparent.prefs.findreplacewholeword)
		self.chkinselection.SetValue(self.grandparent.prefs.findreplaceinselection)
		self.chkfromcursor.SetValue(self.grandparent.prefs.findreplacefromcursor)
		self.chkpromptonreplace.SetValue(self.grandparent.prefs.findreplacepromptonreplace)
		self.chkundercursor.SetValue(self.grandparent.prefs.findreplaceundercursor)

class GeneralPanel(wx.ScrolledWindow):

	def __init__(self, parent, id):

		wx.ScrolledWindow.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
				
		self.EnableScrolling(True, True)
		
		self.SetScrollbars(10, 10, 30, 30)
				
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)		
		
		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")

		self.chkrememberwindowsize = wx.CheckBox(self, -1, "")
		
		self.chkrememberdialogsizesandpositions = wx.CheckBox(self, -1, "")
		
		self.boxiconsize = wx.Choice(self, self.grandparent.ID_ICON_SIZE, wx.Point(225, 135), wx.Size(-1, -1), ["0", "16", "24"])
		
		self.txticonfile = wx.TextCtrl(self, -1, self.grandparent.prefs.iconfile,  wx.Point(15, 325), wx.Size(250, -1))	
		
		self.btnBrowseIcon = wx.Button(self, self.grandparent.ID_ICON_FILE_BROWSE, " &Browse ")
		
		self.txtRecentFiles = wx.TextCtrl(self, self.grandparent.ID_TXT_RECENT_FILES, str(self.grandparent.prefs.recentfileslimit), wx.Point(225, 175), wx.Size(35, -1))

		self.txttabwidth = wx.TextCtrl(self, self.grandparent.ID_TXT_TAB_WIDTH, str(self.grandparent.prefs.tabwidth), wx.Point(225, 215), wx.Size(35, -1))

		self.txtpythonargs = wx.TextCtrl(self, -1, self.grandparent.prefs.pythonargs, wx.Point(225, 255), wx.Size(55, -1))

		self.boxencoding = wx.Choice(self, -1, wx.Point(225, 135), wx.Size(-1, -1), ["Default", "ASCII", "UTF-8", "Custom"])
		
		self.txtcustomencoding = wx.TextCtrl(self, -1, self.grandparent.prefs.customencoding, wx.Point(225, 255), wx.Size(55, -1))

		self.txtdefaultdirectory = wx.TextCtrl(self, -1, self.grandparent.prefs.defaultdirectory,  wx.Point(15, 325), wx.Size(250, -1))	

		self.btnBrowse = wx.Button(self, self.grandparent.ID_FILE_BROWSE, " &Browse ")
		
		self.chkenablefeedback = wx.CheckBox(self, -1, "")
				
		self.chkalwayspromptonexit = wx.CheckBox(self, -1, "")
		
		self.chkbackupfileonsave = wx.CheckBox(self, -1, "")
				
		#utf-8 auto detect limodou 2004/04/14
		self.chkautodetectutf8encoding = wx.CheckBox(self, -1, "")
		#end limodou
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Remember Window Size:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkrememberwindowsize,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Remember Dialog Sizes and Positions:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkrememberdialogsizesandpositions,1,wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Max number of recent files:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtRecentFiles,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Tab width:"),1,wx.SHAPED)
		self.theSizer.Add(self.txttabwidth,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Python arguments (eg \"-i\"):"),1,wx.SHAPED)
		self.theSizer.Add(self.txtpythonargs,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Encoding:"),1,wx.SHAPED)
		self.theSizer.Add(self.boxencoding,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Custom Encoding:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtcustomencoding,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		
		#Moved by DrPython
		#utf-8 auto detect support limodou 2004/04/14
		self.theSizer.Add(wx.StaticText(self, -1, "Auto Detect UTF-8 Encoding:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkautodetectutf8encoding,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		#end limodou
		
		self.theSizer.Add(wx.StaticText(self, -1, "Icon size:"),1,wx.SHAPED)
		self.theSizer.Add(self.boxiconsize,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Icon File:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txticonfile,1,wx.SHAPED)
		self.theSizer.Add(self.btnBrowseIcon,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Default Directory:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtdefaultdirectory,1,wx.SHAPED)
		self.theSizer.Add(self.btnBrowse,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Enable Feedback Messages:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkenablefeedback,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Always Prompt On Exit:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkalwayspromptonexit,1,wx.SHAPED)			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Backup File On Save:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkbackupfileonsave,1,wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)

		self.reset()

		self.Bind(wx.EVT_TEXT,  self.OntxtRecentFiles, id=self.grandparent.ID_TXT_RECENT_FILES)
		self.Bind(wx.EVT_TEXT,  self.Ontxttabwidth, id=self.grandparent.ID_TXT_TAB_WIDTH)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowseIcon, id=self.grandparent.ID_ICON_FILE_BROWSE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowse, id=self.grandparent.ID_FILE_BROWSE)
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
		
	def reset(self):
		self.chkrememberwindowsize.SetValue(self.grandparent.prefs.rememberwindowsize)
		
		self.chkrememberdialogsizesandpositions.SetValue(self.grandparent.prefs.rememberdialogsizesandpositions)
		
		self.txtRecentFiles.SetValue(str(self.grandparent.prefs.recentfileslimit))
			
		if (self.grandparent.prefs.iconsize == 24):
			self.boxiconsize.SetSelection(2)
		elif (self.grandparent.prefs.iconsize == 16):
			self.boxiconsize.SetSelection(1)
		else:
			self.boxiconsize.SetSelection(0)
		
		self.boxencoding.SetSelection(self.grandparent.prefs.encoding)
		
		self.txtcustomencoding.SetValue(self.grandparent.prefs.customencoding)
		
		self.txticonfile.SetValue(self.grandparent.prefs.iconfile)
			
		self.txttabwidth.SetValue(str(self.grandparent.prefs.tabwidth))

		self.txtpythonargs.SetValue(self.grandparent.prefs.pythonargs)

		self.txtdefaultdirectory.SetValue(self.grandparent.prefs.defaultdirectory)		
					
		self.chkenablefeedback.SetValue(self.grandparent.prefs.enablefeedback)
				
		self.chkalwayspromptonexit.SetValue(self.grandparent.prefs.alwayspromptonexit)
		
		self.chkbackupfileonsave.SetValue(self.grandparent.prefs.backupfileonsave)
				
		#utf-8 auto detect limodou 2004/04/14
		self.chkautodetectutf8encoding.SetValue(self.grandparent.prefs.autodetectutf8encoding)
		#end limodou
		
	def OnbtnBrowseIcon(self, event):
		dlg = wx.FileDialog(self.grandparent.parent, "Select Icon File", "", "", "DrPython Icon Set (*.dis)|*.dis|All files (*)|*", wx.OPEN|wx.HIDE_READONLY)
		if (dlg.ShowModal() == wx.ID_OK):
			self.txticonfile.SetValue(dlg.GetPath())
		dlg.Destroy()
		
	def OnbtnBrowse(self, event):
		d = wx.DirDialog(self, "Select Default Directory:", style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON|wx.MAXIMIZE_BOX|wx.THICK_FRAME)
		if d.ShowModal() == wx.ID_OK:
			self.txtdefaultdirectory.SetValue(d.GetPath())
		d.Destroy()	
	
	def OntxtRecentFiles(self, event):
		x = self.txtRecentFiles.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txtRecentFiles.SetValue(str(self.grandparent.prefs.recentfileslimit))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()	
			self.txtRecentFiles.SetValue(str(self.grandparent.prefs.recentfileslimit))
			return
		if (y > 25):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()		
	
	def Ontxttabwidth(self, event):
		x = self.txttabwidth.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txttabwidth.SetValue(str(self.grandparent.prefs.tabwidth))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txttabwidth.SetValue(str(self.grandparent.prefs.tabwidth))
			return
		if (y > 1024):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()	

class LineEndingsPanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.chkCheckFormat = wx.CheckBox(self, -1, "")
		
		self.radFormat = wx.Choice(self, -1, wx.Point(15, 80), wx.DefaultSize, [" Unix ('\\n')  "," DOS/Windows ('\\r\\n')  "," Mac ('\\r')  "])		
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Check Line Ending Format on Open:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkCheckFormat, 1, wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Line Ending Format:"),1,wx.SHAPED)
		self.theSizer.Add(self.radFormat, 1, wx.SHAPED)		

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()
		
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
			
	def reset(self):

		self.chkCheckFormat.SetValue(self.grandparent.prefs.checkeol)
		
		try:
			self.radFormat.SetSelection(self.grandparent.prefs.eolmode)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("FOOL!  What have you done?\n  The eolmode is messed up.\nDon't know why either.\nBest E-Mail the Doctor.\n(And be verbose when you do!)"), "Foolish Error")
			d.ShowModal()
			d.Destroy()

class PluginsPanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
		
		self.txtdefaultdirectory = wx.TextCtrl(self, -1, self.grandparent.prefs.pluginsdefaultdirectory,  wx.Point(15, 325), wx.Size(250, -1))	

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.btnBrowse = wx.Button(self, self.grandparent.ID_DRSCRIPT_BROWSE, " &Browse ")
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Default Directory:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.txtdefaultdirectory,1,wx.SHAPED)
		self.theSizer.Add(self.btnBrowse,1,wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
		
		self.reset()
					
		self.Bind(wx.EVT_BUTTON,  self.OnbtnBrowse, id=self.grandparent.ID_DRSCRIPT_BROWSE)
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
		
	def reset(self):
		self.txtdefaultdirectory.SetValue(self.grandparent.prefs.pluginsdefaultdirectory)	
							
	def OnbtnBrowse(self, event):
		d = wx.DirDialog(self, "Select Default Directory For Themes:", style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON|wx.MAXIMIZE_BOX|wx.THICK_FRAME)
		if d.ShowModal() == wx.ID_OK:
			self.txtdefaultdirectory.SetValue(d.GetPath())
		d.Destroy()

class PrintPanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.chkdoclinenumbers = wx.CheckBox(self, -1, "")
		self.chkpromptlinenumbers = wx.CheckBox(self, -1, "")

		self.txttabwidth = wx.TextCtrl(self, self.grandparent.ID_PRINT_TAB_WIDTH, str(self.grandparent.prefs.printtabwidth), wx.Point(225, 215), wx.Size(35, -1))
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Document Line Numbers:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkdoclinenumbers, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Prompt Line Numbers:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkpromptlinenumbers, 1, wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Tab Width:"),1,wx.SHAPED)
		self.theSizer.Add(self.txttabwidth, 1, wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()
		
		self.Bind(wx.EVT_TEXT,  self.Ontxttabwidth, id=self.grandparent.ID_PRINT_TAB_WIDTH)	
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
		
	def reset(self):
		self.chkdoclinenumbers.SetValue(self.grandparent.prefs.printdoclinenumbers)
		self.chkpromptlinenumbers.SetValue(self.grandparent.prefs.printpromptlinenumbers)
		self.txttabwidth.SetValue(str(self.grandparent.prefs.printtabwidth))		
	
	def Ontxttabwidth(self, event):
		x = self.txttabwidth.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txttabwidth.SetValue(str(self.grandparent.prefs.printtabwidth))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txttabwidth.SetValue(str(self.grandparent.prefs.printtabwidth))
			return
		if (y > 32):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()

class PromptPanel(wx.ScrolledWindow):

	def __init__(self, parent, id):

		wx.ScrolledWindow.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
		
		self.EnableScrolling(True, True)
		
		self.SetScrollbars(18, 10, 30, 30)		

		self.textlist = ["Normal", "Line Number (Margin)", "Brace-Selected (Match)", "Brace-Selected (No Match)", "Character (Single Quoted String)", "Class Name", "Comment", "Function Name", "Keyword", "Number", "Operator", "String", "Triple Quoted String", "Caret Foreground", "Selection"]

		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.txtmarginwidth = wx.TextCtrl(self, self.grandparent.ID_PROMPT_MARGIN_WIDTH, str(self.grandparent.prefs.promptmarginwidth), wx.Point(225, 215), wx.Size(35, -1))

		self.chkVisible = wx.CheckBox(self, -1, "")
		
		self.chkpromptusetabs = wx.CheckBox(self, -1, "")

		self.chkwordwrap = wx.CheckBox(self, -1, "")

		self.txtcaretwidth = wx.TextCtrl(self, self.grandparent.ID_PROMPT_CARET_WIDTH, str(self.grandparent.prefs.promptcaretwidth), wx.Point(225, 215), wx.Size(35, -1))

		self.chkWhitespace = wx.CheckBox(self, -1, "")
		
		self.chkscrollextrapage = wx.CheckBox(self, -1, "")
			
		self.sldrSize =  wx.Slider(self, -1, self.grandparent.prefs.promptsize, 25, 100, wx.Point(150, 55), wx.Size(75, -1), wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS )
		self.sldrSize.SetTickFreq(25, 1)

		self.radusestyles = wx.RadioBox(self, -1, "Styles:", wx.DefaultPosition, wx.DefaultSize, ["Don't Use Styles", "Use Styles", "Only Use \"Normal\", \"Caret Foreground\""], 1, wx.RA_SPECIFY_COLS)	

		self.boxTextType = wx.Choice(self, self.grandparent.ID_PROMPT_TEXT_TYPE, wx.Point(15, 40), wx.Size(-1, -1), self.textlist)
		self.boxTextType.SetSelection(0)

		self.btnStyle = wx.Button(self, self.grandparent.ID_PROMPT_STYLE, " Edit Text Styles ")
		self.btnApplyProperty = wx.Button(self, self.grandparent.ID_PROMPT_APPLY_PROPERTY, " Apply Text Property To All Styles ")

		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Line Number Margin Width:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtmarginwidth,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Visible By Default:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkVisible, 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Use Tabs:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkpromptusetabs,1,wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Word Wrap:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkwordwrap,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Caret Width:"),1,wx.SHAPED)
		self.theSizer.Add(self.txtcaretwidth,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Whitespace is visible on startup:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkWhitespace ,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Scroll Extra Page:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkscrollextrapage ,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Vertical size:"),1,wx.SHAPED)
		self.theSizer.Add(self.sldrSize, 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.radusestyles ,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Select text type to edit:"),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.boxTextType,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnStyle,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnApplyProperty,1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)

		#Buffer
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()

		self.Bind(wx.EVT_TEXT,  self.Ontxtmarginwidth, id=self.grandparent.ID_PROMPT_MARGIN_WIDTH)		
		self.Bind(wx.EVT_TEXT, self.Ontxtcaretwidth, id=self.grandparent.ID_PROMPT_CARET_WIDTH)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnStyle, id=self.grandparent.ID_PROMPT_STYLE)			
		self.Bind(wx.EVT_BUTTON,  self.OnbtnApplyProperty, id=self.grandparent.ID_PROMPT_APPLY_PROPERTY)
		self.Bind(wx.EVT_BUTTON,  self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
	
	def reset(self):
			
		self.txtmarginwidth.SetValue(str(self.grandparent.prefs.promptmarginwidth))
		
		self.chkVisible.SetValue(self.grandparent.prefs.promptisvisible)

		self.chkpromptusetabs.SetValue(self.grandparent.prefs.promptusetabs)

		self.chkwordwrap.SetValue(self.grandparent.prefs.promptwordwrap)

		self.txtcaretwidth.SetValue(str(self.grandparent.prefs.promptcaretwidth))

		self.chkWhitespace.SetValue(self.grandparent.prefs.promptwhitespaceisvisible)	
		
		self.sldrSize.SetValue(self.grandparent.prefs.promptsize)	
		
		self.chkscrollextrapage.SetValue(self.grandparent.prefs.promptscrollextrapage)
	
		self.radusestyles.SetSelection(self.grandparent.prefs.promptusestyles)
	
	def Ontxtcaretwidth(self, event):
		x = self.txtcaretwidth.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txtcaretwidth.SetValue(str(self.grandparent.prefs.promptcaretwidth))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txtcaretwidth.SetValue(str(self.grandparent.prefs.promptcaretwidth))
			return
		if (y > 256):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()
	
	def Ontxtmarginwidth(self, event):
		x = self.txtmarginwidth.GetValue()
		if (len(x) < 1):
			return
		try:
			y = int(x)
		except ValueError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Whatever you are trying to do... Stop it.\nIt won't work.  Positive Numbers only please.\nDrPython will reset to the value last loaded."), "Value Error")
			d.ShowModal()
			d.Destroy()
			self.txtmarginwidth.SetValue(str(self.grandparent.prefs.promptmarginwidth))
			return
		except TypeError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("\"" + y + "\" is not an integer (a number, eg 2, 4, 568)\nDrPython will reset to the value last loaded."), "Type Error")
			d.ShowModal()
			d.Destroy()
			self.txtmarginwidth.SetValue(str(self.grandparent.prefs.promptmarginwidth))
			return
		if (y > 1024):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("DrPython does not recommend going this high.  You can of course..."), "Hmmmm")
			d.ShowModal()
			d.Destroy()
		
	def OnbtnStyle(self, event):
		d = drStyleDialog(self, -1, ("Edit Prompt Styles"), True)
		d.ShowModal()
		if d.ClickedOk():
			self.grandparent.prefs.txtPromptStyleArray = d.GetArrays()
		d.Destroy()
	
	def OnbtnApplyProperty(self, event):
		d = wx.MessageDialog(self, "This will set a property (either font, size, foreground or background) for all text styles\n(except for line number, caret foreground and selection)\nto the property for normal text.\nAre you sure you want to do this?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
		d = wx.SingleChoiceDialog(self, "Select the property you wish to apply to all styles:", "Apply Property To All Styles", ["Font", "Size", "Background", "Foreground"], wx.OK|wx.CANCEL)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_OK):
			s = d.GetStringSelection()
			if (s == "Font"):
				prop = "face"
				dummy = ",face:terminal"
			elif (s == "Size"):
				prop = "size"
				dummy = ",size:10"
			elif (s == "Background"):
				prop = "back"
				dummy = ",back:#000000"
			elif (s == "Foreground"):
				prop = "fore"
				dummy = ",fore:#FFFFFF"
			l = len(self.grandparent.prefs.txtPromptStyleArray) - 3		
			#ignore caret foreground, selections
			x = 2
			#Ignore default text, and line number text (0, and 1)
			bstring = getStyleProperty(prop, self.grandparent.prefs.txtPromptStyleArray[0])
			while (x < l):
				tstring = self.grandparent.prefs.txtPromptStyleArray[x]
				self.grandparent.prefs.txtPromptStyleArray.pop(x)
				try:
					tstring.index(prop)
				except:
					tstring = tstring + dummy
				self.grandparent.prefs.txtPromptStyleArray.insert(x, setStyleProperty(prop, tstring, bstring))
				x = x + 1			

class SidePanePanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
		
		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
		
		self.sldrSizeLeft =  wx.Slider(self, -1, self.grandparent.prefs.sidepaneleftsize, 15, 100, wx.Point(150, 55), wx.Size(75, -1), wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS )
		self.sldrSizeLeft.SetTickFreq(25, 1)
		
		self.sldrSizeRight =  wx.Slider(self, -1, self.grandparent.prefs.sidepanerightsize, 15, 100, wx.Point(150, 55), wx.Size(75, -1), wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS )
		self.sldrSizeRight.SetTickFreq(25, 1)

		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Left Pane Size:"),1,wx.SHAPED)
		self.theSizer.Add(self.sldrSizeLeft, 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Right Pane Size:"),1,wx.SHAPED)
		self.theSizer.Add(self.sldrSizeRight, 1, wx.SHAPED)	
		
		#Buffer
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()
		
		self.Bind(wx.EVT_BUTTON, self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
	
	def reset(self):
			
		self.sldrSizeLeft.SetValue(self.grandparent.prefs.sidepaneleftsize)
		
		self.sldrSizeRight.SetValue(self.grandparent.prefs.sidepanerightsize)	

class SourceBrowserPanel(wx.Panel):

	def __init__(self, parent, id):

		wx.Panel.__init__(self, parent, id)

		self.grandparent = parent.GetParent()
		
		self.btnResetPanel = wx.Button(self, self.grandparent.ID_RESET_PANEL, " Reset Panel ")
				
		self.radposition = wx.RadioBox(self, -1, "Position:", wx.DefaultPosition, wx.DefaultSize, ["Left", "Right"], 1, wx.RA_SPECIFY_COLS)
	
		self.chkIsVisible = wx.CheckBox(self, -1, "")
	
		self.chkcloseonactivate = wx.CheckBox(self, -1, "")

		self.chkissorted = wx.CheckBox(self, -1, "")

		self.btnStyle = wx.Button(self, self.grandparent.ID_CLASS_BROWSER_STYLE, " Edit Source Browser Style ")
		
		self.theSizer = wx.FlexGridSizer(8, 3, 5, 10)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
			
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Reset Just These Preferences:"), 1, wx.SHAPED)	
		self.theSizer.Add(self.btnResetPanel,1,wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.radposition, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""),1,wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Visible By Default:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkIsVisible, 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Close On Activate:"),1,wx.SHAPED)
		self.theSizer.Add(self.chkcloseonactivate, 1, wx.SHAPED)	
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Sorted (Alphabetical):"),1,wx.SHAPED)
		self.theSizer.Add(self.chkissorted, 1, wx.SHAPED)		
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 1, wx.SHAPED)
		self.theSizer.Add(self.btnStyle, 1, wx.SHAPED)		

		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)	
			
		self.reset()	
					
		self.Bind(wx.EVT_BUTTON, self.OnbtnStyle, id=self.grandparent.ID_CLASS_BROWSER_STYLE)
		self.Bind(wx.EVT_BUTTON, self.grandparent.OnbtnResetPanel, id=self.grandparent.ID_RESET_PANEL)
		
	def reset(self):
		self.radposition.SetSelection(self.grandparent.prefs.sourcebrowserpane)
		self.chkIsVisible.SetValue(self.grandparent.prefs.sourcebrowserisvisible)
		self.chkcloseonactivate.SetValue(self.grandparent.prefs.sourcebrowsercloseonactivate)
		self.chkissorted.SetValue(self.grandparent.prefs.sourcebrowserissorted)
			
	def OnbtnStyle(self, event):		
		d = drSimpleStyleDialog(self, -1, ("Style: Source Browser"), self.grandparent.prefs.sourcebrowserstyle, self.grandparent.prefs.sourcebrowserstyle)

		d.ShowModal()	
		if (d.ClickedOk()):
			self.grandparent.prefs.sourcebrowserstyle = d.GetStyleString()
			
		d.Destroy()

class drPrefsDialog(wx.Dialog):

	def __init__(self, parent, id, title):

		wx.Dialog.__init__(self, parent, id, title, wx.Point(50, 50), wx.Size(600, 400), wx.DEFAULT_DIALOG_STYLE | wx.MAXIMIZE_BOX | wx.THICK_FRAME | wx.RESIZE_BORDER)
		
		wx.Yield()
		
		self.parent = parent
		
		self.ID_PAGES = 100
		
		self.ID_CLOSE = 101
		
		self.ID_RESET = 102
		
		self.ID_UPDATE = 103
		
		self.ID_SAVE = 104
				
		self.ID_FILE_BROWSE = 109
		
		self.ID_ICON_FILE_BROWSE = 1090
		
		self.ID_TEXT_TYPE = 110
		
		self.ID_FILE_STYLE = 111
		
		self.ID_APPLY_PROPERTY = 112
		
		self.ID_DOC_MARGIN_WIDTH = 113
		
		self.ID_DOC_CARET_WIDTH = 1235
		
		self.ID_PROMPT_CARET_WIDTH = 1237
		
		self.ID_LONGLINE_COL = 116
		
		self.ID_ICON_SIZE = 350
				
		self.ID_PROMPT_STYLE = 1140
		self.ID_PROMPT_MARGIN_WIDTH = 115
		self.ID_PROMPT_TEXT_TYPE = 117
		self.ID_PROMPT_APPLY_PROPERTY = 118
		
		self.ID_CLASS_BROWSER_STYLE = 201
		
		self.ID_BOOKMARKS_STYLE = 202
		
		self.ID_DRSCRIPT_STYLE = 203		
		self.ID_DRSCRIPT_BROWSE = 211
						
		self.ID_PRINT_TAB_WIDTH = 251
		
		self.ID_DOCUMENTATION_BROWSE = 231		
				
		self.ID_DOCUMENTATION_BROWSE_P = 232
		self.ID_DOCUMENTATION_BROWSE_W = 233
		self.ID_DOCUMENTATION_BROWSE_R = 234
		
		
		self.ID_TXT_RECENT_FILES = 301
		self.ID_TXT_TAB_WIDTH = 302
		self.ID_USE_TABS = 303
		
		self.ID_RESET_PANEL = 405
		
		self.ID_LB_PREFS = 505

		self.homedirectory = parent.homedirectory
				
		self.prefs = drPreferences(parent.prefs.platform_is_windows, parent.programdirectory)
		self.prefs.Copy(parent.prefs)
		self.oldprefs = drPreferences(parent.prefs.platform_is_windows, parent.programdirectory)
		self.oldprefs.Copy(parent.prefs)

		self.lbPrefs = wx.Listbook(self, self.ID_LB_PREFS, wx.Point(0, 0), wx.Size(450, 255))
	
		self.pnlBookmarks = BookmarksPanel(self.lbPrefs, -1)
		self.pnlBookmarks.SetName("Bookmarks")
		
		self.pnlDocument = DocumentPanel(self.lbPrefs, -1)
		self.pnlDocument.SetName("Document")
				
		self.pnlDocumentation = DocumentationPanel(self.lbPrefs, -1)	
		self.pnlDocumentation.SetName("Documentation")	
		
		self.pnlDrScript = DrScriptPanel(self.lbPrefs, -1)
		self.pnlDrScript.SetName("DrScript")
		
		self.pnlFindReplace = FindReplacePanel(self.lbPrefs, -1)
		self.pnlFindReplace.SetName("Find/Replace")
				
		self.pnlGeneral = GeneralPanel(self.lbPrefs, -1)
		self.pnlGeneral.SetName("General")
		
		self.pnlLineEndings = LineEndingsPanel(self.lbPrefs, -1)
		self.pnlLineEndings.SetName("Line Endings")
		
		self.pnlPlugins = PluginsPanel(self.lbPrefs, -1)
		self.pnlPlugins.SetName("Plugins")
		
		self.pnlPrint = PrintPanel(self.lbPrefs, -1)
		self.pnlPrint.SetName("Printing")
				
		self.pnlPrompt = PromptPanel(self.lbPrefs, -1)
		self.pnlPrompt.SetName("Prompt")
						
		self.pnlSidePanes = SidePanePanel(self.lbPrefs, -1)
		self.pnlSidePanes.SetName("Side Panes")	
		
		self.pnlSourceBrowser = SourceBrowserPanel(self.lbPrefs, -1)
		self.pnlSourceBrowser.SetName("Source Browser")

		imagesize = (24,24)
		
		self.imagelist = wx.ImageList(imagesize[0], imagesize[1])
		
		bDir = self.parent.programdirectory + "/bitmaps/24/"
		
		self.images = [wx.BitmapFromImage(wx.Image(bDir+"bookmarks.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"document.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"Help.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"drscript.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"Find.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"drpython.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"line endings.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"Configure Plugins.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"Print File.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"prompt.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"side panes.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(bDir+"Toggle Source Browser.png", wx.BITMAP_TYPE_PNG))]
		
		map(self.imagelist.Add, self.images)
		
		self.lbPrefs.AssignImageList(self.imagelist)

		self.lbPrefs.AddPage(self.pnlBookmarks, "Bookmarks", imageId=0)
		self.lbPrefs.AddPage(self.pnlDocument, "Document", imageId=1)
		self.lbPrefs.AddPage(self.pnlDocumentation, "Documentation", imageId=2)
		self.lbPrefs.AddPage(self.pnlDrScript, "DrScript Menu", imageId=3)
		self.lbPrefs.AddPage(self.pnlFindReplace, "Find/Replace", imageId=4)
		self.lbPrefs.AddPage(self.pnlGeneral, "General", imageId=5)
		self.lbPrefs.AddPage(self.pnlLineEndings, "Line Endings", imageId=6)
		self.lbPrefs.AddPage(self.pnlPlugins, "Plugins", imageId=7)
		self.lbPrefs.AddPage(self.pnlPrint, "Printing", imageId=8)
		self.lbPrefs.AddPage(self.pnlPrompt, "Prompt", imageId=9)		
		self.lbPrefs.AddPage(self.pnlSidePanes, "Side Panes", imageId=10)
		self.lbPrefs.AddPage(self.pnlSourceBrowser, "Source Browser", imageId=11)
		
		self.lbPrefs.SetSelection(self.parent.prefdialogposition)

		self.theSizer = wx.BoxSizer(wx.VERTICAL)
						
		self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)

		self.btnCancel = wx.Button(self, self.ID_CLOSE, "   &Close   ")

		self.btnCancel.SetDefault()	

		#Have dialog box pop up:  reset to file in your home directory, or program defaults?
		self.btnReset = wx.Button(self, self.ID_RESET, "   &Reset All   ")

	   	self.btnUpdate = wx.Button(self, self.ID_UPDATE, "   &Update   ")

		self.btnSave = wx.Button(self, self.ID_SAVE, "   &Save   ")

		self.buttonSizer.Add(self.btnCancel, 0, wx.SHAPED)
		self.buttonSizer.Add(self.btnReset, 0, wx.SHAPED)
		self.buttonSizer.Add(self.btnUpdate, 0, wx.SHAPED)
		self.buttonSizer.Add(self.btnSave, 0, wx.SHAPED)
		
		self.theSizer.Add(self.lbPrefs, 1, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.buttonSizer, 0, wx.SHAPED | wx.ALIGN_CENTER)
				
		self.SetAutoLayout(True)
		self.SetSizer(self.theSizer)
		self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.OnPageChanged, id=self.ID_LB_PREFS)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnClose, id=self.ID_CLOSE)	
		self.Bind(wx.EVT_BUTTON,  self.OnbtnReset, id=self.ID_RESET)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnUpdate, id=self.ID_UPDATE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnSave, id=self.ID_SAVE)

		self.parent.LoadDialogSizeAndPosition(self, 'preferencesdialog.sizeandposition.dat')

	def OnCloseW(self, event):
		self.parent.SaveDialogSizeAndPosition(self, 'preferencesdialog.sizeandposition.dat')
		if event is not None:
			event.Skip()
				
	def OnbtnClose(self, event):
		self.OnCloseW(None)
		self.EndModal(0)
		
	def OnbtnReset(self, event):		
		d = wx.MessageDialog(self, "This will reset all preferences to the program default.\n(You still need to click update and/or save)\nAre you sure you want to do this?", "Reset Preferences", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_YES):
		
			self.prefs.reset()
			
			self.pnlSourceBrowser.reset()
			self.pnlDocument.reset()
			self.pnlDocumentation.reset()
			self.pnlDrScript.reset()
			self.pnlFindReplace.reset()
			self.pnlGeneral.reset()
			self.pnlLineEndings.reset()
			self.pnlPlugins.reset()
			self.pnlPrint.reset()
			self.pnlPrompt.reset()
			self.pnlSidePanes.reset()
	
	def OnbtnResetPanel(self, event):		
		d = wx.MessageDialog(self, "This will reset all of this panel's preferences to the program default.\n(You still need to click update and/or save)\nAre you sure you want to do this?", "Reset Panel Preferences", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_YES):
			panel = event.GetEventObject().GetParent()
			self.prefs.resetjust(panel.GetName())
			panel.reset()
			
	def OnbtnUpdate(self, event):
		self.GetParent().prefs = self.GetPreferences()
		self.GetParent().updatePrefs(self.oldprefs)
		self.oldprefs.Copy(self.prefs)
		if self.GetParent().prefs.enablefeedback:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Succesfully updated the current instance of DrPython.\nClick Save to make it permanent."), "Updated Preferences")
			d.ShowModal()
			d.Destroy()
		
	def OnbtnSave(self, event):
		if (not os.path.exists(self.homedirectory)):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Dude, you've got some problems...\nYour homedirectory (" + self.homedirectory + ") does not exist!\nLet's not bother speculating about how or why.\nRead the help file for this truly screwed up situation.\nDrPython will now politely ignore your request to save.\nTry again when you have fixed this problem."), "Huge Error")
			d.ShowModal()
			d.Destroy()
			return
		preffile = self.homedirectory + "/preferences.dat"
		try:					
			drPrefsFile.WritePreferences(self.GetPreferences(), preffile)
		except IOError:			
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("There were some problems writing to:\n"  + preffile + "\nEither the file is having metaphysical issues, or you do not have permission to write.\nFor metaphysical issues, consult the documentation.\nFor permission issues, change the permissions on the directory to allow yourself write access.\nDrPython will now politely ignore your request to save.\nTry again when you have fixed the problem."), "Write Error")
			d.ShowModal()
			d.Destroy()
			return
		self.GetParent().prefs = self.GetPreferences()
		self.GetParent().updatePrefs(self.oldprefs)
		self.oldprefs.Copy(self.prefs)
		if self.GetParent().prefs.enablefeedback:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Succesfully wrote to:\n"  + preffile + "\nand updated the current instance of DrPython."), "Saved Preferences")
			d.ShowModal()
			d.Destroy()
		
	def GetPreferences(self):
		#General
		self.prefs.rememberwindowsize = int(self.pnlGeneral.chkrememberwindowsize.GetValue())
		self.prefs.rememberdialogsizesandpositions = int(self.pnlGeneral.chkrememberdialogsizesandpositions.GetValue())
		self.prefs.recentfileslimit = self.pnlGeneral.txtRecentFiles.GetValue()
		if (len(self.prefs.recentfileslimit) < 1):
			self.prefs.recentfileslimit = self.oldprefs.recentfileslimit
		self.prefs.recentfileslimit = int(self.prefs.recentfileslimit)
		self.prefs.tabwidth = self.pnlGeneral.txttabwidth.GetValue()
		if (len(self.prefs.tabwidth) < 1):
			self.prefs.tabwidth = self.oldprefs.tabwidth
		self.prefs.tabwidth = int(self.prefs.tabwidth)
		self.prefs.iconsize = int(self.pnlGeneral.boxiconsize.GetStringSelection())
		self.prefs.pythonargs = self.pnlGeneral.txtpythonargs.GetValue()
		self.prefs.encoding = self.pnlGeneral.boxencoding.GetSelection()
		self.prefs.customencoding = self.pnlGeneral.txtcustomencoding.GetValue()
		self.prefs.iconfile = self.pnlGeneral.txticonfile.GetValue()
		self.prefs.defaultdirectory = self.pnlGeneral.txtdefaultdirectory.GetValue()			
		self.prefs.enablefeedback = int(self.pnlGeneral.chkenablefeedback.GetValue())	
		self.prefs.alwayspromptonexit = int(self.pnlGeneral.chkalwayspromptonexit.GetValue())		
		self.prefs.backupfileonsave = int(self.pnlGeneral.chkbackupfileonsave.GetValue())
		#utf-8 auto detect limodou 2004/04/14
		self.prefs.autodetectutf8encoding = int(self.pnlGeneral.chkautodetectutf8encoding.GetValue())
		#end limodou
		
		#Document
		self.prefs.docmarginwidth = self.pnlDocument.txtmarginwidth.GetValue()
		if (len(self.prefs.docmarginwidth) < 1):
			self.prefs.docmarginwidth = self.oldprefs.docmarginwidth
		self.prefs.docmarginwidth = int(self.prefs.docmarginwidth)
		self.prefs.docupdateindentation = int(self.pnlDocument.chkdocupdateindentation.GetValue())
		self.prefs.docautoindent = self.pnlDocument.raddocautoindent.GetSelection()
		self.prefs.docwordwrap = int(self.pnlDocument.chkdocwordwrap.GetValue())
		self.prefs.docusetabs = int(self.pnlDocument.chkdocusetabs.GetValue())
		self.prefs.docwhitespaceisvisible = int(self.pnlDocument.chkWhitespace.GetValue())
		self.prefs.docfolding = int(self.pnlDocument.chkfolding.GetValue())
		self.prefs.docusestyles = self.pnlDocument.radusestyles.GetSelection()
		self.prefs.docuseindentationguides = int(self.pnlDocument.chkuseindentationguides.GetValue())
		
		self.prefs.doccaretwidth = self.pnlDocument.txtcaretwidth.GetValue()
		if (len(self.prefs.doccaretwidth) < 1):
			self.prefs.doccaretwidth = self.oldprefs.doccaretwidth
		self.prefs.doccaretwidth = int(self.prefs.doccaretwidth)
			
		self.prefs.dochighlightcurrentline = int(self.pnlDocument.chkhighlightcurrentline.GetValue())
		#Chris McDonough (I added the txt prefix)
		self.prefs.doclonglinecol = self.pnlDocument.txtdoclonglinecol.GetValue()
		if len(self.prefs.doclonglinecol) < 1:
			self.prefs.doclonglinecol = self.oldprefs.doclonglinecol
		self.prefs.doclonglinecol = int(self.prefs.doclonglinecol)
		#/Chris McDonough
		self.prefs.docscrollextrapage = int(self.pnlDocument.chkscrollextrapage.GetValue())		
		self.prefs.docdefaultsyntaxhighlighting = self.pnlDocument.boxdefaultsyntaxhighlighting.GetSelection()
		self.prefs.doconlyusedefaultsyntaxhighlighting = self.pnlDocument.chkonlyusedefaultsyntaxhighlighting.GetValue()
		
		#Prompt
		self.prefs.promptmarginwidth = self.pnlPrompt.txtmarginwidth.GetValue()
		if (len(self.prefs.promptmarginwidth) < 1):
			self.prefs.promptmarginwidth = self.oldprefs.promptmarginwidth
		self.prefs.promptmarginwidth = int(self.prefs.promptmarginwidth)
		self.prefs.promptusetabs = int(self.pnlPrompt.chkpromptusetabs.GetValue())
		self.prefs.promptisvisible = int(self.pnlPrompt.chkVisible.GetValue())
		self.prefs.promptsize = int(self.pnlPrompt.sldrSize.GetValue())
		self.prefs.promptwordwrap = int(self.pnlPrompt.chkwordwrap.GetValue())
		
		self.prefs.promptcaretwidth = self.pnlPrompt.txtcaretwidth.GetValue()
		if (len(self.prefs.promptcaretwidth) < 1):
			self.prefs.promptcaretwidth = self.oldprefs.promptcaretwidth
		self.prefs.promptcaretwidth = int(self.prefs.promptcaretwidth)
		
		self.prefs.promptwhitespaceisvisible = int(self.pnlPrompt.chkWhitespace.GetValue())
		self.prefs.promptusestyles = self.pnlPrompt.radusestyles.GetSelection()
		self.prefs.promptscrollextrapage = int(self.pnlPrompt.chkscrollextrapage.GetValue())
		
		#Find/Replace
		self.prefs.findreplaceregularexpression = int(self.pnlFindReplace.chkregularexpression.GetValue())
		self.prefs.findreplacematchcase = int(self.pnlFindReplace.chkmatchcase.GetValue())
		self.prefs.findreplacefindbackwards = int(self.pnlFindReplace.chkfindbackwards.GetValue())
		self.prefs.findreplacewholeword = int(self.pnlFindReplace.chkwholeword.GetValue())
		self.prefs.findreplaceinselection = int(self.pnlFindReplace.chkinselection.GetValue())
		self.prefs.findreplacefromcursor = int(self.pnlFindReplace.chkfromcursor.GetValue())
		self.prefs.findreplacepromptonreplace = int(self.pnlFindReplace.chkpromptonreplace.GetValue())		
		self.prefs.findreplaceundercursor = int(self.pnlFindReplace.chkundercursor.GetValue())
		
		#Line Endings
		self.prefs.checkeol = int(self.pnlLineEndings.chkCheckFormat.GetValue())
		self.prefs.eolmode = self.pnlLineEndings.radFormat.GetSelection()
		
		#Side Panes
		self.prefs.sidepaneleftsize = int(self.pnlSidePanes.sldrSizeLeft.GetValue())
		self.prefs.sidepanerightsize = int(self.pnlSidePanes.sldrSizeRight.GetValue())
		
		#Source Browser
		self.prefs.sourcebrowserpane = self.pnlSourceBrowser.radposition.GetSelection()
		self.prefs.sourcebrowserisvisible = int(self.pnlSourceBrowser.chkIsVisible.GetValue())
		self.prefs.sourcebrowsercloseonactivate = int(self.pnlSourceBrowser.chkcloseonactivate.GetValue())
		self.prefs.sourcebrowserissorted = int(self.pnlSourceBrowser.chkissorted.GetValue())		
		
		#Plugins
		self.prefs.pluginsdefaultdirectory = self.pnlPlugins.txtdefaultdirectory.GetValue()
		
		#DrScript
		self.prefs.drscriptdefaultdirectory = self.pnlDrScript.txtdefaultdirectory.GetValue()
		self.prefs.drscriptloadexamples = int(self.pnlDrScript.chkloadexamples.GetValue())
		
		#Printing
		self.prefs.printdoclinenumbers = int(self.pnlPrint.chkdoclinenumbers.GetValue())
		self.prefs.printpromptlinenumbers = int(self.pnlPrint.chkpromptlinenumbers.GetValue())
		self.prefs.printtabwidth = self.pnlPrint.txttabwidth.GetValue()
		if (len(self.prefs.printtabwidth) < 1):
			self.prefs.printtabwidth = self.oldprefs.printtabwidth
		self.prefs.printtabwidth = int(self.prefs.printtabwidth)
		
		#Documentation
		self.prefs.documentationbrowser = self.pnlDocumentation.txtbrowser.GetValue()
		self.prefs.documentationpythonlocation = self.pnlDocumentation.txtpython.GetValue()
		self.prefs.documentationwxwidgetslocation = self.pnlDocumentation.txtwxwidgets.GetValue()
		self.prefs.documentationrehowtolocation = self.pnlDocumentation.txtrehowto.GetValue()
		
		return self.prefs
	
	def OnPageChanged(self, event):
		i = self.lbPrefs.GetSelection()
		self.boxpages.SetSelection(i)
		self.parent.prefdialogposition = i