#!/usr/bin/env python

#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Requirements(Dependencies):  Install Python(Version >= 2.2), and wxPython(Corresponding version)(>= 2.4.2).
#
#	Tested On Windows, Linux, Mac OS X
#
#	Icons taken from "Klassic New Crisp Icons" by Asif Ali Rizwaan (therizwaan) from the KDE-LOOK site (some edited a bit).
#	A humble and excellent artist.
#	Oh, the python icon is taken from wxPython.
#	The basic design of the program is meant to roughly (ROUGHLY) mimick DrScheme.
#	The purpose is the same, to provide a simple IDE(integrated development environment) ideal for teaching.
#	The DrPython icon itself was based on the DrScheme icon, with a slightly edited wxpython icon inserted(note yellow tongue, googly eyes).
#	
#	This program could not have been written without the wonderful work of the people behind
#	python and wxPython, in particular the Styled Text Control.  Thank you.  Hopefully this tool will be of use.
#
#	Replaced all rstrip('\n') with rstrip(), thanks Christof Ecker: (drpython.py and DrPrefs.py).
#
#	Version: 3.4.0

#franz: stat is not used
import os.path, sys, os, shutil, locale, re, string
import wx, wx.stc, drScrolledMessageDialog
from drText import DrText
from drPrompt import DrPrompt
from drPrinter import DrPrinter
from drFindReplaceDialog import drFindReplaceDialog, drFinder
from drBookmarksMenu import drBookmarksMenu
from drScriptMenu import drScriptMenu
from drPluginMenu import drPluginMenu, drPluginAboutMenu, drPluginHelpMenu, drPluginPreferencesMenu
import drFileDialog
import drPrefsFile
import drPopUp
from drPreferences import drPreferences
import drShortcutsFile
import drShortcuts
import drToolBarFile
from drSourceBrowser import drSourceBrowserPanel
#Moved here by drpython
import drDetectUTF_8

#*******************************************************************************************************

def getEndOfLineCharacter(character):
	if character == '\r' or character == '\n':
		return character
	return ""

#*******************************************************************************************************

class drNotebook(wx.Notebook):
	def __init__(self, parent, id, position, size, flags = 0):
		wx.Notebook.__init__(self, parent, id, position, size, flags)
		
		self.parent = parent
		
		self.PromptHasFocus = 0
		
		imagesize = (16,16)
		
		self.imagelist = wx.ImageList(imagesize[0], imagesize[1])
		self.images = [wx.BitmapFromImage(wx.Image(self.parent.programdirectory + "/bitmaps/16/unmodified.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(self.parent.programdirectory + "/bitmaps/16/modified.png", wx.BITMAP_TYPE_PNG))]
	
		map(self.imagelist.Add, self.images)
		
		self.AssignImageList(self.imagelist)
		
		self.Bind(wx.EVT_LEFT_DOWN, self.OnFindFocus)
		self.Bind(wx.EVT_LEFT_UP, self.OnSelectTab)
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnPopUp)
	
	def OnFindFocus(self, event):
		self.PromptHasFocus = self.parent.txtPrompt.GetSTCFocus()
		
		event.Skip()
	
	def OnPopUp(self, event):
	
		closeallmenu = wx.Menu()				
		closeallmenu.Append(self.parent.ID_CLOSE_ALL, "Close &All Tabs"," Close All Tabs")
		closeallmenu.Append(self.parent.ID_CLOSE_ALL_OTHER_DOCUMENTS, "Close All &Other Tabs"," Close All Other Tabs")
		
		tabmenu = wx.Menu()
		tabmenu.Append(self.parent.ID_CLOSE, "&Close", "Close the file")
		tabmenu.Append(self.parent.ID_CLOSE_ALL, "Close &All Tabs")
		tabmenu.Append(self.parent.ID_CLOSE_ALL_OTHER_DOCUMENTS, "Close All &Other Tabs")
		tabmenu.AppendSeparator()
		tabmenu.Append(self.parent.ID_NEXT_DOCUMENT, "Next Tab")
		tabmenu.Append(self.parent.ID_PREVIOUS_DOCUMENT, "Previous Tab")
		tabmenu.Append(self.parent.ID_FIRST_DOCUMENT, "First Tab")
		tabmenu.Append(self.parent.ID_LAST_DOCUMENT, "Last Tab")
		tabmenu.AppendSeparator()
		tabmenu.Append(self.parent.ID_RELOAD, "&Reload File"," Reload the current file")
		tabmenu.Append(self.parent.ID_RESTORE_FROM_BACKUP, "&Restore From Backup"," Restore from the last backup made")
		tabmenu.AppendSeparator()
		tabmenu.Append(self.parent.ID_SAVE, "&Save"," Save the file")
		tabmenu.Append(self.parent.ID_SAVE_AS, "Save &As..."," Save a file as ... another file")
		
		tabmenu.Enable(self.parent.ID_RELOAD, len(self.parent.txtDocument.filename) > 0)
		tabmenu.Enable(self.parent.ID_RESTORE_FROM_BACKUP, len(self.parent.txtDocument.filename) > 0)
		self.PopupMenu(tabmenu, event.GetPosition())
		tabmenu.Destroy()
	
	def SetTab(self):
		selection = self.GetSelection()
		page = self.GetPage(selection)
		self.parent.setDocumentTo(selection)
		if self.PromptHasFocus and page.PromptIsVisible:
			self.parent.txtPromptArray[self.parent.docPosition].SetSTCFocus(True)
		else:
			self.parent.txtDocumentArray[self.parent.docPosition].SetSTCFocus(True)
			
	def OnSelectTab(self, event):		
		selection = self.GetSelection()
		if selection is not self.parent.docPosition:
			self.SetTab()
		event.Skip()

class drSashLayoutWindow(wx.SashLayoutWindow):
	def __init__(self, parent, id, pos, size, style):
		wx.SashLayoutWindow.__init__(self, parent, id, pos, size, style)
		
		self.grandparent = parent.grandparent
		
		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
										
	def OnKeyDown(self, event):
		self.grandparent.RunShortcuts(event)

		event.Skip()
		
class drPanel(wx.Panel):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		
		self.grandparent = parent.GetParent()
		
		width, height = self.GetSizeTuple()
		
		#Variables to Keep Track of what is being used.
			
		self.ID_DOCUMENT = 6001 + id
		self.ID_PROMPT = 6002 + id
		self.ID_LEFT = 6003 + id
		self.ID_RIGHT = 6004 + id
		
		self.PromptIsVisible = self.grandparent.prefs.promptisvisible
		self.LeftIsVisible = 0
		self.RightIsVisible = 0
				
		self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_DOCUMENT)			
		self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_PROMPT)		
		self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_LEFT)	
		self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_RIGHT)

		self.toptuple = (width, height)
		self.lefttuple = (0, 0)
		self.righttuple = (0, 0)
		self.bottomtuple = (0, 0)

		self.promptsize = self.grandparent.prefs.promptsize
		self.leftsize = 0
		self.rightsize = 0

		self.left = drSashLayoutWindow(self, self.ID_LEFT, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.left.SetDefaultSize((0, 0))
		self.left.SetOrientation(wx.LAYOUT_VERTICAL)
		self.left.SetAlignment(wx.LAYOUT_LEFT)
		self.left.SetSashVisible(wx.SASH_RIGHT, True)
		self.left.SetSashBorder(wx.SASH_RIGHT, True)

		self.right = drSashLayoutWindow(self, self.ID_RIGHT, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.right.SetDefaultSize((0, 0))
		self.right.SetOrientation(wx.LAYOUT_VERTICAL)
		self.right.SetAlignment(wx.LAYOUT_RIGHT)
		self.right.SetSashVisible(wx.SASH_LEFT, True)
		self.right.SetSashBorder(wx.SASH_LEFT, True)

		self.top = drSashLayoutWindow(self, self.ID_DOCUMENT, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.top.SetDefaultSize((width, height))
		self.top.SetOrientation(wx.LAYOUT_HORIZONTAL)
		self.top.SetAlignment(wx.LAYOUT_TOP)
		self.top.SetSashVisible(wx.SASH_BOTTOM, True)
		self.top.SetSashBorder(wx.SASH_BOTTOM, True)
		self.top.SetSashVisible(wx.SASH_LEFT, True)
		self.top.SetSashBorder(wx.SASH_LEFT, True)
		self.top.SetSashVisible(wx.SASH_RIGHT, True)
		self.top.SetSashBorder(wx.SASH_RIGHT, True)

		self.bottom = drSashLayoutWindow(self, self.ID_PROMPT, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.bottom.SetDefaultSize((width, height))
		self.bottom.SetOrientation(wx.LAYOUT_HORIZONTAL)
		self.bottom.SetAlignment(wx.LAYOUT_BOTTOM)
		self.bottom.SetSashVisible(wx.SASH_TOP, True)
		self.bottom.SetSashBorder(wx.SASH_TOP, True)
		
		self.Bind(wx.EVT_SIZE, self.OnSize)
		
		self.LeftPanel = None
		self.RightPanel = None
	
	def AppendPanel(self, panel, Left=True):
		if Left:
			self.LeftPanel = panel
		else:
			self.RightPanel = panel
	
	def ClearPanel(self, Left=True):
		if Left:
			if self.LeftPanel is not None:
				self.LeftPanel.Destroy()
		else:
			if self.RightPanel is not None:
				self.RightPanel.Destroy()
		
	def GetTargetSashWindow(self, Left=True):
			if Left:
				target = self.left
				self.leftsize = self.grandparent.prefs.sidepaneleftsize
				self.LeftIsVisible = True
			else:
				target = self.right
				self.rightsize = self.grandparent.prefs.sidepanerightsize
				self.RightIsVisible = True
			return target
		
	def OnSashDrag(self, event):
		evtheight = event.GetDragRect().height
		evtwidth = event.GetDragRect().width
		width, height = self.GetSizeTuple()
		if (evtwidth < 0):
			evtwidth = 0
		elif (evtwidth > width):
			evtwidth = width
		if event.GetDragStatus() == wx.SASH_STATUS_OUT_OF_RANGE:
			if (not self.PromptIsVisible) or (evtheight < height):
				evtheight = 0
			else:
				evtheight = height
		elif evtheight > height:
			evtheight = height

		oldsize = self.promptsize
		loldsize = self.leftsize
		roldsize = self.rightsize

		e = event.GetId()
		edge = event.GetEdge()
		if e == self.ID_DOCUMENT:
			if edge == wx.SASH_BOTTOM:
				self.top.SetDefaultSize((width, evtheight))
				self.bottom.SetDefaultSize((width, height-evtheight))
				self.promptsize = ((height*100) - (evtheight*100)) / height
				self.toptuple = (self.toptuple[0], evtheight)
				self.bottomtuple = (self.bottomtuple[0], height-evtheight)
			elif edge == wx.SASH_LEFT:
				if self.RightIsVisible:
					evtwidth = evtwidth + self.righttuple[0]					
				self.top.SetDefaultSize((evtwidth, height))
				self.left.SetDefaultSize((width-evtwidth, height))
				self.lefttuple = (width-evtwidth, height)
				self.toptuple = (evtwidth, self.toptuple[1])
				self.leftsize = ((width*100) - (evtwidth*100)) / width
			elif edge == wx.SASH_RIGHT:
				if self.LeftIsVisible:
					evtwidth = evtwidth + self.lefttuple[0]	
				self.top.SetDefaultSize((evtwidth, height))
				self.right.SetDefaultSize((width-evtwidth, height))
				self.righttuple = (width-evtwidth, height)
				self.toptuple = (evtwidth, self.toptuple[1])
				self.rightsize = ((width*100) - (evtwidth*100)) / width
		elif e == self.ID_PROMPT:
			self.top.SetDefaultSize((width, height-evtheight))
			self.promptsize = ((evtheight*100) / height)
		elif e == self.ID_LEFT:
			if self.LeftIsVisible:				
				if self.lefttuple[0] == evtwidth:
					evtwidth = 0
			self.top.SetDefaultSize((width-evtwidth, height))
			self.left.SetDefaultSize((evtwidth, height))
			self.lefttuple = (evtwidth, height)
			self.toptuple = (width-evtwidth, self.toptuple[1])
			self.leftsize = (evtwidth*100) / width
		elif e == self.ID_RIGHT:
			if self.RightIsVisible:				
				if self.righttuple[0] == evtwidth:
					evtwidth = 0
			self.top.SetDefaultSize((width-evtwidth, height))
			self.right.SetDefaultSize((evtwidth, height))
			self.righttuple = (evtwidth, height)
			self.toptuple = (width-evtwidth, self.toptuple[1])
			self.rightsize = (evtwidth*100) / width
		if self.promptsize == 0:
			self.promptsize = oldsize
			self.PromptIsVisible = False
		elif not self.PromptIsVisible and self.bottomtuple[1] > 0:
			self.PromptIsVisible = True
		
		if self.leftsize == 0:
			self.leftsize = loldsize
			self.LeftIsVisible = False
		elif not self.LeftIsVisible and self.lefttuple[0] > 0:
			self.LeftIsVisible = True
		
		if self.rightsize == 0:
			self.rightsize = roldsize
			self.RightIsVisible = False
		elif not self.RightIsVisible and self.righttuple[0] > 0:
			self.RightIsVisible = True
					
		self.OnSize(event)
		self.Refresh()

	def OnSize(self, event):
		width, height = self.GetSizeTuple()
		if self.PromptIsVisible:
			self.bottom.Show(1)
			heightDocument = (height * (100 - self.promptsize)) / 100
			heightPrompt = (height * self.promptsize) / 100
		else:
			self.bottom.Show(0)
			heightDocument = height
			heightPrompt = 0
		if self.LeftIsVisible and self.RightIsVisible:
			w = (width * (100 - self.leftsize - self.rightsize)) / 100
		elif self.LeftIsVisible:
			w = (width * (100 - self.leftsize)) / 100
		elif self.RightIsVisible:
			w = (width * (100 - self.rightsize)) / 100
		else:
			w = width
		wl = 0
		wr = 0
		if self.LeftIsVisible:
			wl = (width * self.leftsize) / 100
		if self.RightIsVisible:
			wr = (width * self.rightsize) / 100
							
		self.toptuple = (w, heightDocument)
		self.lefttuple = (wl, height)
		self.righttuple = (wr, height)
		self.bottomtuple = (w, heightPrompt)		
		
		self.top.SetDefaultSize(self.toptuple)
		self.bottom.SetDefaultSize(self.bottomtuple)
		self.left.SetDefaultSize(self.lefttuple)
		self.right.SetDefaultSize(self.righttuple)
		
#wxPython Mac Bug Workaround
		if wx.Platform == '__WXMAC__':
			try:
				self.grandparent.txtDocument.SetSize(self.toptuple)
				self.grandparent.txtPrompt.SetSize(self.bottomtuple)
				self.LeftPanel.SetSize(self.lefttuple)
				self.RightPanel.SetSize(self.righttuple)
			except:
				pass
#/wxPython Mac Bug Workaround
		
		wx.LayoutAlgorithm().LayoutWindow(self)

class drObject(wx.Object):
	def __init__(self):
		#wx.Object.__init__(self)
		pass
				
	def VariableExists(self, varname):
		try:
			eval("self." + varname)
		except:
			return False
		return True
		
#*******************************************************************************************************

class DrFrame(wx.Frame):
	def __init__(self, parent, id, title, fn = "", EditRecentFiles = 1, Prefs = None):

		wx.Frame.__init__(self, parent, id, title, wx.Point(0, 0), wx.Size(640, 480))

		self.InitializeConstants()
				
		#Directories:
		if (not os.path.exists(self.homedirectory)):
			os.mkdir(self.homedirectory)
		if not os.path.exists(self.homedirectory + "/plugins"):
			os.mkdir(self.homedirectory + "/plugins")
				
		sys.path.append(self.homedirectory + "/plugins")
			
		self.lastprogargs = ""

		self.DrScript = drObject()
		self.DrPlugins = drObject()

		#Sets all image handlers.  DrPython uses png, jpg, gif.
		wx.InitAllImageHandlers()

		if (not os.path.exists(self.bitmapdirectory)):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Bitmap Directory (" + self.bitmapdirectory + ") Does Not Exist.\nThis is either a bug with DrPython,\n an error with your installation,\nor the bitmap directory was simply removed."), "DrPython Fatal Error")
			d.ShowModal()
			d.Destroy()
			sys.exit(1)

		self.Printer = DrPrinter(self)

		self.breakpoints = []
		
		#Find/Replace
		
		self.FindHistory = []
		self.ReplaceHistory = []
		
		self.FindInFilesHistory = []
		self.ReplaceInFilesHistory = []
		
		self.FindOptions = []
		self.ReplaceOptions = []
		
		#Used for current directory with open/save
		self.ddirectory = ""
		
		self.stcshortcutlist = drShortcutsFile.GetSTCShortcutList()
						
		#Preferences
		
		self.prefs = drPreferences(self.PLATFORM_IS_WIN, self.programdirectory)
				
		if Prefs is not None:
			self.prefs.Copy(Prefs)
		else:
			self.LoadPreferences()	
		
		WindowWidth = 640
		WindowHeight = 480
		if self.prefs.rememberwindowsize:
			if os.path.exists(self.homedirectory + '/windowsize.dat'):
				try:
					f = file(self.homedirectory + '/windowsize.dat', 'r')
					text = f.read()
					WindowWidth, WindowHeight = map(int, text.split('\n'))
					f.close()
					self.SetSize((WindowWidth, WindowHeight))
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Error Loading Window Size", 'Error')
					d.ShowModal()
					d.Destroy()
									
		if (len(self.prefs.defaultdirectory) > 0):
			self.ddirectory = self.prefs.defaultdirectory

		#Default position is for "General" preferences.
		self.prefdialogposition = 5

		icon = wx.EmptyIcon()
		icon.CopyFromBitmap(wx.BitmapFromImage(wx.Image((self.bitmapdirectory + "/drpython.png"), wx.BITMAP_TYPE_PNG)))
		self.SetIcon(icon)
		
		self.mdinotebook = drNotebook(self, -1, wx.DefaultPosition, wx.Size(WindowWidth, WindowHeight), wx.CLIP_CHILDREN)
			
		self.mdinotebook.AddPage(drPanel(self.mdinotebook, self.ID_APP), "Untitled 1")
		
		self.currentpage = self.mdinotebook.GetPage(0)
	
		self.txtDocument = DrText(self.currentpage.top, self.ID_DOCUMENT_BASE, self)
		self.txtPrompt = DrPrompt(self.currentpage.bottom, self.ID_PROMPT_BASE, self)
		self.txtDocument.SetTargetPosition(0)
		self.txtDocument.untitlednumber = 1
		
		self.mdinotebook.SetPageImage(0, 0)
		
		#Filenames
		self.txtDocument.filename = fn
				
		#Pop Up Menu
		
		self.popupmenulist = []
		
		self.LoadPopUpFile()
							
		self.Finder = drFinder(self)
					
		#Position in the Arrays Below:
		self.docPosition = 0
				
		#Arrays for MDI Support (Only Hold One Item if SDI):
		self.txtDocumentArray = [self.txtDocument]
		self.txtPromptArray = [self.txtPrompt]
		self.FinderArray = [self.Finder]
		self.lastprogargsArray = [self.lastprogargs]
		
		#file encoding array limodou 2004/04/16
		self.txtDocument.locale = self.prefs.encoding	#default encoding
		self.txtPrompt.locale = self.prefs.encoding
		#end limodou
		
		#Shortcuts
		
		self.STCShortcuts = []
		self.STCKeycodeArray = []
		self.STCShortcutsArgumentArray = drShortcuts.GetSTCCommandList()
		
		self.Shortcuts = []
		self.KeycodeArray = []
		self.ShortcutsIgnoreString = ""
		
		self.ShortcutsActionArray = []
		self.ShortcutsArgumentsArray = []
		
		#DrScript Shortcuts
		
		self.DrScriptShortcuts = []
		self.DrScriptKeycodeArray = []
		
		#Plugins
		self.LoadedPlugins = []
		
		self.PluginShortcutFunctionNames = []
		self.PluginShortcutFunctions = []
		
		self.PluginShortcuts = []
		self.PluginKeycodeArray = []
		self.PluginAction = []		
		
		self.PluginPopUpMenuNames = []
		self.PluginPopUpMenuLabels = []
		self.PluginPopUpMenuFunctions = []
				
		self.PluginToolBarLabels = []
		self.PluginToolBarIconFiles16 = []
		self.PluginToolBarIconFiles24 = []
		self.PluginToolBarFunctions = []
		
		self.STCUseDefault = 1
		
		#Load Shortcuts
		self.LoadShortcuts()
		
		#Shortcuts
		drShortcuts.SetSTCShortcuts(self.txtPrompt, self.STCShortcuts, self.STCUseDefault)
		self.STCKeycodeArray = drShortcuts.SetSTCShortcuts(self.txtDocument, self.STCShortcuts, self.STCUseDefault)
		self.KeycodeArray, self.ShortcutsActionArray, self.ShortcutsArgumentsArray = drShortcuts.SetShortcuts(self, self.Shortcuts)
			
		#Sizer
		self.bSizer = wx.BoxSizer(wx.VERTICAL)	
						
		self.windowArray = []
				
		self.recentfiles = []
		
		self.LoadRecentFiles()

		#edited by drpython
		if self.prefs.defaultdirectory == '':
			#add limodou 2004/04/17
			#if defaultdirectory is empty, then use the last recently file's dir
			if self.ddirectory == '' and len(self.recentfiles)>0:	
				self.ddirectory = os.path.dirname(self.recentfiles[0])
			#end limodou
							
		self.filemenu = wx.Menu()
		self.filemenu.Append(self.ID_NEW, "&New")
		self.filemenu.Append(self.ID_OPEN, "&Open...")
		self.recentmenu = wx.Menu()
		self.CreateRecentFileMenu()
		self.filemenu.AppendMenu(self.ID_OPEN_RECENT, "Open &Recent", self.recentmenu)
		self.filemenu.Append(self.ID_RELOAD, "&Reload File")
		self.filemenu.Append(self.ID_RESTORE_FROM_BACKUP, "&Restore From Backup")
		self.filemenu.AppendSeparator()
		self.filemenu.Append(self.ID_CLOSE, "&Close")
		self.filemenu.AppendSeparator()
		self.filemenu.Append(self.ID_CLEAR_RECENT, "Clear Recent File List")
		self.filemenu.AppendSeparator()
		self.filemenu.Append(self.ID_SAVE, "&Save")
		self.filemenu.Append(self.ID_SAVE_AS, "Save &As...")
		self.filemenu.Append(self.ID_SAVE_PROMPT, "Save Prompt Output To File...")
		self.filemenu.AppendSeparator()
		self.filemenu.Append(self.ID_PRINT_SETUP,"Print Setup...")
		self.filemenu.Append(self.ID_PRINT,"&Print File...")
		self.filemenu.Append(self.ID_PRINTPROMPT,"Print Prompt...")
		self.filemenu.AppendSeparator()
		self.filemenu.Append(self.ID_EXIT,"E&xit")

		self.commentmenu = wx.Menu()
		self.commentmenu.Append(self.ID_COMMENT_REGION, "&Comment")
		self.commentmenu.Append(self.ID_UNCOMMENT_REGION, "U&nComment")

		self.whitespacemenu = wx.Menu()		
		self.whitespacemenu.Append(self.ID_INDENT_REGION, "&Indent")
		self.whitespacemenu.Append(self.ID_DEDENT_REGION, "&Dedent")
		self.whitespacemenu.AppendSeparator()
		self.whitespacemenu.Append(self.ID_CHECK_INDENTATION, "Check Indentation Type...")
		self.whitespacemenu.Append(self.ID_CLEAN_UP_TABS, "Set Indentation To Tabs...")
		self.whitespacemenu.Append(self.ID_CLEAN_UP_SPACES, "Set Indentation To Spaces...")
		self.whitespacemenu.AppendSeparator()
		self.formatmenu = wx.Menu()
		self.formatmenu.Append(self.ID_UNIXMODE, "Unix Mode (\"\\n\")")
		self.formatmenu.Append(self.ID_WINMODE, "DOS/Windows Mode (\"\\r\\n\")")
		self.formatmenu.Append(self.ID_MACMODE, "Mac Mode (\"\\r\")")
		self.whitespacemenu.AppendMenu(self.ID_FORMATMENU, "Set Line Endings To", self.formatmenu)
				
		self.casemenu = wx.Menu()
		self.casemenu.Append(self.ID_UPPERCASE, "&Uppercase")
		self.casemenu.Append(self.ID_LOWERCASE, "&Lowercase")

		self.selectmenu = wx.Menu()
		self.selectmenu.Append(self.ID_SELECT_ALL, "&Select All")
		self.selectmenu.Append(self.ID_SELECT_NONE, "Select &None")

		self.encodingmenu = wx.Menu()
		self.encodingmenu.AppendRadioItem(self.ID_ENCODING_DEFAULT, "&Default")
		self.encodingmenu.AppendRadioItem(self.ID_ENCODING_ASCII, "&ASCII")
		self.encodingmenu.AppendRadioItem(self.ID_ENCODING_UTF_8, "&UTF-8 (Unicode)")
		self.encodingmenu.AppendRadioItem(self.ID_ENCODING_CUSTOM, "&Custom")

		#comment limodou 2004/04/16
		self.encodingmenu.Check(self.ID_ENCODING_DEFAULT + self.txtDocument.locale, True)
		#end limodou
		
		self.editmenu = wx.Menu()
		#franz:10_07
		self.editmenu.Append(self.ID_UNDO, "&Undo")
		self.editmenu.Append(self.ID_REDO, "R&edo")
		self.editmenu.AppendSeparator()
		#Order changed by drpython
		self.editmenu.Append(self.ID_CUT, "&Cut")
		self.editmenu.Append(self.ID_COPY, "&Copy")
		self.editmenu.Append(self.ID_PASTE, "&Paste")
		self.editmenu.Append(self.ID_DELETE, "&Delete")
		#end_franz:10_07
		self.editmenu.AppendSeparator()
		self.editmenu.Append(self.ID_INSERT_REGEX, "&Insert Regular Expression...")
		self.editmenu.AppendSeparator()
		self.editmenu.Append(self.ID_FIND_AND_COMPLETE, "Find And Complete")
		self.editmenu.AppendSeparator()
		self.editmenu.AppendMenu(self.ID_COMMENT, "&Comments", self.commentmenu)
		self.editmenu.AppendMenu(self.ID_WHITESPACE, "&Whitespace", self.whitespacemenu)
		self.editmenu.AppendMenu(self.ID_CASE, "Case", self.casemenu)
		self.editmenu.AppendMenu(self.ID_SELECT, "Select", self.selectmenu)
		self.editmenu.AppendSeparator()
		self.editmenu.AppendMenu(self.ID_ENCODING, "Encoding", self.encodingmenu)

		self.searchmenu = wx.Menu()
		self.searchmenu.Append(self.ID_FIND, "&Find...")
		self.searchmenu.Append(self.ID_FIND_NEXT, "Find &Next")
		self.searchmenu.Append(self.ID_FIND_PREVIOUS, "Find Previous")
		self.searchmenu.Append(self.ID_REPLACE, "&Replace...")
			
		self.foldmenu = wx.Menu()
		self.foldmenu.Append(self.ID_FOLD_ALL, "Fold All")
		self.foldmenu.Append(self.ID_EXPAND_ALL, "Expand All")

		self.highlightmenu = wx.Menu()
		self.highlightmenu.AppendRadioItem(self.ID_HIGHLIGHT_PYTHON, "Python")
		self.highlightmenu.AppendRadioItem(self.ID_HIGHLIGHT_CPP, "C/C++")
		self.highlightmenu.AppendRadioItem(self.ID_HIGHLIGHT_HTML, "HTML")
		self.highlightmenu.AppendRadioItem(self.ID_HIGHLIGHT_PLAIN_TEXT, "Plain Text")
		self.highlightmenu.Check(self.ID_HIGHLIGHT_PYTHON, True)
		
		self.viewmenu = wx.Menu()
		self.viewmenu.Append(self.ID_GOTO, "&Go To...")
		self.viewmenu.AppendSeparator()
		self.viewmenu.Append(self.ID_ZOOM_IN, "Zoom &In")
		self.viewmenu.Append(self.ID_ZOOM_OUT, "Zoom &Out")		
		self.viewmenu.AppendSeparator()
		self.viewmenu.AppendMenu(self.ID_HIGHLIGHT, "&Syntax Highlighting", self.highlightmenu)
		self.viewmenu.AppendSeparator()
		self.viewmenu.AppendMenu(self.ID_FOLDING, "&Folding", self.foldmenu)
		self.viewmenu.AppendSeparator()
		self.viewmenu.Append(self.ID_TOGGLE_SOURCEBROWSER, "Toggle &Source Browser...")
		self.viewmenu.AppendSeparator()
		#fix bug someone refered in forum limodou 2004/04/20
		self.viewmenu.Append(self.ID_TOGGLE_VIEWWHITESPACE, "Toggle View &Whitespace")
		#end limodou
		self.viewmenu.Append(self.ID_TOGGLE_PROMPT, "&Toggle Prompt")
		self.viewmenu.Append(self.ID_CLEAR_PROMPT, "C&lear Prompt")	
		
		self.programmenu = wx.Menu()
		self.programmenu.Append(self.ID_RUN, "&Run")
		self.programmenu.Append(self.ID_SET_ARGS, "Set &Arguments...")
		self.programmenu.Append(self.ID_PYTHON, "Open a Python &Interpreter")
		self.programmenu.Append(self.ID_END, "&End")
				
		self.bookmarksmenu = drBookmarksMenu(self)
		self.drscriptmenu = drScriptMenu(self)
		
		#DrScript Shortcuts
		self.DrScriptShortcutsAction = self.drscriptmenu.OnScript
		
		self.pluginsmenu = drPluginMenu(self)
		self.pluginsprefsmenu = drPluginPreferencesMenu(self)

		self.documentsmenu = wx.Menu()
		self.setupdocumentsmenu()
				
		self.optionsmenu = wx.Menu()
		self.optionsmenu.Append(self.ID_PREFS, "&Preferences...")
		self.optionsmenu.Append(self.ID_SHORTCUTS, "&Customize Shortcuts...")
		self.optionsmenu.Append(self.ID_POPUP, "&Customize Pop Up Menu...")
		self.optionsmenu.Append(self.ID_CUSTOMIZE_TOOLBAR, "&Customize ToolBar...")
		self.optionsmenu.AppendSeparator()		
		self.optionsmenu.Append(self.ID_EDIT_BOOKMARKS, "Edit Bookmarks...")
		self.optionsmenu.Append(self.ID_EDIT_SCRIPT_MENU, "&Edit Script Menu...")
		self.optionsmenu.AppendSeparator()
		self.optionsmenu.Append(self.ID_CONFIGURE_PLUGINS, "&Configure Plugins...")
		self.optionsmenu.AppendMenu(self.ID_PLUGIN_PREFS, "Plugin Preferences", self.pluginsprefsmenu)
		self.optionsmenu.AppendSeparator()
		self.optionsmenu.AppendMenu(self.ID_LOAD_PLUGIN, "&Load Plugin(s) From Index", self.pluginsmenu)
				
		self.pluginshelpmenu = drPluginHelpMenu(self)
		self.pluginsaboutmenu = drPluginAboutMenu(self)
				
		self.helpmenu = wx.Menu()
		self.helpmenu.Append(self.ID_ABOUT, "&About DrPython...")
		self.helpmenu.AppendMenu(self.ID_PLUGIN_ABOUT, "About Plugin", self.pluginsaboutmenu)
		self.helpmenu.AppendSeparator()
		self.helpmenu.Append(self.ID_HELP, "DrPython &Help...")
		self.helpmenu.AppendMenu(self.ID_PLUGIN_HELP, "Plugin Help", self.pluginshelpmenu)
		self.helpmenu.AppendSeparator()
		self.helpmenu.Append(self.ID_PYTHON_DOCS, "&Python Docs")
		self.helpmenu.Append(self.ID_WXWIDGETS_DOCS, "&WxWidgets Docs")
		self.helpmenu.Append(self.ID_REHOWTO_DOCS, "&Regular Expression Howto")

		self.menuBar = wx.MenuBar()
		self.menuBar.Append(self.filemenu,"&File")
		self.menuBar.Append(self.editmenu,"&Edit")
		self.menuBar.Append(self.searchmenu,"&Search")
		self.menuBar.Append(self.viewmenu,"&View")
		self.menuBar.Append(self.programmenu,"&Program")
		self.menuBar.Append(self.bookmarksmenu,"&Bookmarks")
		self.menuBar.Append(self.drscriptmenu,"&DrScript")
		self.menuBar.Append(self.documentsmenu, "&Documents")
		self.menuBar.Append(self.optionsmenu,"&Options")
		self.menuBar.Append(self.helpmenu,"&Help")
		
		self.SetMenuBar(self.menuBar)
		
		#ToolBar
		self.hasToolBar = False
		try:
			self.ToolBarList = drToolBarFile.getToolBarList(self.homedirectory)
		except:		
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Loading ToolBar List"), "DrPython Error")
			d.ShowModal()
			d.Destroy()
			
		if (self.prefs.iconsize > 0):		
			self.hasToolBar = True
			self.toolbar = wx.ToolBar(self, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_HORIZONTAL)

			self.ToolBarIdList = self.SetupToolBar()
			
			self.bSizer.Add(self.toolbar, 0, wx.EXPAND)

		#Sizer
		
		self.bSizer.Add(self.mdinotebook, 1, wx.EXPAND)
		self.SetAutoLayout(True)
		self.SetSizer(self.bSizer)				
			
		self.UpdateMenuAndToolbar()
			
		self.Bind(wx.EVT_MENU,  self.OnNew, id=self.ID_NEW)
		self.Bind(wx.EVT_MENU,  self.OnOpen, id=self.ID_OPEN)		
				
		self.Bind(wx.EVT_MENU,  self.OnClose, id=self.ID_CLOSE)
		self.Bind(wx.EVT_MENU,  self.OnCloseAllDocuments, id=self.ID_CLOSE_ALL)
		self.Bind(wx.EVT_MENU,  self.OnCloseAllOtherDocuments, id=self.ID_CLOSE_ALL_OTHER_DOCUMENTS)
		
		self.Bind(wx.EVT_MENU,  self.OnReload, id=self.ID_RELOAD)
		self.Bind(wx.EVT_MENU,  self.OnRestoreFromBackup, id=self.ID_RESTORE_FROM_BACKUP)
		self.Bind(wx.EVT_MENU,  self.OnClearRecent, id=self.ID_CLEAR_RECENT)
		self.Bind(wx.EVT_MENU,  self.OnSave, id=self.ID_SAVE)
		self.Bind(wx.EVT_MENU,  self.OnSaveAs, id=self.ID_SAVE_AS)
		self.Bind(wx.EVT_MENU,  self.OnSaveAll, id=self.ID_SAVE_ALL)
		self.Bind(wx.EVT_MENU,  self.OnSavePrompt, id=self.ID_SAVE_PROMPT)
		self.Bind(wx.EVT_MENU,  self.OnPrintSetup, id=self.ID_PRINT_SETUP)
		self.Bind(wx.EVT_MENU,  self.OnPrint, id=self.ID_PRINT)
		self.Bind(wx.EVT_MENU,  self.OnPrintPrompt, id=self.ID_PRINTPROMPT)
		self.Bind(wx.EVT_MENU,  self.OnExit, id=self.ID_EXIT)
		
		self.Bind(wx.EVT_MENU,  self.OnMenuFind, id=self.ID_FIND)
		self.Bind(wx.EVT_MENU,  self.OnMenuFindNext, id=self.ID_FIND_NEXT)
		self.Bind(wx.EVT_MENU,  self.OnMenuFindPrevious, id=self.ID_FIND_PREVIOUS)
		self.Bind(wx.EVT_MENU,  self.OnMenuReplace, id=self.ID_REPLACE)
								
		self.Bind(wx.EVT_MENU,  self.OnInsertRegEx, id=self.ID_INSERT_REGEX)
		
		self.Bind(wx.EVT_MENU,  self.OnSelectAll, id=self.ID_SELECT_ALL)
		self.Bind(wx.EVT_MENU,  self.OnSelectNone, id=self.ID_SELECT_NONE)
		
		self.Bind(wx.EVT_MENU,  self.OnCommentRegion, id=self.ID_COMMENT_REGION)
		self.Bind(wx.EVT_MENU,  self.OnUnCommentRegion, id=self.ID_UNCOMMENT_REGION)
		
		self.Bind(wx.EVT_MENU,  self.OnIndentRegion, id=self.ID_INDENT_REGION)
		self.Bind(wx.EVT_MENU,  self.OnDedentRegion, id=self.ID_DEDENT_REGION)
		
		self.Bind(wx.EVT_MENU,  self.OnCheckIndentation, id=self.ID_CHECK_INDENTATION)
		self.Bind(wx.EVT_MENU,  self.OnCleanUpTabs, id=self.ID_CLEAN_UP_TABS)
		self.Bind(wx.EVT_MENU,  self.OnCleanUpSpaces, id=self.ID_CLEAN_UP_SPACES)
		
		self.Bind(wx.EVT_MENU,  self.OnFormatUnixMode, id=self.ID_UNIXMODE)
		self.Bind(wx.EVT_MENU,  self.OnFormatWinMode, id=self.ID_WINMODE)
		self.Bind(wx.EVT_MENU,  self.OnFormatMacMode, id=self.ID_MACMODE)
				
		self.Bind(wx.EVT_MENU,  self.OnEncodingDefault, id=self.ID_ENCODING_DEFAULT)
		self.Bind(wx.EVT_MENU,  self.OnEncodingASCII, id=self.ID_ENCODING_ASCII)
		self.Bind(wx.EVT_MENU,  self.OnEncodingUTF8, id=self.ID_ENCODING_UTF_8)
		self.Bind(wx.EVT_MENU,  self.OnEncodingCustom, id=self.ID_ENCODING_CUSTOM)
		
		self.Bind(wx.EVT_MENU,  self.OnFindAndComplete, id=self.ID_FIND_AND_COMPLETE)		
		
		self.Bind(wx.EVT_MENU,  self.OnUppercase, id=self.ID_UPPERCASE)
		self.Bind(wx.EVT_MENU,  self.OnLowercase, id=self.ID_LOWERCASE)
		self.Bind(wx.EVT_MENU,  self.OnUndo, id=self.ID_UNDO)
		self.Bind(wx.EVT_MENU,  self.OnRedo, id=self.ID_REDO)		
		
		self.Bind(wx.EVT_MENU,  self.OnGoTo, id=self.ID_GOTO)
		self.Bind(wx.EVT_MENU,  self.OnZoomIn, id=self.ID_ZOOM_IN)
		self.Bind(wx.EVT_MENU,  self.OnZoomOut, id=self.ID_ZOOM_OUT)
		
		self.Bind(wx.EVT_MENU,  self.OnSyntaxHighlightingPython, id=self.ID_HIGHLIGHT_PYTHON)
		self.Bind(wx.EVT_MENU,  self.OnSyntaxHighlightingCPP, id=self.ID_HIGHLIGHT_CPP)
		self.Bind(wx.EVT_MENU,  self.OnSyntaxHighlightingHTML, id=self.ID_HIGHLIGHT_HTML)
		self.Bind(wx.EVT_MENU,  self.OnSyntaxHighlightingText, id=self.ID_HIGHLIGHT_PLAIN_TEXT)
		
		self.Bind(wx.EVT_MENU,  self.OnFoldAll, id=self.ID_FOLD_ALL)
		self.Bind(wx.EVT_MENU,  self.OnExpandAll, id=self.ID_EXPAND_ALL)
		
		self.Bind(wx.EVT_MENU,  self.OnToggleSourceBrowser, id=self.ID_TOGGLE_SOURCEBROWSER)
		self.Bind(wx.EVT_MENU,  self.OnToggleViewWhiteSpace, id=self.ID_TOGGLE_VIEWWHITESPACE)		
		self.Bind(wx.EVT_MENU,  self.OnTogglePrompt, id=self.ID_TOGGLE_PROMPT)
		self.Bind(wx.EVT_MENU,  self.OnClearPrompt, id=self.ID_CLEAR_PROMPT)
		
		self.Bind(wx.EVT_MENU,  self.OnRun, id=self.ID_RUN)
		self.Bind(wx.EVT_MENU,  self.OnSetArgs, id=self.ID_SET_ARGS)
		self.Bind(wx.EVT_MENU,  self.OnPython, id=self.ID_PYTHON)
		self.Bind(wx.EVT_MENU,  self.OnEnd, id=self.ID_END)

		self.Bind(wx.EVT_MENU,  self.OnPrefs, id=self.ID_PREFS)
		self.Bind(wx.EVT_MENU,  self.OnCustomizeShortcuts, id=self.ID_SHORTCUTS)
		self.Bind(wx.EVT_MENU,  self.OnCustomizePopUpMenu, id=self.ID_POPUP)
		self.Bind(wx.EVT_MENU,  self.OnCustomizeToolBar, id=self.ID_CUSTOMIZE_TOOLBAR)		
		self.Bind(wx.EVT_MENU,  self.OnEditBookmarks, id=self.ID_EDIT_BOOKMARKS)		
		self.Bind(wx.EVT_MENU,  self.OnEditScriptMenu, id=self.ID_EDIT_SCRIPT_MENU)
		self.Bind(wx.EVT_MENU,  self.OnConfigurePlugins, id=self.ID_CONFIGURE_PLUGINS)
				
		self.Bind(wx.EVT_MENU,  self.OnViewAbout, id=self.ID_ABOUT)
		self.Bind(wx.EVT_MENU,  self.OnViewHelp, id=self.ID_HELP)
		self.Bind(wx.EVT_MENU,  self.OnViewPythonDocs, id=self.ID_PYTHON_DOCS)
		self.Bind(wx.EVT_MENU,  self.OnViewWxWidgetsDocs, id=self.ID_WXWIDGETS_DOCS)
		self.Bind(wx.EVT_MENU,  self.OnViewREHowtoDocs, id=self.ID_REHOWTO_DOCS)		
		
		self.Bind(wx.EVT_MENU, self.DoBuiltIn, id=self.ID_COPY)		
		self.Bind(wx.EVT_MENU, self.DoBuiltIn, id=self.ID_PASTE)
		self.Bind(wx.EVT_MENU, self.DoBuiltIn, id=self.ID_CUT)
		self.Bind(wx.EVT_MENU, self.DoBuiltIn, id=self.ID_DELETE)
		
		self.Bind(wx.EVT_MENU, self.OnSelectDocumentNext, id=self.ID_NEXT_DOCUMENT)
		self.Bind(wx.EVT_MENU, self.OnSelectDocumentPrevious, id=self.ID_PREVIOUS_DOCUMENT)
		self.Bind(wx.EVT_MENU, self.OnSelectDocumentFirst, id=self.ID_FIRST_DOCUMENT)
		self.Bind(wx.EVT_MENU, self.OnSelectDocumentLast, id=self.ID_LAST_DOCUMENT)		
		
		self.CreateStatusBar()

		self.GetStatusBar().SetFieldsCount(3)
		#First field is hidden, to absorb wxMenuHighlight events from the menu and toolbar.
		self.GetStatusBar().SetStatusWidths([-0, -6, -4])

		if self.prefs.eolmode == 1:
			eolmodestr = "WIN"
		elif self.prefs.eolmode == 2:
			eolmodestr = "MAC"
		else:
			eolmodestr = "UNIX"
		if self.txtDocument.GetOvertype():
			ovrstring = "OVR"
		else:
			ovrstring = "INS"				

		self.txtDocument.SetupPrefsDocument()

		self.txtDocument.SetFocus()
		
		self.txtDocument.OnPositionChanged(None)
		
		self.txtPrompt.SetReadOnly(1)
		
		self.txtPrompt.SetupPrefsPrompt()
				
		#Arguments To Program
		if (len(sys.argv) > 1) and (len(self.txtDocument.filename) == 0):
			f = sys.argv[1]
			if self.PLATFORM_IS_WIN:
				f = f.replace("\\", "/")
			self.DestroyRecentFileMenu()
			self.OpenFile(f, False)
			self.txtDocument.OnModified(None)
			x = 2
			l = len(sys.argv)
			while x < l:
				f = sys.argv[x]
				if self.PLATFORM_IS_WIN:
					f = f.replace("\\", "/")
				self.OpenFile(f, True)
				self.txtDocument.OnModified(None)
				x = x + 1			
			self.CreateRecentFileMenu()
		#Argument to DrFrame.
		elif (len(self.txtDocument.filename) > 0):			
			#Make sure the path is absolute
			self.txtDocument.filename = os.path.abspath(self.txtDocument.filename)
			#Convert Filename to correct format. (Only needed when called from the command line)
			#When called from function, already converted.
			if self.PLATFORM_IS_WIN:
				self.txtDocument.filename = self.txtDocument.filename.replace("\\", "/")
			self.DestroyRecentFileMenu()
			self.OpenFile(self.txtDocument.filename, False, EditRecentFiles)
			self.CreateRecentFileMenu()
		
		#Load SourceBrowser:
		if self.prefs.sourcebrowserisvisible:
			self.ShowSourceBrowser()
			
		self.Bind(wx.EVT_END_PROCESS,  self.OnProcessEnded, id=-1)	
			
		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
		self.Bind(wx.EVT_CLOSE, self.OnCloseW)
		
		self.LoadPlugins()

	def AddKeyEvent(self, function, Keycode, Control=0, Shift=0, Alt=0, Meta=0):
		if Keycode == -1:
			return
	
		d = drShortcuts.drShortcut()
		
		d.Name = "Plugin"
		d.Control = Control
		d.Shift = Shift
		d.Meta = Meta
		d.Alt = Alt
		d.Keycode = str(Keycode)
		self.PluginShortcuts.append(d)
		self.PluginKeycodeArray.append(d.Keycode)
		self.PluginAction.append(function)

	def AddPluginFunction(self, plugin, label, function):
		self.PluginShortcutFunctionNames.append(plugin + ":" + label)
		self.PluginShortcutFunctions.append(function)
		
		self.PluginPopUpMenuNames.append(plugin)
		self.PluginPopUpMenuLabels.append(label)
		self.PluginPopUpMenuFunctions.append(function)
		
		self.PluginToolBarLabels.append("<Plugin>:"+label)
		self.PluginToolBarFunctions.append(function)

	def AddPluginIcon(self, name, location16, location24):
		f = file(self.homedirectory + "/toolbar.custom.icons.dat", 'r')
		lines = f.read().split('\n')
		f.close()		
		name = "<Plugin>:" + name
		f = file(self.homedirectory + "/toolbar.custom.icons.dat", 'w')
		for line in lines:
			if len(line) > 0:
				currentname = drPrefsFile.ExtractPreferenceFromText(line, "Name")
				if currentname != name:
					f.write(line + '\n')
		f.write("<Name>" + name + "</Name><16>" + location16 + "</16><24>" + \
		location24 + "</24>\n")
		f.close()		

	def AddPluginShortcutFunction(self, plugin, label, function):
		self.PluginShortcutFunctionNames.append(plugin + ":" + label)
		self.PluginShortcutFunctions.append(function)

	def AddPluginPopUpMenuFunction(self, plugin, label, function):
		self.PluginPopUpMenuNames.append(plugin)
		self.PluginPopUpMenuLabels.append(label)
		self.PluginPopUpMenuFunctions.append(function)
	
	def AddPluginToolBarFunction(self, label, function):
		self.PluginToolBarLabels.append("<Plugin>:"+label)
		self.PluginToolBarFunctions.append(function)
	
	def checkiffileisCPP(self, filename):
		cpp = re.compile("(\.c$)|(\.cc$)|(\.cpp$)|(\.cxx$)|(\.h$)|(\.hh$)|(\.hpp$)|(\.hxx$)", re.IGNORECASE)
		return cpp.search(filename) is not None
				
	def checkiffileisHTML(self, filename):
		cpp = re.compile("(\.htm$)|(\.html$)|(\.shtm$)|(\.shtml$)|(\.xml$)", re.IGNORECASE)
		return cpp.search(filename) is not None
		
	def checkiffileisPlainText(self, filename):
		cpp = re.compile("(\.txt$)|(\.dat$)|(\.log$)", re.IGNORECASE)
		return cpp.search(filename) is not None
	
	def checkiffileisPython(self, filename):
		cpp = re.compile("(\.py$)|(\.pyw$)", re.IGNORECASE)
		return cpp.search(filename) is not None
			
	def CheckIndentation(self):
		text = self.txtDocument.GetText()
		if len(text) < 1:
			return 2
		#franz: mixed is not used
		eol = self.txtDocument.GetEndOfLineCharacter()
		
		retab = re.compile(eol + '\t', re.MULTILINE)
		respaces = re.compile(eol + ' ', re.MULTILINE)
			
		tabs = (retab.search(text) is not None) or (text[0] == '\t')
		spaces = (respaces.search(text) is not None) or (text[0] == ' ')	
		
		if tabs and spaces:
			return 0
		elif tabs:
			return 1
		elif spaces:
			return -1
		return 2		
			
	def CreateRecentFileMenu(self):
		x = 0
		numfiles = len(self.recentfiles)
		while (x < numfiles):
			self.recentmenu.Append(self.ID_RECENT_FILES_BASE+x, self.recentfiles[x], ("Open " + self.recentfiles[x]))
			self.Bind(wx.EVT_MENU,  self.OnOpenRecentFile, id=self.ID_RECENT_FILES_BASE+x)
			x = x + 1
	
	def DestroyDocument(self):	
		self.txtDocumentArray.pop(self.docPosition)
		self.txtPromptArray.pop(self.docPosition)	
		self.FinderArray.pop(self.docPosition)
		self.lastprogargsArray.pop(self.docPosition)
	
	def DestroyRecentFileMenu(self):
		#You need to call this function BEFORE you update the list of recent files.
		x = 0
		mnuitems = self.recentmenu.GetMenuItems()
		num = len(mnuitems)
		while (x < num):
			self.recentmenu.Remove(self.ID_RECENT_FILES_BASE+x)
			mnuitems[x].Destroy()
			x = x + 1
			
	def DestroyToolBar(self):
		x = 0
		toolbarsize = len(self.ToolBarIdList)
		while (x < toolbarsize):
			if self.ToolBarIdList[x] == -300:
				self.toolbar.DeleteToolByPos(0)
			else:
				self.toolbar.DeleteTool(self.ToolBarIdList[x])
			x = x + 1
	
	def DoBuiltIn(self, event):
		objid = event.GetId()
			
		if objid == self.ID_COPY:
			self.txtDocument.CmdKeyExecute(wx.stc.STC_CMD_COPY)
		elif objid == self.ID_PASTE:
			self.txtDocument.CmdKeyExecute(wx.stc.STC_CMD_PASTE)
		elif objid == self.ID_CUT:
			self.txtDocument.CmdKeyExecute(wx.stc.STC_CMD_CUT)
		elif objid == self.ID_DELETE:
			self.txtDocument.CmdKeyExecute(wx.stc.STC_CMD_CLEAR)
	
	def dynamicdrscript(self, event):
		self.drscriptmenu.OnDynamicScript(event)
	
	def Execute(self, command, statustext = ""):
		if len(statustext) < 1:
			statustext = "Running Command"
		self.runcommand(command, statustext)
		
	def ExecutePython(self, command = "", statustext = ""):
		if len(statustext) < 1:
			statustext = "Running Python Interpreter"
		self.txtPrompt.pythonintepreter = 1
		if len(command) > 0:
			command = " " + command
		if self.PLATFORM_IS_WIN:
			self.runcommand((self.pythexecw + " -u " + self.prefs.pythonargs + command), "Running Python Interpreter")
		else:
			self.runcommand((self.pythexec + " -u " + self.prefs.pythonargs + command), "Running Python Interpreter")
	
	#GetText function limodou 2004/04/16
	#if encoding is None, then use txtctrl.locale
	#most of the time should be encodingid when save a file
	def GetEncodedText(self, txtctrl, encodingid=0):
		if encodingid == 0:
			encodingid = txtctrl.locale
		if encodingid == 3: #custom encoding
			encoding = self.prefs.customencoding
		else:
			encoding = self.encodings[encodingid]
		if encoding == "<None>":
			encoding=self.encodings[0]
		isUnicode=txtctrl.GetCodePage()!=0
		text=txtctrl.GetText()
		try:
			if isUnicode:
				sText=text.encode(encoding)
			else:
				sText=unicode(text, self.encodings[0]).encode(encoding)
		except:
			pass
			sText=text

		return sText
	#end limodou
	
	def GetFileName(self):
		return self.txtDocument.filename
	
	def GetNewId(self):		
		return 10000 + wx.NewId()	
	
	def GetPluginDirectory(self):
		return self.homedirectory + "/plugins"
	
	def GetPluginLabels(self, filename, doNotAppend = False):
		try:
			f = file(filename, 'r')
			text = f.read()
			f.close()
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(frame, 'File error with: "' + filename + '".', "ERROR")
			d.ShowModal()
			d.Destroy()
			return []
		
		rePopUpMenu = re.compile(r'^\s*?DrFrame\.AddPluginFunction\(.*\)', re.MULTILINE)
			
		allPopUps = rePopUpMenu.findall(text)
	
		PopUpArray = []
				
		for s in allPopUps:
			#From the Left most '('
			start = s.find('(')
			#To the Right most ')'
			end = s.rfind(')')	
			
			if (start > -1) and (end > -1):
				s = s[start+1:end]
				i = s.find(',')
				e = i + 1 + s[i+1:].find(',')
				arglabel = s[i+1:e].strip().strip('"')
				if doNotAppend:
					PopUpArray.append(arglabel)
				else:
					PopUpArray.append("<Plugin>:"+arglabel)
				
		return PopUpArray	
	
	def InitializeConstants(self):
		#Constant messages for file format checking.
		self.FFMESSAGE = ["Unix Mode ('\\n')", "DOS/Windows Mode ('\\r\\n')", "Mac Mode ('\\r')"]
		
		self.ID_DOCUMENT_BASE = 50
		self.ID_PROMPT_BASE = 340
		
		#Application ID Constants
		self.ID_APP = 101
		self.ID_NEW = 102
		self.ID_OPEN = 103
		self.ID_OPEN_RECENT = 104
		self.ID_RELOAD = 105
		self.ID_RESTORE_FROM_BACKUP = 1051
		self.ID_CLOSE = 106
		self.ID_CLOSE_ALL = 6061
		self.ID_CLOSE_ALL_OTHER_DOCUMENTS = 6062
		self.ID_CLEAR_RECENT = 107
		self.ID_SAVE = 108
		self.ID_SAVE_AS = 109
		self.ID_SAVE_ALL = 1098
		self.ID_SAVE_PROMPT = 1091
		self.ID_PRINT_SETUP = 1010
		self.ID_PRINT = 1011
		self.ID_PRINTPROMPT = 1012
		self.ID_EXIT = 1014
				
		self.ID_NEXT_DOCUMENT = 801
		self.ID_PREVIOUS_DOCUMENT = 802
		self.ID_FIRST_DOCUMENT = 803
		self.ID_LAST_DOCUMENT = 804
		self.ID_DOCUMENT_NAVIGATION_MENU = 810
		self.ID_DOCUMENTS_BASE = 8000
		self.ID_DOCUMENTS_MENU_BASE = 7950
		
		self.ID_COPY = 850		
		self.ID_PASTE = 851	
		self.ID_CUT = 852	
		self.ID_DELETE = 853
				
		self.ID_FIND = 111
		self.ID_FIND_NEXT = 112
		self.ID_FIND_PREVIOUS = 1122
		self.ID_REPLACE = 113
		self.ID_GOTO = 115
		self.ID_SELECT = 1150
		self.ID_SELECT_ALL = 1151
		self.ID_SELECT_NONE = 1152
		
		self.ID_INSERT_REGEX = 1153
		
		self.ID_COMMENT = 1116
		self.ID_COMMENT_REGION = 116
		self.ID_UNCOMMENT_REGION = 117
		
		self.ID_WHITESPACE = 1118
		self.ID_INDENT_REGION = 118
		self.ID_DEDENT_REGION = 119
		self.ID_CHECK_INDENTATION = 1650
				
		self.ID_CLEAN_UP_TABS = 1670
		self.ID_CLEAN_UP_SPACES = 1671
		
		self.ID_FORMATMENU = 2000
		self.ID_UNIXMODE = 2001
		self.ID_WINMODE = 2002
		self.ID_MACMODE = 2003
		
		self.ID_ENCODING = 2010
		self.ID_ENCODING_DEFAULT = 2011
		self.ID_ENCODING_ASCII = 2012
		self.ID_ENCODING_UTF_8 = 2013
		#comment limodou 2004/04/13
		self.ID_ENCODING_CUSTOM = 2014
		#end limodou
						
		self.ID_FIND_AND_COMPLETE = 2071
		
		self.ID_CASE = 1191
		self.ID_UPPERCASE = 1192
		self.ID_LOWERCASE = 1193
		
		self.ID_UNDO = 1111
		self.ID_REDO = 1112
		
		self.ID_ZOOM_IN = 161
		self.ID_ZOOM_OUT = 162
		self.ID_FOLDING = 1600
		self.ID_FOLD_ALL = 1601
		self.ID_EXPAND_ALL = 1602
		self.ID_TOGGLE_SOURCEBROWSER = 163
		self.ID_TOGGLE_VIEWWHITESPACE = 164
		self.ID_TOGGLE_PROMPT = 165
		self.ID_CLEAR_PROMPT = 166
		
		self.ID_HIGHLIGHT = 580
		
		self.ID_HIGHLIGHT_PYTHON = 585
		self.ID_HIGHLIGHT_CPP = 586
		self.ID_HIGHLIGHT_HTML = 587
		self.ID_HIGHLIGHT_PLAIN_TEXT = 589
		
		self.ID_RUN = 121
		self.ID_SET_ARGS = 122
		self.ID_PYTHON = 123
		self.ID_END = 125
		
		self.ID_PREFS = 131
		self.ID_SHORTCUTS = 133
		self.ID_POPUP = 134
		self.ID_CUSTOMIZE_TOOLBAR = 135
		
		self.ID_CONFIGURE_PLUGINS = 4005
		self.ID_LOAD_PLUGIN = 4050		
		self.ID_PLUGIN_HELP = 4051
		self.ID_PLUGIN_PREFS = 4052
		self.ID_PLUGIN_ABOUT = 4053
				
		self.ID_EDIT_BOOKMARKS = 301
		self.ID_EDIT_SCRIPT_MENU = 3004
				
		self.ID_ABOUT = 140
		self.ID_HELP = 141
		self.ID_PYTHON_DOCS = 142
		self.ID_WXWIDGETS_DOCS = 143
		self.ID_REHOWTO_DOCS = 144
		
		self.ID_OTHER = 9000
		
		self.ID_RECENT_FILES_BASE = 9930
		
		self.ID_RECENT_SESSIONS_BASE = 8330
		
		self.ID_SCRIPT_BASE = 7500
						
		#System constants
		
		self.PLATFORM_IS_WIN = (sys.platform == "win32")
		#If homedirectory does not work correctly on your self.PLATFORM,
		#comment it out, and set the variable to your homedirectory.
		#Just in case you don't check this code, and
		#homedirectory does not work, drpython will plop
		#its stuff in your root directory.
		self.homedirectory = os.path.expanduser("~")
		if (not os.path.exists(self.homedirectory)):
			if self.PLATFORM_IS_WIN:
				try:
					self.homedirectory = os.environ["APPDATA"].replace("\\", "/") + "/drpython"
				except:
					self.homedirectory = "c:/drpython"		
			else:
				self.homedirectory = "/.drpython"
		else:
			if self.PLATFORM_IS_WIN:
				self.homedirectory = self.homedirectory.replace("\\", "/") + "/drpython"
			else:
				self.homedirectory = self.homedirectory + "/.drpython"
		
		if self.PLATFORM_IS_WIN:
			self.pythexec = sys.prefix.replace("\\", "/") + "/python.exe"
			self.pythexecw = sys.prefix.replace("\\", "/") + "/pythonw.exe"
			#File Types
			self.wildcard = "Python Source (*.pyw *.py)|*.pyw;*.py|C/C++ Source (*.c *.cc *.cpp *.cxx *.h *.hh *.hpp *.hxx)|*.c;*.cc;*.cpp;*.cxx;*.h;*.hh;*.hpp;*.hxx|HTML Files (*.htm *.html *.shtm *.shtml *.xml)|*.htm;*.html;*.shtm;*.shtml;*.xml|Backup Files (*.bak)|*.bak|Plain Text (*.txt *.dat *.log)|*.txt;*.dat;*.log|All Files (*)|*"
		else:
			self.pythexec = sys.executable
			#File Types
			self.wildcard = "Python Source (*.py *.pyw)|*.py;*.pyw|C/C++ Source (*.c *.cc *.cpp *.cxx *.h *.hh *.hpp *.hxx)|*.c;*.cc;*.cpp;*.cxx;*.h;*.hh;*.hpp;*.hxx|HTML Files (*.htm *.html *.shtm *.shtml *.xml)|*.htm;*.html;*.shtm;*.shtml;*.xml|Backup Files (*.bak)|*.bak|Plain Text (*.txt *.dat *.log)|*.txt;*.dat;*.log|All Files (*)|*"
				
		#Thanks to Mark Rees.
		#Thanks to Guillermo Fernandez.
		#Thanks Bjorn Breid
		self.programdirectory = os.path.dirname(os.path.abspath(sys.argv[0])).replace("\\", "/")
		self.bitmapdirectory = self.programdirectory + "/bitmaps"
		
		defaultlocale = locale.getdefaultlocale()[1]
		if defaultlocale is None:
			defaultlocale = "ascii"
		
		self.encodings = [defaultlocale, "ascii", "utf-8"]
	
	def InitializePlugin(self, plugin, ShowDialog = True):
		#Check to see if the plugin is already loaded:
		if plugin in self.LoadedPlugins:
			if ShowDialog:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ('"' + plugin + '" has already been loaded.\nDrPython will not reload this plugin.'), "Plugin Already Loaded")
				d.ShowModal()
				d.Destroy()
			return
		
		#Load the Plugin
		pluginfile = self.homedirectory + "/plugins/" + plugin + ".py"
		self.LoadedPlugins.append(plugin)
		try:
			exec(compile("import " + plugin, pluginfile, 'exec'))
			exec(compile(plugin + ".Plugin(self)", pluginfile, 'exec'))
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + plugin + "\nDrPython will not load this plugin."), "Plugin Error")
			d.ShowModal()
			d.Destroy()
		
		#Menus
		self.pluginsaboutmenu.AddItem(plugin)
		self.pluginshelpmenu.AddItem(plugin)
		self.pluginsprefsmenu.AddItem(plugin)
			
		#Shortcuts:
		shortcutfile = self.homedirectory + "/plugins/" + plugin + ".shortcuts.dat"
		if not os.path.exists(shortcutfile):
			return
		try:
			shortcuts, keycodes, ignorestring = drShortcutsFile.ReadShortcuts(shortcutfile, 0)
			for shortcut in shortcuts:
				try:
					i = self.PluginShortcutFunctionNames.index(plugin + ":" + shortcut.Name)
					self.PluginAction.append(self.PluginShortcutFunctions[i])
					shortcut.Name = "Plugin"
					self.PluginShortcuts.append(shortcut)
					self.PluginKeycodeArray.append(shortcut.Keycode)
				except:
					pass
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + plugin + "\nDrPython will not load shortcuts for this plugin."), "Plugin Shortcuts Error")
			d.ShowModal()
			d.Destroy()
		
	def isvalidbreakpoint(self, text):
		if (len(text) <= 0):
			return False
		elif (text.isspace()):
			return False
		else:
			ind = text.find('#')
			if not (ind == -1):
				if (text[:ind].isspace()):
					return False
				elif (ind == 0):
					return False
		return True		
	
	def LoadDialogSizeAndPosition(self, dialog, dialogfile):
		if self.prefs.rememberdialogsizesandpositions:
			try:
				if os.path.exists(self.homedirectory + '/' + dialogfile):
					f = file(self.homedirectory + '/' + dialogfile, 'r')
					text = f.read()
					x, y, px, py = map(int, text.split('\n'))
					f.close()
					dialog.SetSize((x, y))
					dialog.Move(wx.Point(px, py))
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(dialog, "Error Loading Dialog Size", 'Error')
				d.ShowModal()
				d.Destroy()
		dialog.Bind(wx.EVT_CLOSE, dialog.OnCloseW)
	
	def LoadPlugins(self, plugins_file = ""):
		if len(plugins_file) > 0:
			pluginsfile = plugins_file
		else:
			pluginsfile = self.homedirectory + "/plugins/plugins.dat"
		if os.path.exists(pluginsfile):
			try:
				f = file(pluginsfile, 'r')
				pluginstoload = f.read().rstrip().split('\n')
				f.close()
								
				for plugin in pluginstoload:
					if len(plugin) > 0:
						self.InitializePlugin(plugin)					
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + pluginsfile + "\nDrPython will not load plugins."), "Plugins Error")
				d.ShowModal()
				d.Destroy()
	
	def LoadPopUpFile(self):
		#check for preferences file in user homedirectory
		if os.path.exists(self.homedirectory + "/popupmenu.dat"):
			popupfile = self.homedirectory + "/popupmenu.dat"
			try:
				f = file(popupfile, 'r')
				line = f.readline()
				while len(line) > 0:
					self.popupmenulist.append(line.rstrip())
					line = f.readline()
				f.close()
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + popupfile + "\nDrPython will use the program defaults."), "Pop Up Menu Error")
				d.ShowModal()
				d.Destroy()		
				
	def LoadPreferences(self, UseDefault = False):
		#check for preferences file in user homedirectory
		if (os.path.exists(self.homedirectory + "/preferences.dat")) and (not UseDefault):
			f = self.homedirectory + "/preferences.dat"
			try:
				drPrefsFile.ReadPreferences(self.prefs, f)
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + f + "\nDrPython will load the program defaults."), "Preferences Error")
				d.ShowModal()
				d.Destroy()
				self.LoadPreferences(True)
		else:
			self.prefs.reset()
				
	def LoadRecentFiles(self):
		f = self.homedirectory + "/recent_files.log"
		if (not os.path.exists(f)):
			try:
				t = open(f, 'w')
				t.close()
			except IOError:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Creating: " + f), "Recent Files Error")
				d.ShowModal()
				d.Destroy()
		try:
			fin = open(f, 'r')
			s = fin.readline()
			x = 0
			while (len(s) > 0) and (x < self.prefs.recentfileslimit):
				s = s.rstrip()
				if (len(s) > 0):
					self.recentfiles.append(s)
				x = x + 1
				s = fin.readline()
			fin.close()
		except IOError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Reading: " + f), "Recent Files Error")
			d.ShowModal()
			d.Destroy()
	
	def LoadShortcuts(self, UseDefault = False):
		#Load STC Shortcuts
		if (os.path.exists(self.homedirectory + "/stcshortcuts.dat")) and (not UseDefault):
			f = self.homedirectory + "/stcshortcuts.dat"
			try:
				self.STCShortcuts, self.STCKeycodeArray, t = drShortcutsFile.ReadShortcuts(f, 0)
				self.STCUseDefault = 0
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + f + "\nDrPython will not load DrScript shortcuts."), "DrScript Shortcuts Error")
				d.ShowModal()
				d.Destroy()	
		else:
			self.STCShortcuts, self.STCKeycodeArray = drShortcutsFile.GetDefaultSTCShortcuts()
		
		#check for shortcuts file in user homedirectory
		if (os.path.exists(self.homedirectory + "/shortcuts.dat")) and (not UseDefault):
			f = self.homedirectory + "/shortcuts.dat"
			try:
				self.Shortcuts, self.KeycodeArray, self.ShortcutsIgnoreString = drShortcutsFile.ReadShortcuts(f)
			except:	
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + f + "\nDrPython will load the program defaults."), "Shortcuts Error")
				d.ShowModal()
				d.Destroy()
				self.LoadShortcuts(True)
		else:
			self.Shortcuts, self.KeycodeArray, self.ShortcutsIgnoreString = drShortcutsFile.GetDefaultShortcuts()
			
		#Load DrScript Shortcuts
		if (os.path.exists(self.homedirectory + "/drscript.shortcuts.dat")) and (not UseDefault):
			f = self.homedirectory + "/drscript.shortcuts.dat"
			try:
				self.DrScriptShortcuts, self.DrScriptKeycodeArray, t = drShortcutsFile.ReadShortcuts(f, 0)
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error with: " + f + "\nDrPython will not load DrScript shortcuts."), "DrScript Shortcuts Error")
				d.ShowModal()
				d.Destroy()
				
	def OnCheckIndentation(self, event):
		wx.BeginBusyCursor()
		result = self.CheckIndentation()
		if result == 2:
			msg = "No indentation was found in this document."
		elif result == -1:
			msg = "This document uses spaces for indentation."
		elif result == 1:
			msg = "This document uses tabs for indentation."
		elif result == 0:
			msg = "This document is mixed.  It uses tabs and spaces for indentation."
		wx.EndBusyCursor()
		d = drScrolledMessageDialog.ScrolledMessageDialog(self, msg, "Check Indentation Results")
		d.ShowModal()
		d.Destroy()
	
	def OnCleanUpSpaces(self, event):
		d = wx.TextEntryDialog(self, "Replace a tab with how many spaces?:", "Replace Tabs With Spaces", str(self.prefs.tabwidth))
		answer = d.ShowModal()
		value = d.GetValue()
		d.Destroy()
		eol = self.txtDocument.GetEndOfLineCharacter()
		regex = re.compile('(\S)|' + eol)
		if answer == wx.ID_OK:
			wx.BeginBusyCursor()
			wx.Yield()
			try:
				x = int(value)
			except:					
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "You must enter an integer (number, eg 1,2,128)", "DrPython")
				d.ShowModal()
				d.Destroy()
				wx.EndBusyCursor()	
				return
			if (x > -1) and (x <= 128):
				text = self.txtDocument.GetText()
				lines = text.split(eol)
				new_lines = []
				for line in lines:
					result = regex.search(line + eol)
					if result is not None:
						end = result.start()
						new_lines.append(line[0:end].expandtabs(x) + line[end:])
					else:
						new_lines.append(line)
				newtext = string.join(new_lines, eol)
				self.txtDocument.SetText(newtext)
			else:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "That number seems WAY too high.  Just what are you doing, replacing  a tab with more than 128 spaces?", "DrPython Foolish Error")
				d.ShowModal()
				d.Destroy()			
				wx.EndBusyCursor()	
				return
			wx.EndBusyCursor()

	def OnCleanUpTabs(self, event):
		d = wx.TextEntryDialog(self, "Number of spaces to replace with a tab:", "Replace Spaces With Tabs", str(self.prefs.tabwidth))
		answer = d.ShowModal()
		value = d.GetValue()
		d.Destroy()
		eol = self.txtDocument.GetEndOfLineCharacter()
		regex = re.compile('(\S)|' + eol)
		if answer == wx.ID_OK:
			wx.BeginBusyCursor()		
			wx.Yield()
			try:
				x = int(value) - 1
			except:					
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "You must enter an integer (number, eg 1,2,128)", "DrPython")
				d.ShowModal()
				d.Destroy()				
				wx.EndBusyCursor()
				return
			if (x > -1) and (x <= 128):
				#Create Target String
				y = 0
				oof = " "
				while (y < x):
					oof = oof + " "
					y = y + 1
				#Continue
				text = self.txtDocument.GetText()
				lines = text.split(eol)
				new_lines = []
				for line in lines:
					result = regex.search(line + eol)
					if result is not None:
						end = result.start()
						new_lines.append(line[0:end].replace(oof, "\t") + line[end:])
					else:
						new_lines.append(line)
				newtext = string.join(new_lines, eol)
				self.txtDocument.SetText(newtext)
			else:	
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "That number seems WAY too high.  Just what are you doing, replacing more than 128 spaces with a tab?", "DrPython Foolish Error")
				d.ShowModal()
				d.Destroy()				
				wx.EndBusyCursor()
				return				
			wx.EndBusyCursor()

	def OnClearPrompt(self, event):
		d = wx.MessageDialog(self, "This will clear all text from the prompt.\nAre you sure you want to do this?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_YES):
			iii = self.txtPrompt.GetReadOnly()
			self.txtPrompt.SetReadOnly(0)
			self.txtPrompt.editpoint = 0
			self.txtPrompt.writeposition = 0
			self.txtPrompt.ClearAll()
			self.txtPrompt.SetReadOnly(iii)							

	def OnClearRecent(self, event):
		d = wx.MessageDialog(self, "This will clear all recent files.\nAre you sure you want to do this?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_YES):
			self.recentfiles = []
			self.DestroyRecentFileMenu()
			self.WriteRecentFiles()

	def OnClose(self, event):
		if (self.txtDocument.GetModify()):
			#prompt saving filename limodou 2004/04/19
			d = wx.MessageDialog(self, 'Would you like to save "%s"?' % self.txtDocument.GetFilename(), "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
			#end limodou
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_YES):
				self.OnSave(event)
			elif (answer == wx.ID_CANCEL):
				return
		self.txtDocument.SetText("")
		self.txtDocument.filename = ""
		self.txtDocument.EmptyUndoBuffer()
		self.txtDocument.SetSavePoint()				
		self.UpdateMenuAndToolbar()
		#franz: oldpos is not used
		if len(self.txtDocumentArray) > 1:
			self.mdinotebook.DeletePage(self.docPosition)
			self.DestroyDocument()
			if (self.docPosition > 0):				
				self.docPosition = self.docPosition - 1
			elif (len(self.txtDocumentArray) > 1):
				if self.docPosition > 0:
					self.docPosition = self.docPosition + 1
			self.setDocumentTo(self.docPosition, 1)
		else:
			#The set size stuff ensures that wx.widgets repaints the tab.
			x, y = self.GetSizeTuple()
			self.SetSize(wx.Size(x-1, y-1))
			self.SetSize(wx.Size(x, y))
			self.txtDocument.untitlednumber = 1
			
		#Update txtDocument.targetPosition
		x = 0
		l = len(self.txtDocumentArray)
		while x < l:
				self.txtDocumentArray[x].targetPosition = x
				x = x + 1
				
		self.txtDocument.OnModified(None)
			
	
	def OnCloseAllDocuments(self, event):
		x = len(self.txtDocumentArray) - 1
		while x > -1:
			self.setDocumentTo(x, True)
			if self.txtDocument.GetModify():
				#prompt saving filename limodou 2004/04/19
				d = wx.MessageDialog(self, 'Would you like to save "%s"?' % self.txtDocument.GetFilename(), "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
				#end limodou
				answer = d.ShowModal()
				d.Destroy()
				if (answer == wx.ID_YES):
					self.OnSave(event)
			self.OnClose(event)					
			x = x - 1
	
	def OnCloseAllOtherDocuments(self, event):
		if len(self.txtDocument.filename) < 1:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Sorry, does not work when an untitled file is selected.", "DrPython Error")
			d.ShowModal()
			d.Destroy()
			return				
		farray = map(lambda document: document.filename, self.txtDocumentArray)
		try:
			i = farray.index(self.txtDocument.filename)
		except:
			#franz: (Updated Namespace)
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Something went wrong trying to close all other tabs.", "DrPython Error")
			d.ShowModal()
			d.Destroy()
			return				
		
		x = len(farray) - 1	
		while x > -1:
			if x is not i:	
				self.setDocumentTo(x, True)
				if self.txtDocument.GetModify():
					#prompt saveing filename limodou 2004/04/19
					d = wx.MessageDialog(self, 'Would you like to save "%s"?' % self.txtDocument.GetFilename(), "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
					#end limodou
					answer = d.ShowModal()
					d.Destroy()
					if (answer == wx.ID_YES):
						self.OnSave(event)
				self.OnClose(event)
			x = x - 1
		
	def OnCloseW(self, event):
		if event.CanVeto():
				try:
					x = self.docPosition
					if (self.docPosition > 0):
						fromzero = self.docPosition
					l = len(self.txtDocumentArray)
					while x < l:
						if self.txtDocumentArray[x].GetModify():
							d = wx.MessageDialog(self, 'Would you like to save "%s"?' % self.txtDocumentArray[x].GetFilename(), "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
							answer = d.ShowModal()
							d.Destroy()
							if (answer == wx.ID_YES):
								self.setDocumentTo(x)
								self.OnSave(event)
							elif (answer == wx.ID_CANCEL):
								return
						x = x + 1
						
					if fromzero > 0:
						x = 0
						l = fromzero
						while x < l:
							if self.txtDocumentArray[x].GetModify():
								d = wx.MessageDialog(self, 'Would you like to save "%s"?' % self.txtDocumentArray[x].GetFilename(), "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
								answer = d.ShowModal()
								d.Destroy()
								if (answer == wx.ID_YES):
									self.setDocumentTo(x)
									self.OnSave(event)
								elif (answer == wx.ID_CANCEL):
									return
							x = x + 1
				except:
					if self.prefs.alwayspromptonexit:
						d = wx.MessageDialog(self, "Are you sure you want to exit?   ", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
						answer = d.ShowModal()
						d.Destroy()
						if (answer == wx.ID_NO):
							return
						
		if self.prefs.rememberwindowsize:
			try:
				f = file(self.homedirectory + '/windowsize.dat', 'w')
				x, y = self.GetSizeTuple()
				f.write(str(x) + '\n' + str(y))
				f.close()
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Error Saving Window Size", 'Error')
				d.ShowModal()
				d.Destroy()
		event.Skip()
		
	def OnCommentRegion(self, event):
		pos = self.txtDocument.GetCurrentPos()
		selstart, selend = self.txtDocument.GetSelection()
		#From the start of the first line selected
		startline = self.txtDocument.LineFromPosition(selstart)
		self.txtDocument.GotoLine(startline)
		start = self.txtDocument.GetCurrentPos()
		#To the end of the last line selected
		#Bugfix Chris Wilson
		end = self.txtDocument.GetLineEndPosition(self.txtDocument.LineFromPosition(selend-1))
		#End Bugfix Chris Wilson
		self.txtDocument.SetSelection(start, end)
		text = "#" + self.txtDocument.GetSelectedText()
		text = text.replace('\n', "\n#")
		self.txtDocument.ReplaceSelection(text)
		self.txtDocument.GotoPos(pos)		
	
	def OnConfigurePlugins(self, event):
		from drPluginDialog import drPluginDialog
		d = drPluginDialog(self)
		d.Show()
	
	def OnCustomizePopUpMenu(self, event):
		from drPopUpMenuDialog import drPopUpMenuDialog
		d = drPopUpMenuDialog(self)
		d.ShowModal()
		d.Destroy()
	
	def OnCustomizeToolBar(self, event):
		from drToolBarDialog import drToolBarDialog
		d = drToolBarDialog(self)
		d.ShowModal()
		d.Destroy()		
	
	def OnCustomizeShortcuts(self, event):
		from drShortcutsDialog import drShortcutsDialog
		d = drShortcutsDialog(self)
		d.ShowModal()
		d.Destroy()
	
	def OnDedentRegion(self, event):
		#Submitted Patch:  Christian Daven
		self.txtDocument.BackTab()

	def OnEditBookmarks(self, event):	
		from drBookmarksDialog import drBookmarksDialog
		d = drBookmarksDialog(self, (self.homedirectory + "/bookmarks.dat"))
		d.ShowModal()
		d.Destroy()
		self.bookmarksmenu.reloadBookmarks()

	def OnEditScriptMenu(self, event):
		from drScriptDialog import drScriptDialog
		d = drScriptDialog(self)
		d.ShowModal()
		d.Destroy()
		self.drscriptmenu.reloadscripts()
	
	def OnEncodingASCII(self, event):
		self.txtDocument.locale = 1
		self.txtPrompt.locale = 1
		idnum = self.ID_ENCODING_ASCII
		self.encodingmenu.Check(idnum, True)
	
	def OnEncodingCustom(self, event):
		self.txtDocument.locale = 3
		self.txtPrompt.locale = 3
		idnum = self.ID_ENCODING_CUSTOM
		self.encodingmenu.Check(idnum, True)
	
	def OnEncodingDefault(self, event):
		self.txtDocument.locale = 0
		self.txtPrompt.locale = 0
		idnum = self.ID_ENCODING_DEFAULT
		self.encodingmenu.Check(idnum, True)
		
	def OnEncodingUTF8(self, event):
		self.txtDocument.locale = 2
		self.txtPrompt.locale = 2
		idnum = self.ID_ENCODING_UTF_8
		self.encodingmenu.Check(idnum, True)
				
	def OnEnd(self, event):
		if (self.txtPrompt.pid is not -1):
			self.UpdateMenuAndToolbar()
			wx.Process_Kill(self.txtPrompt.pid, wx.SIGKILL)
			self.txtPrompt.SetReadOnly(1)

	def OnExit(self, event):
		self.Close(False)
	
	def OnExpandAll(self, event):
		if (self.prefs.docfolding):
			wx.BeginBusyCursor()
			self.txtDocument.FoldAll(True)
			wx.EndBusyCursor()
	
	def OnFindAndComplete(self, event):
		#Get The Current Word
		text = self.txtDocument.GetText()
		pos = self.txtDocument.GetCurrentPos()	
		repre = re.compile("\(|\)|\[|\]|\{|\}|\<|\>", re.IGNORECASE | re.MULTILINE)
		regex = re.compile("\S*\Z", re.IGNORECASE | re.MULTILINE)
		#franz: regexend is not used
		eol = self.txtDocument.GetEndOfLineCharacter()

		#Get the left bit
		i = text[0:pos].rfind(eol)
		if i == -1:
			i = 0
		else:
			i = i + len(eol)

		#Check for characters to stop at.
		t = re.finditer(repre, text[i:pos])
		if t is not None:
			try:
				preresult = t.next()
			except:
				preresult = None
			try:
				while t.next() is not None:					
					preresult = t.next()
			except:
				if preresult is not None:
					t = i + preresult.start() + 1
					#If t == pos, then you do not want to stop
					#at the character.
					if t < pos:
						i = t
		
		#Find Non Whitespace Characters.
		result = regex.search(text[i:pos])
		
		if result == None:
			start = i
		else:
			start = i + result.start()
		
		#Handle special characters
		oword = text[start:pos]
		word = oword.replace('\\', "\\\\").replace('^', "\\^").replace('*', "\\*").replace('$', "\\$")
		word = word.replace('+', "\\+").replace('?', "\\?").replace('{', "\\{").replace('}', "\\}")
		word = word.replace('[', "\\[").replace(']', "\\]").replace('(', "\\(").replace(')', "\\)")
		word = word.replace('.', "\\.").replace('|', "\\|").replace('<', "\\<").replace('>', "\\>")
						
		#Find all matches in the document.
		findandcompleteregex = re.compile(r"\b" + word + r"\S*\b", re.MULTILINE)
		r = findandcompleteregex.findall(text)
		r.sort()
		results = ""
		tr = []
		for item in r:
			try:
				tr.index(item)
			except:
				if not item == oword:
					results = results + " " + item
					tr.append(item)
		
		results = results.strip()
							
		if len(tr) > 0:
			try:
				self.txtDocument.AutoCompShow(len(oword), results)
			except:
				#What is this mess?
				pass
			
	def OnFoldAll(self, event):
		if (self.prefs.docfolding):
			wx.BeginBusyCursor()
			self.txtDocument.FoldAll(False)
			wx.EndBusyCursor()
				
	def OnFormatMacMode(self, event):
		wx.BeginBusyCursor()
		wx.Yield()
		self.txtDocument.SetEOLMode(wx.stc.STC_EOL_CR)
		self.txtDocument.ConvertEOLs(wx.stc.STC_EOL_CR)
		self.txtDocument.lineendingsaremixed = 0
		self.txtDocument.OnPositionChanged(event)
		wx.EndBusyCursor()
	
	def OnFormatUnixMode(self, event):
		wx.BeginBusyCursor()
		wx.Yield()
		self.txtDocument.SetEOLMode(wx.stc.STC_EOL_LF)
		self.txtDocument.ConvertEOLs(wx.stc.STC_EOL_LF)
		self.txtDocument.lineendingsaremixed = 0
		self.txtDocument.OnPositionChanged(event)
		wx.EndBusyCursor()
		
	def OnFormatWinMode(self, event):
		wx.BeginBusyCursor()
		wx.Yield()
		self.txtDocument.SetEOLMode(wx.stc.STC_EOL_CRLF)
		self.txtDocument.ConvertEOLs(wx.stc.STC_EOL_CRLF)
		self.txtDocument.lineendingsaremixed = 0
		self.txtDocument.OnPositionChanged(event)
		wx.EndBusyCursor()
	
	def OnGoTo(self, event):
		d = wx.TextEntryDialog(self, "Go To Line Number:", "DrPython - Go To", "")
		answer = d.ShowModal()
		v = d.GetValue()
		d.Destroy()
		if (answer == wx.ID_OK):
			try:
				v = int(v) - 1
				if (v >= 0) and (v < self.txtDocument.GetLineCount()):
					if self.prefs.docfolding:
						self.txtDocument.EnsureVisible(v)
					self.txtDocument.ScrollToLine(v)
					self.txtDocument.GotoLine(v)
				else:
					e = drScrolledMessageDialog.ScrolledMessageDialog(self, "That line does not exist", "DrPython Error")
					e.ShowModal()
					e.Destroy()
			except StandardError:
				e = drScrolledMessageDialog.ScrolledMessageDialog(self, "You must enter an integer (1, 2, 3, etc)", "DrPython Error")
				e.ShowModal()
				e.Destroy()

	def OnIdle(self, event):
		self.txtPrompt.OnIdle(event)

	def OnIndentRegion(self, event):
		#Submitted Patch:  Christian Daven
		self.txtDocument.Tab()
	
	def OnInsertRegEx(self, event):
		from drRegularExpressionDialog import drRegularExpressionDialog
		d = drRegularExpressionDialog(self, -1, "Insert Regular Expression", self.txtPrompt.GetSTCFocus())
		d.Show()
		
	def OnKeyDown(self, event):	
		self.RunShortcuts(event)

		event.Skip()
	
	def OnLowercase(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			self.txtPrompt.CmdKeyExecute(wx.stc.STC_CMD_LOWERCASE)
		else:
			self.txtDocument.CmdKeyExecute(wx.stc.STC_CMD_LOWERCASE)

	def OnMenuFind(self, event):
		d = drFindReplaceDialog(self, -1, "Find")
		d.SetOptions(self.FindOptions)
		if (self.txtDocument.GetSelectionStart() < self.txtDocument.GetSelectionEnd()):
			d.SetFindString(self.txtDocument.GetSelectedText())
		elif self.prefs.findreplaceundercursor:
			pos = self.txtDocument.GetCurrentPos()
			d.SetFindString(self.txtDocument.GetTextRange(self.txtDocument.WordStartPosition(pos, 1), self.txtDocument.WordEndPosition(pos, 1)))
		d.Show(True)
	
	def OnMenuFindNext(self, event):
		self.Finder.DoFindNext()
	
	def OnMenuFindPrevious(self, event):
		self.Finder.DoFindPrevious()
		
	def OnMenuReplace(self, event):	
		d = drFindReplaceDialog(self, -1, "Replace", 1)
		d.SetOptions(self.ReplaceOptions)
		if (self.txtDocument.GetSelectionStart() < self.txtDocument.GetSelectionEnd()):
			d.SetFindString(self.txtDocument.GetTextRange(self.txtDocument.GetSelectionStart(), self.txtDocument.GetSelectionEnd()))
		else:
			d.SetFindString(self.Finder.GetFindText())
		d.Show(True)
			
	def OnNew(self, event):
		nextpage = drPanel(self.mdinotebook, self.ID_APP)
		self.sizertarget = nextpage
		unumbers = map(lambda x: x.untitlednumber, self.txtDocumentArray)
		unumbers.sort()
		x = 0
		last = 0
		l = len(self.txtDocumentArray)
		while x < l:
			if unumbers[x] > 0:
				if unumbers[x] != (last + 1):
					x = l
				else:
					last = unumbers[x]
					x = x + 1
			else:
				x = x + 1	
		last = last + 1
		self.mdinotebook.AddPage(nextpage, "Untitled " + str(last))
		self.txtDocumentArray.append(DrText(nextpage.top, self.ID_APP, self))
		self.txtPromptArray.append(DrPrompt(nextpage.bottom, self.ID_APP, self))
		self.txtDocumentArray[l].untitlednumber = last
		#copy old finder limodou 2004/04/19
		self.FinderArray.append(drFinder(self, self.Finder))
		#end limodou
		self.lastprogargsArray.append("")
		self.setDocumentTo(l)		
		self.mdinotebook.SetPageImage(l, 0)
		self.txtDocument.SetTargetPosition(l)
		#utf-8 auto detecting limodou 2004/04/16
		self.txtDocument.locale = self.prefs.encoding
		self.txtPrompt.local = self.prefs.encoding
		self.encodingmenu.Check(self.ID_ENCODING_DEFAULT + self.txtDocument.locale, True)
		#end limodou
		
		if self.prefs.sourcebrowserisvisible:
			self.ShowSourceBrowser()
		
		self.reloaddocumentsmenu()
		
#BUG WORKAROUND:
		if self.PLATFORM_IS_WIN:
			w, h = self.currentpage.GetSizeTuple()
			self.currentpage.SetSize(wx.Size(w+1, h+1))
			self.currentpage.SetSize(wx.Size(w, h))
#END BUG WORKAROUND
		
	def OnOpen(self, event):
		dlg = drFileDialog.FileDialog(self, "Open", self.wildcard, MultipleSelection=True)
		if (len(self.ddirectory) > 0):
			try:
				dlg.SetDirectory(self.ddirectory)
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Setting Default Directory To: " + self.ddirectory), "DrPython Error")
				d.ShowModal()
				d.Destroy()
		if (dlg.ShowModal() == wx.ID_OK):
			alreadyopen = map(lambda x: x.filename, self.txtDocumentArray)
			old = self.txtDocument.filename
			filenames = dlg.GetPaths()
			filenames = map(lambda x: x.replace("\\", '/'), filenames)
			l = len(filenames)
			c = 0
			while c < l:
				try:
					#franz: pychecker: i is not referenced
					#Franz Fixed the fix.
					alreadyopen.index(filenames[c])	
					d = drScrolledMessageDialog.ScrolledMessageDialog(self, ('"' + filenames[c] + '" is already open.'), "Already Open")
					d.ShowModal()
					d.Destroy()
					filenames.pop(c)
					c = c - 1
					l = l - 1
				except:
					pass
				c = c + 1
			if l < 1:
				return
			x = 1
			self.DestroyRecentFileMenu()
			if l > 1:
				self.OpenFile(filenames[0], len(old) > 0)
			while (x < l):
				self.OpenFile(filenames[x], True)
				x = x + 1
			if l <= 1:
				if (len(old) > 0) or (self.txtDocument.GetModify()):					
					self.OpenFile(filenames[0], True)
				else:
					self.OpenFile(filenames[0], False)
			self.CreateRecentFileMenu()
			dlg.Destroy()

	def OnOpenRecentFile(self, event):
		recentmenuindex = event.GetId() - self.ID_RECENT_FILES_BASE
		alreadyopen = map(lambda x: x.filename, self.txtDocumentArray)
		try:
			#franz: pychecker: i is not referenced
			#Franz fixed the fix.
			alreadyopen.index(self.recentfiles[recentmenuindex])
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ('"' + self.recentfiles[recentmenuindex] + '" is already open.'), "Already Open")
			d.ShowModal()
			d.Destroy()
			return
		except:
			pass
		if not os.path.exists(self.recentfiles[recentmenuindex]):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ('"' + self.recentfiles[recentmenuindex] + '" Does Not Exist.'), "Recent File No Longer Exists")
			d.ShowModal()
			d.Destroy()
			return
		old = self.txtDocument.filename
		filename = self.recentfiles[recentmenuindex]
		self.DestroyRecentFileMenu()
		if (len(old) > 0) or (self.txtDocument.GetModify()):
			self.OpenFile(filename, True)
		else:
			self.OpenFile(filename, False)
		self.CreateRecentFileMenu()

	def OnPrefs(self, event):
		from drPrefsDialog import drPrefsDialog
		d = drPrefsDialog(self, -1, "DrPython - Preferences")
		d.ShowModal()		
		d.Destroy()
	
	def OnPrint(self, event):
		self.Printer.Print(self.txtDocument.GetText(), self.txtDocument.filename, self.prefs.printdoclinenumbers)

	def OnPrintSetup(self, event):
		self.Printer.PrinterSetup()
				
	def OnPrintPrompt(self, event):
		self.Printer.Print(self.txtPrompt.GetText(), self.txtDocument.filename, self.prefs.printpromptlinenumbers)
	
	def OnProcessEnded(self, event):
		#First, check for any leftover output.
		self.txtPrompt.OnIdle(event)
		
		#Set the process info to the correct position in the array.
		i = 0
		epid = event.GetPid()
		try:
			i = map(lambda tprompt: tprompt.pid == epid, self.txtPromptArray).index(True)
		except:
			return				
		#If this is the process for the current window:
		if self.docPosition == i:
			self.txtPrompt.process.Destroy()
			self.txtPrompt.process = None
			self.txtPrompt.pid = -1
			self.txtPrompt.SetReadOnly(1)
			self.txtPrompt.pythonintepreter = 0
			self.UpdateMenuAndToolbar()
			self.SetStatusText("", 2)
		else:	
			self.txtPromptArray[i].process.Destroy()
			self.txtPromptArray[i].process = None
			self.txtPromptArray[i].pid = -1
			self.txtPromptArray[i].SetReadOnly(1)
			self.txtPromptArray[i].pythonintepreter = 0		
		
	def OnPython(self, event):
		self.ExecutePython("-i")

	def OnRedo(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			self.txtPrompt.Redo()
		else:
			self.txtDocument.Redo()

	def OnReload(self, event):
		d = wx.MessageDialog(self, "This will reload the current file.\nAny changes will be lost.\nAre you sure you want to do this?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_YES):
			if (len(self.txtDocument.filename) > 0):
				self.OpenFile(self.txtDocument.filename, False)
		event.Skip()
				
	def OnRestoreFromBackup(self, event):
		if os.path.exists(self.txtDocument.filename + ".bak"):
			d = wx.MessageDialog(self, "This will restore the current file from the last backup.\nAny changes will be lost.\nAre you sure you want to do this?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_YES):
				if (len(self.txtDocument.filename) > 0):
					old = self.txtDocument.filename
					filename = self.txtDocument.filename + ".bak"
					self.OpenFile(filename, False, False, False)
					self.txtDocument.filename = old
		else:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("No Backup File For: \"" + self.txtDocument.filename + "\""), "DrPython Error")
			d.ShowModal()
			d.Destroy()		
		
	def OnRun(self, event):
		if (self.txtDocument.GetModify()):
			d = wx.MessageDialog(self, "The file has been modified.\nIt would be wise to save before running.\nWould you like to save the file?", "DrPython", wx.YES_NO | wx.ICON_QUESTION)
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_YES):
				self.OnSave(event)			
		if len(self.txtDocument.filename) > 0:
			cdir, filen = os.path.split(self.txtDocument.filename)
			try:
				os.chdir(cdir)
			except:		
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Error Setting current directory for Python.", "DrPython RunError")
				d.ShowModal()
				d.Destroy()				
			largs = ""
			if (len(self.lastprogargs) > 0):
				largs = "\" " + self.lastprogargs
			if self.PLATFORM_IS_WIN:
				self.runcommand((self.pythexecw + " -u " +  self.prefs.pythonargs + " \"" + self.txtDocument.filename.replace("\\", "/") + largs), "Running " + filen)
			else:
				self.runcommand((self.pythexec + " -u " +  self.prefs.pythonargs + " \"" + self.txtDocument.filename + largs), "Running " + filen)
		
	def OnSave(self, event):
		if (len(self.txtDocument.filename) <= 0):
			self.OnSaveAs(event)
		else:
			self.SaveFile(False)
	
	def OnSaveAll(self, event):
		l = len(self.txtDocumentArray)
		x = 0
		while x < l:
			self.setDocumentTo(x)
			self.OnSave(None)
			x = x + 1

	def OnSaveAs(self, event):		
		dlg = drFileDialog.FileDialog(self, "Save File As", self.wildcard, IsASaveDialog=True)
		if (len(self.ddirectory) > 0):
			try:
				dlg.SetDirectory(self.ddirectory)			
			except:
				#franz: ddirectory
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Setting Default Directory To: " + self.ddirectory), "DrPython Error")
				d.ShowModal()
				d.Destroy()
		if (dlg.ShowModal() == wx.ID_OK):
			old = self.txtDocument.filename
			if self.txtDocument.untitlednumber > 0:
				self.txtDocument.untitlednumber = -1
			self.txtDocument.filename = dlg.GetPath().replace("\\", "/")
			self.ddirectory = os.path.dirname(self.txtDocument.filename)
			self.DestroyRecentFileMenu()
			self.SaveFile(not (old == self.txtDocument.filename))
			self.UpdateMenuAndToolbar()
			
			#Highlighting
			if not self.prefs.doconlyusedefaultsyntaxhighlighting:
				if self.checkiffileisPython(self.txtDocument.filename):
					#Python
					self.txtDocument.currentlanguage = 0
					self.highlightmenu.Check(self.ID_HIGHLIGHT_PYTHON, True)
				elif self.checkiffileisCPP(self.txtDocument.filename):
					#C/C++
					self.txtDocument.currentlanguage = 1
					self.highlightmenu.Check(self.ID_HIGHLIGHT_CPP, True)
				elif self.checkiffileisHTML(self.txtDocument.filename):
					#HTML
					self.txtDocument.currentlanguage = 2			
					self.highlightmenu.Check(self.ID_HIGHLIGHT_HTML, True)	
				elif self.checkiffileisPlainText(self.txtDocument.filename):
					#Plain Text
					self.txtDocument.currentlanguage = 3
					self.highlightmenu.Check(self.ID_HIGHLIGHT_PLAIN_TEXT, True)
				else:
					#Default
					pass
				self.txtDocument.SetupPrefsDocument()	
			
			#Update Recent Files
			if (self.recentfiles.count(self.txtDocument.filename) is not 0):
				self.recentfiles.remove(self.txtDocument.filename)
			if (len(self.recentfiles) is self.prefs.recentfileslimit):
				self.recentfiles.pop()
			self.recentfiles.insert(0, self.txtDocument.filename)
			self.WriteRecentFiles()
			self.CreateRecentFileMenu()
			dlg.Destroy()
			self.reloaddocumentsmenu()
			#Refreshes the tab.
			x, y = self.mdinotebook.GetSizeTuple()
			self.mdinotebook.SetSize(wx.Size(x+1, y+1))
			self.mdinotebook.SetSize(wx.Size(x, y))

	def OnSavePrompt(self, event):
		dlg = wx.FileDialog(self, "Save Prompt Text To", "", "", "Text File (*.txt)|*.txt|All files (*)|*", wx.SAVE|wx.OVERWRITE_PROMPT)
		if (len(self.ddirectory) > 0):
			try:
				dlg.SetDirectory(self.ddirectory)			
			except:
				#franz: ddirectory
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Setting Default Directory To: " + self.ddirectory), "DrPython Error")
				d.ShowModal()
				d.Destroy()	
		if (dlg.ShowModal() == wx.ID_OK):	
			pfilename = dlg.GetPath().replace("\\", "/")
			self.ddirectory = os.path.dirname(pfilename)
			try:
				cfile = file(pfilename, 'wb')
				#comment limodou 2004/04/13
				cfile.write(self.GetEncodedText(self.txtPrompt))
				#end limodou
				cfile.close()
			except IOError:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Writing: " + pfilename), "DrPython Error")
				d.ShowModal()
				d.Destroy()   		
			dlg.Destroy()

	def OnSelectAll(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			self.txtPrompt.SelectAll()
		else:
			self.txtDocument.SelectAll()
	
	def OnSelectDocument(self, event):
		eid = event.GetId()
		i = eid - self.ID_DOCUMENTS_BASE
		self.mdinotebook.SetSelection(i)
		self.mdinotebook.SetTab()

	def OnSelectDocumentFirst(self, event):
		self.mdinotebook.SetSelection(0)
		self.mdinotebook.SetTab()

	def OnSelectDocumentLast(self, event):
		self.mdinotebook.SetSelection(self.mdinotebook.GetPageCount()-1)
		self.mdinotebook.SetTab()
		
	def OnSelectDocumentNext(self, event):
		self.mdinotebook.AdvanceSelection(True)
		self.mdinotebook.SetTab()

	def OnSelectDocumentPrevious(self, event):
		self.mdinotebook.AdvanceSelection(False)
		self.mdinotebook.SetTab()
			
	def OnSelectNone(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			self.txtPrompt.SetSelection(0, 0)
		else:
			self.txtDocument.SetSelection(0, 0)

	def OnSetArgs(self, event):
		if (len(self.txtDocument.filename) > 0):
			d = wx.TextEntryDialog(self, "Arguments:", "DrPython - Set Arguments", self.lastprogargs)
			if (d.ShowModal() == wx.ID_OK):
				self.lastprogargs = d.GetValue()
				self.lastprogargsArray[self.docPosition] = self.lastprogargs
		d.Destroy()
	
	def OnSyntaxHighlightingPython(self, event):
		self.txtDocument.currentlanguage = 0
		self.txtDocument.SetupPrefsDocument()
		
	def OnSyntaxHighlightingCPP(self, event):
		self.txtDocument.currentlanguage = 1
		self.txtDocument.SetupPrefsDocument()
		
	def OnSyntaxHighlightingHTML(self, event):
		self.txtDocument.currentlanguage = 2
		self.txtDocument.SetupPrefsDocument()

	def OnSyntaxHighlightingText(self, event):
		self.txtDocument.currentlanguage = 3
		self.txtDocument.SetupPrefsDocument()					
						
	def OnTogglePrompt(self, event):
		if (self.currentpage.PromptIsVisible):
			self.currentpage.PromptIsVisible = False
			if self.hasToolBar:
				self.toolbar.ToggleTool(self.ID_TOGGLE_PROMPT,  False)	
			self.currentpage.OnSize(None)
			self.txtDocument.SetFocus()
		else:
			self.currentpage.PromptIsVisible = True
			if self.hasToolBar:
				self.toolbar.ToggleTool(self.ID_TOGGLE_PROMPT,  True)
			self.currentpage.OnSize(None)
			self.txtPrompt.SetFocus()
	
	def OnToggleSourceBrowser(self, event):
		if self.txtDocument.SourceBrowser is None:
			target = self.currentpage.GetTargetSashWindow(self.prefs.sourcebrowserpane == 0)
			self.txtDocument.SourceBrowser = drSourceBrowserPanel(target, -1, (self.prefs.sourcebrowserpane == 0))
			self.currentpage.ClearPanel(self.prefs.sourcebrowserpane == 0)
			self.currentpage.AppendPanel(self.txtDocument.SourceBrowser, (self.prefs.sourcebrowserpane == 0))
			self.currentpage.OnSize(event)
		else:
			if ((self.prefs.sourcebrowserpane == 0) and not self.currentpage.LeftIsVisible) or \
			((self.prefs.sourcebrowserpane == 1) and not self.currentpage.RightIsVisible):
				self.txtDocument.SourceBrowser.Browse()
			if (self.prefs.sourcebrowserpane == 0):
				self.currentpage.LeftIsVisible = not self.currentpage.LeftIsVisible
			else:
				self.currentpage.RightIsVisible = not self.currentpage.RightIsVisible
			self.currentpage.OnSize(event)
			
	def OnToggleViewWhiteSpace(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			c = self.txtPrompt.GetViewWhiteSpace()
			self.txtPrompt.SetViewWhiteSpace(not c)
			self.txtPrompt.SetViewEOL(not c)
		else:
			c = self.txtDocument.GetViewWhiteSpace()
			self.txtDocument.SetViewWhiteSpace(not c)
			self.txtDocument.SetViewEOL(not c)
	
	def OnToolBar(self, event):
		try:
			i = event.GetId() - self.ID_OTHER
			txt = self.ToolBarList[i]
			if txt in self.stcshortcutlist:
				pos = self.stcshortcutlist.index(txt)
				if (self.txtPrompt.GetSTCFocus()):
					self.txtPrompt.CmdKeyExecute(self.txtPrompt.STCCOMMANDLIST[pos])
				else:
					self.txtDocument.CmdKeyExecute(self.txtDocument.STCCOMMANDLIST[pos])
			else:
				if txt in self.PluginToolBarLabels:				
					pos = self.PluginToolBarLabels.index(txt)
					self.PluginToolBarFunctions[pos](event)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "ToolBar Action Error", "DrPython Error")
			d.ShowModal()
			d.Destroy()			
				
	def OnUnCommentRegion(self, event):	
		#franz: pos is not used
		selstart, selend = self.txtDocument.GetSelection()
		#From the start of the first line selected
		startline = self.txtDocument.LineFromPosition(selstart)
		self.txtDocument.GotoLine(startline)
		start = self.txtDocument.GetCurrentPos()
		#To the end of the last line selected
		#Bugfix Chris Wilson
		end = self.txtDocument.GetLineEndPosition(self.txtDocument.LineFromPosition(selend-1))
		#End Bugfix Chris Wilson
		self.txtDocument.SetSelection(start, end)
		text = self.txtDocument.GetSelectedText()
		
		x = 1
		l = len(text)
		newtext = ""
		if not (text[0] == '#'):
			newtext = text[0]
		while (x < l):
			if (text[x] == '#'):
				prestyle = self.txtDocument.GetStyleAt(start + (x - 1))
				style = self.txtDocument.GetStyleAt(start + x)
				if not ((not (prestyle == wx.stc.STC_P_COMMENTLINE) and not (prestyle == wx.stc.STC_P_COMMENTBLOCK)) and ((style == wx.stc.STC_P_COMMENTLINE) or (style == wx.stc.STC_P_COMMENTBLOCK))):
					newtext = newtext + text[x]
			else:
				newtext = newtext + text[x]
			x = x + 1
		self.txtDocument.ReplaceSelection(newtext)
	
	def OnUndo(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			self.txtPrompt.Undo()
		else:
			self.txtDocument.Undo()
	
	def OnUppercase(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			self.txtPrompt.CmdKeyExecute(wx.stc.STC_CMD_UPPERCASE)
		else:
			self.txtDocument.CmdKeyExecute(wx.stc.STC_CMD_UPPERCASE)
			
	def OnViewAbout(self, event):
		wx.Execute((self.prefs.documentationbrowser + ' "file:///' + self.programdirectory + '/documentation/about.html"'), wx.EXEC_ASYNC)			
				
	def OnViewHelp(self, event):
		wx.Execute((self.prefs.documentationbrowser + ' "file:///' + self.programdirectory + '/documentation/help.html"'), wx.EXEC_ASYNC)			
			
	def OnViewPythonDocs(self, event):
		wx.Execute((self.prefs.documentationbrowser + ' "' + self.prefs.documentationpythonlocation + '"'), wx.EXEC_ASYNC)
	
	def OnViewREHowtoDocs(self, event):
		wx.Execute((self.prefs.documentationbrowser + ' "' + self.prefs.documentationrehowtolocation + '"'), wx.EXEC_ASYNC)
	
	def OnViewWxWidgetsDocs(self, event):
		wx.Execute((self.prefs.documentationbrowser + ' "' + self.prefs.documentationwxwidgetslocation + '"'), wx.EXEC_ASYNC)
	
	def OnZoomIn(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			zoom = self.txtPrompt.GetZoom()
			if (zoom < 20):
				self.txtPrompt.SetZoom(zoom + 1)
		else:
			zoom = self.txtDocument.GetZoom()
			if (zoom < 20):
				self.txtDocument.SetZoom(zoom + 1)
	
	def OnZoomOut(self, event):
		if (self.txtPrompt.GetSTCFocus()):
			zoom = self.txtPrompt.GetZoom()
			if (zoom > -9):
				self.txtPrompt.SetZoom(zoom - 1)
		else:
			zoom = self.txtDocument.GetZoom()
			if (zoom > -9):
				self.txtDocument.SetZoom(zoom - 1)
	
	def OpenFile(self, filename, OpenInNewWindow, editrecentfiles = True, setTitle = True):		
		wx.BeginBusyCursor()
		if self.txtDocument.untitlednumber > 0:
			self.txtDocument.untitlednumber = -1
		filename = os.path.abspath(filename).replace("\\", '/')
		if (not os.path.exists(filename)):
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Opening: " + filename), "DrPython Error")
			d.ShowModal()
			d.Destroy()
			wx.EndBusyCursor()	
			return
		if (editrecentfiles):
			if (self.recentfiles.count(filename) is not 0):			
				self.recentfiles.remove(filename)
			if (len(self.recentfiles) is self.prefs.recentfileslimit):
				self.recentfiles.pop()
			self.recentfiles.insert(0, filename)
			self.WriteRecentFiles()
		if (((not (self.txtDocument.filename == filename))) and (self.txtDocument.GetModify())) or OpenInNewWindow:
			nextpage = drPanel(self.mdinotebook, self.ID_APP)
			self.sizertarget = nextpage
			self.mdinotebook.AddPage(nextpage, "Untitled")			
			l = len(self.txtDocumentArray)
			self.txtDocumentArray.append(DrText(nextpage.top, self.ID_APP, self))
			self.txtPromptArray.append(DrPrompt(nextpage.bottom, self.ID_APP, self))				
			self.txtDocumentArray[l].filename = filename
			self.txtDocumentArray[l].SetTargetPosition(l)
			#copy old finder limodou 2004/04/19
			self.FinderArray.append(drFinder(self, self.Finder))
			#end limodou
			self.lastprogargsArray.append("")
			self.setDocumentTo(l, True)
			#utf-8 auto detecting limodou 2004/04/16
			self.txtDocument.locale = self.prefs.encoding
			self.txtPrompt.local = self.prefs.encoding
			#end limodou
#BUG WORKAROUND:
			if self.PLATFORM_IS_WIN:
				w, h = self.currentpage.GetSizeTuple()
				self.currentpage.SetSize(wx.Size(w+1, h+1))
				self.currentpage.SetSize(wx.Size(w, h))
#END BUG WORKAROUND
		else:
			self.txtDocumentArray[self.docPosition].filename = filename
		try:		
			cfile = file(filename, 'rb')
			oof = cfile.read()
			if not self.prefs.doconlyusedefaultsyntaxhighlighting:
				if self.checkiffileisPython(filename):
					#Python
					self.txtDocument.currentlanguage = 0
					self.highlightmenu.Check(self.ID_HIGHLIGHT_PYTHON, True)
				elif self.checkiffileisCPP(filename):
					#C/C++
					self.txtDocument.currentlanguage = 1
					self.highlightmenu.Check(self.ID_HIGHLIGHT_CPP, True)
				elif self.checkiffileisHTML(filename):
					#HTML
					self.txtDocument.currentlanguage = 2			
					self.highlightmenu.Check(self.ID_HIGHLIGHT_HTML, True)	
				elif self.checkiffileisPlainText(filename):
					#Plain Text
					self.txtDocument.currentlanguage = 3
					self.highlightmenu.Check(self.ID_HIGHLIGHT_PLAIN_TEXT, True)
				else:
					#Default
					pass
				self.txtDocument.SetupPrefsDocument()
			#utf-8 auto detect limodou 2004/04/16
			#franz: pychecker: comparision against true superfluos
			if self.prefs.autodetectutf8encoding and drDetectUTF_8.utf8Detect(oof):
				self.SetEncodedText(self.txtDocument, oof, 2)
				#utf-8 detect limodou 2004/04/16
				self.txtDocument.locale = 2
				self.txtPrompt.locale = 2
				#end
			else:
				self.SetEncodedText(self.txtDocument, oof)
			#end limodou
			self.txtDocument.EmptyUndoBuffer()
			self.txtDocument.SetSavePoint()
			cfile.close()
			
			if setTitle:
				self.mdinotebook.SetPageText(self.docPosition, os.path.split(filename)[1])
				self.mdinotebook.SetPageImage(self.docPosition, 0)
			
			self.txtDocument.SetScrollWidth(1)
			
			#Check to see if scroll length is appropriate:
			
			#Encoding add limodou 2004/04/16
			self.encodingmenu.Check(self.ID_ENCODING_DEFAULT + self.txtDocument.locale, True)
			#endif 
			
			counter = 0
			length = self.txtDocument.GetLineCount()
			ll = self.txtDocument.TextWidth(wx.stc.STC_STYLE_DEFAULT, "OOO")
			x = 0
			spaces = "\t".expandtabs(self.prefs.tabwidth)
			current_width = self.txtDocument.GetScrollWidth()
			while (counter < length):					
				line = self.txtDocument.GetLine(counter).replace('\t', spaces)
				actual_width = self.txtDocument.TextWidth(wx.stc.STC_STYLE_DEFAULT, line)
				if (current_width < actual_width):
					current_width = actual_width + ll
				counter = counter + 1
			self.txtDocument.SetScrollWidth(current_width)
									
			self.txtDocument.SetXOffset(0)				
			
			self.UpdateMenuAndToolbar()
			
			if setTitle:
				self.SetTitle("DrPython - " + filename)
			
			eollist = "".join(map(getEndOfLineCharacter, oof))
			
			self.txtDocument.lineendingsaremixed = 0
			
			if eollist.count('\r\n'):
				emodenum = 1			
			elif eollist.count('\n'):
				emodenum = 0
			elif eollist.count('\r'):
				emodenum = 2
			else:
				emodenum = self.prefs.eolmode
			if len(eollist) >= 2:
				win = eollist.count('\r\n')
				unix = eollist.count('\n')
				mac = eollist.count('\r')
				
				if (win and not (unix == mac)) or \
				((not win) and (unix and mac)):
					self.txtDocument.lineendingsaremixed = 1				
								
			dmodenum = self.prefs.eolmode
																																	
			if (self.prefs.checkeol):
				if (not emodenum == self.prefs.eolmode):
					if self.txtDocument.lineendingsaremixed:
						d = wx.MessageDialog(self, (filename + " is currently "\
+ self.FFMESSAGE[emodenum] + "(Mixed).\nWould you like to change it to the default?\n(Since the file is mixed, \
this is highly recommended.\nThe Default is: " + self.FFMESSAGE[dmodenum]), "Mixed Line Ending", wx.YES_NO | wx.ICON_QUESTION)
					else:
						d = wx.MessageDialog(self, (filename + " is currently " + self.FFMESSAGE[emodenum] + ".\nWould you like to change it to the default?  The Default is: " + self.FFMESSAGE[dmodenum]), "Line Ending", wx.YES_NO | wx.ICON_QUESTION)					
					answer = d.ShowModal()
					d.Destroy()
					if (answer == wx.ID_YES):
						#Bugfix, Thanks Stephen Anderson.
						if (self.prefs.eolmode == 1):
							emode = wx.stc.STC_EOL_CRLF
						elif (self.prefs.eolmode == 2):
							emode = wx.stc.STC_EOL_CR
						else:
							emode = wx.stc.STC_EOL_LF
						self.txtDocument.SetEOLMode(emode)
						self.txtDocument.ConvertEOLs(emode)
						self.txtDocument.lineendingsaremixed = 0						
						emodenum = dmodenum
			if emodenum == 1:
				emode = wx.stc.STC_EOL_CRLF
			elif emodenum == 2:
				emode = wx.stc.STC_EOL_CR
			else:
				emode = wx.stc.STC_EOL_LF
			self.txtDocument.SetEOLMode(emode)
			if self.txtDocument.lineendingsaremixed:
				eolmodestr = "MIX"
			else:
				emode = self.txtDocument.GetEOLMode()
				if emode == wx.stc.STC_EOL_CR:
					eolmodestr = "MAC"
				elif emode == wx.stc.STC_EOL_CRLF:
					eolmodestr = "WIN"
				else:
					eolmodestr = "UNIX"		
			if self.txtDocument.GetOvertype():
				ovrstring = "OVR"
			else:
				ovrstring = "INS"				
			
			self.txtDocument.OnPositionChanged(None)
			
			#Load SourceBrowser:
			if self.prefs.sourcebrowserisvisible:
				self.ShowSourceBrowser()
				
			if (editrecentfiles):
				self.ddirectory = os.path.dirname(filename)
		except IOError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Opening: " + filename), "DrPython Error")
			d.ShowModal()
			d.Destroy()
				
		#The following chunk of code is an ugly way to work around a bug in wx.STC.
		#As things stand, word wrap may not update on file load.
		#This fixes the problem, by forcing drpython to reset the wx.STC instances.
		if (self.prefs.docwordwrap):
			x, y = self.GetSizeTuple()
			self.SetSize(wx.Size(x+1, y+1))
			self.SetSize(wx.Size(x, y))
		#End of the chunk.
		
		self.reloaddocumentsmenu()
		
		wx.EndBusyCursor()

	def reloaddocumentsmenu(self):
		mnuitems = self.documentsmenu.GetMenuItems()
		num = len(mnuitems)
		x = 0
		while (x < num):
			self.documentsmenu.Remove(mnuitems[x].GetId())
			mnuitems[x].Destroy()
			x = x + 1
			
		self.setupdocumentsmenu()

	def RemovePluginIcon(self, name):
		f = file(self.homedirectory + "/toolbar.custom.icons.dat", 'r')
		lines = f.read().split('\n')
		f.close()		
		name = "<Plugin>:" + name
		f = file(self.homedirectory + "/toolbar.custom.icons.dat", 'w')
		for line in lines:
			if len(line) > 0:
				currentname = drPrefsFile.ExtractPreferenceFromText(line, "Name")
				if currentname != name:
					f.write(line + '\n')
		f.close()
	
	def runcommand(self, command, statustext = "Running Command"):
		if (self.txtPrompt.pid is -1):
			if (not self.currentpage.PromptIsVisible):
				self.currentpage.PromptIsVisible = True
				self.currentpage.OnSize(None)
			self.txtPrompt.SetReadOnly(0)
			self.txtPrompt.ClearAll()
			self.txtPrompt.SetScrollWidth(1)
			self.txtPrompt.editpoint = 0
			self.SetStatusText(statustext, 2)
			self.txtPrompt.process = wx.Process(self)
			self.txtPrompt.process.Redirect()
			if self.PLATFORM_IS_WIN:
				self.txtPrompt.pid = wx.Execute(command, wx.EXEC_ASYNC | wx.EXEC_NOHIDE, self.txtPrompt.process)
			else:
				self.txtPrompt.pid = wx.Execute(command, wx.EXEC_ASYNC, self.txtPrompt.process)					
			self.txtPrompt.inputstream = self.txtPrompt.process.GetInputStream()		
			self.txtPrompt.errorstream = self.txtPrompt.process.GetErrorStream()
			self.txtPrompt.outputstream = self.txtPrompt.process.GetOutputStream()			
			self.UpdateMenuAndToolbar()
			self.txtPrompt.SetFocus()

	def RunShortcuts(self, event, stc = None):
		return drShortcuts.RunShortcuts(self, event, stc)

	def SaveDialogSizeAndPosition(self, dialog, dialogfile):
		if self.prefs.rememberdialogsizesandpositions:
			try:
				f = file(self.homedirectory + '/' + dialogfile, 'w')
				x, y = dialog.GetSizeTuple()
				px, py = dialog.GetPositionTuple()
				f.write(str(x) + '\n' + str(y) + '\n' + str(px) + '\n' + str(py))
				f.close()
			except:				
				d = drScrolledMessageDialog.ScrolledMessageDialog(dialog, "Error Saving Dialog Size", 'Error')
				d.ShowModal()
				d.Destroy()

	def SaveFile(self, IsSaveAs = True):
		try:
			if self.prefs.backupfileonsave and not IsSaveAs:
				try:
					shutil.copyfile(self.txtDocument.filename, self.txtDocument.filename+".bak")
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Backing up to: " + self.txtDocument.filename + ".bak"), "DrPython Error")
					d.ShowModal()
					d.Destroy()					
			cfile = file(self.txtDocument.filename, 'wb')
			#comment limodou 2004/04/16
			cfile.write(self.GetEncodedText(self.txtDocument))
			#end limodou
			cfile.close()
		except IOError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Writing: " + self.txtDocument.filename), "DrPython Error")
			d.ShowModal()
			d.Destroy()
		self.txtDocument.SetSavePoint()
		self.txtDocument.OnModified(None)
				
	def setDocumentTo(self, number, ignoreold = 0):
		prompthasfocus = (self.prefs.promptsize == 100)
		if not ignoreold:
			self.lastprogargsArray[self.docPosition] = self.lastprogargs
			prompthasfocus = self.txtPrompt.GetSTCFocus()
		#copy old finder limodou 2004/04/19
		oldfinder = self.FinderArray[self.docPosition]
		#end limodou
		self.docPosition = number
		self.txtDocument = self.txtDocumentArray[self.docPosition]
		self.txtPrompt = self.txtPromptArray[self.docPosition]
		self.Finder = self.FinderArray[self.docPosition]
		#copy old finder limodou 2004/04/19
		self.Finder.Copy(oldfinder)
		#end limodou
		self.lastprogargs = self.lastprogargsArray[self.docPosition]
		
		self.currentpage = self.mdinotebook.GetPage(number)
		
		if len(self.txtDocument.filename) > 0:
			self.ddirectory = os.path.split(self.txtDocument.filename)[0]
		
		#franz: (Bad Argument).
		self.updatePrefsMDI() 
		
		#Encoding
		self.encodingmenu.Check(self.ID_ENCODING_DEFAULT + self.txtDocument.locale, True)
		
		#Syntax Highlighting
		if self.txtDocument.currentlanguage == 0:
			self.highlightmenu.Check(self.ID_HIGHLIGHT_PYTHON, True)
		if self.txtDocument.currentlanguage == 1:
			self.highlightmenu.Check(self.ID_HIGHLIGHT_CPP, True)
		if self.txtDocument.currentlanguage == 2:
			self.highlightmenu.Check(self.ID_HIGHLIGHT_HTML, True)
		if self.txtDocument.currentlanguage == 3:
			#comment limodou 2004/04/13
			self.highlightmenu.Check(self.ID_HIGHLIGHT_PLAIN_TEXT, True)
			#end limodou
							
		if (self.txtDocument.GetModify()):
			if (len(self.txtDocument.filename) <= 0):
				self.SetTitle("DrPython - Untitled " + str(self.txtDocument.untitlednumber) + '*')
			else:
				self.SetTitle("DrPython - " + self.txtDocument.filename + "*")
		else:
			if (len(self.txtDocument.filename) <= 0):
				self.SetTitle("DrPython - Untitled " + str(self.txtDocument.untitlednumber))
			else:
				self.SetTitle("DrPython - " + self.txtDocument.filename)
			
		if self.txtPromptArray[self.docPosition].pid is not -1:
			if self.txtPrompt.pythonintepreter:
				self.SetStatusText("Running Python Interpreter", 2)
			else:
				self.SetStatusText(("Running " + os.path.split(self.txtDocument.filename)[1]), 2)
		else:
			self.SetStatusText("", 2)		
			
		self.txtDocument.OnPositionChanged(None)
			
		if prompthasfocus:
			self.txtPrompt.SetFocus()
		else:
			self.txtDocument.SetFocus()		
			
		self.mdinotebook.SetSelection(self.docPosition)		

	#SetText function limodou 2004/04/16
	#Edited a tad by drpython 2004/04/13
	#if encoding is None, then use default encoding
	#only support specified encoding or default encoding
	#should support utf-8 encoding auto detect
	def SetEncodedText(self, txtctrl, text, encodingid=0):
		if encodingid == 0:
			encodingid == txtctrl.locale
		if encodingid == 3:
			encoding = self.prefs.customencoding
		else:
			encoding = self.encodings[encodingid]
		if encoding == '<None>':
			encoding = self.encodings[0]
		isUnicode=txtctrl.GetCodePage()!=0
		if isUnicode:
			try:
				sText=unicode(text, encoding)
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Encoding %s\n\nTrying other encoding...") % encoding, "DrPython Error")
				d.ShowModal()
				d.Destroy()
				sText=text
		else:
			#Additions of two try->except statements by drpython 4/22/2004
			#Bug report John Bell
			try:
				#utf-8 auto detect limodou 2004/04/14
				sText=unicode(text, encoding).encode(self.encodings[0])	#convert utf-8 encoding to default encoding
				#set utf-8 encoding in preference
			except:
				#Do Nothing
				sText = text

		txtctrl.SetText(sText)
	#end limodou
	
	def setupdocumentsmenu(self):
		self.tabnavmenu = wx.Menu()
		self.tabnavmenu.Append(self.ID_NEXT_DOCUMENT, "Next Document")
		self.tabnavmenu.Append(self.ID_PREVIOUS_DOCUMENT, "Previous Document")
		self.tabnavmenu.Append(self.ID_FIRST_DOCUMENT, "First Document")
		self.tabnavmenu.Append(self.ID_LAST_DOCUMENT, "Last Document")
		self.documentsmenu.AppendMenu(self.ID_DOCUMENT_NAVIGATION_MENU, "Navigation", self.tabnavmenu)
		self.documentsmenu.AppendSeparator()
		self.documentsmenu.Append(self.ID_SAVE_ALL, "Save All Documents")
		self.documentsmenu.AppendSeparator()
		self.documentsmenu.Append(self.ID_CLOSE_ALL, "Close &All Documents")
		self.documentsmenu.Append(self.ID_CLOSE_ALL_OTHER_DOCUMENTS, "Close All &Other Documents")
		self.documentsmenu.AppendSeparator()
		
		x = 0
		l = len(self.txtDocumentArray)
		if l > 10:
			y = 0
			yl = 10
			if yl > l:
				yl = l
			a = 0
			self.documentsubmenus = []
			while y < yl:		
				self.documentsubmenus.append(wx.Menu())
				self.documentsmenu.AppendMenu(self.ID_DOCUMENTS_MENU_BASE+a, str(y+1) + " - " + str(yl), self.documentsubmenus[a])
				while x < yl:
					if len(self.txtDocumentArray[x].filename) > 0:					
						self.documentsubmenus[a].Append(self.ID_DOCUMENTS_BASE+x, os.path.basename(self.txtDocumentArray[x].filename))
					else:
						self.documentsubmenus[a].Append(self.ID_DOCUMENTS_BASE+x, "Untitled " + str(self.txtDocumentArray[x].untitlednumber))
					self.Bind(wx.EVT_MENU, self.OnSelectDocument, id=self.ID_DOCUMENTS_BASE+x)
					x = x + 1
				if y == l:
					break
				y = y + 10
				yl = yl + 10
				a = a + 1
				if yl > l:
					yl = l
		else:
			while x < l:
				if len(self.txtDocumentArray[x].filename) > 0:					
					self.documentsmenu.Append(self.ID_DOCUMENTS_BASE+x, os.path.basename(self.txtDocumentArray[x].filename))
				else:
					self.documentsmenu.Append(self.ID_DOCUMENTS_BASE+x, "Untitled " + str(self.txtDocumentArray[x].untitlednumber))
				self.Bind(wx.EVT_MENU, self.OnSelectDocument, id=self.ID_DOCUMENTS_BASE+x)
				x = x + 1			
	
	def SetupToolBar(self):
		return drToolBarFile.SetupToolBar(self)
	
	def ShowMessage(self, msg, title='DrPython'):
		d = drScrolledMessageDialog.ScrolledMessageDialog(self, msg, title)
		d.ShowModal()
		d.Destroy()
	
	def ShowSourceBrowser(self):
		if not (self.txtDocument.SourceBrowser):
			target = self.currentpage.GetTargetSashWindow(self.prefs.sourcebrowserpane == 0)
			self.txtDocument.SourceBrowser = drSourceBrowserPanel(target, -1, (self.prefs.sourcebrowserpane == 0))
			self.currentpage.ClearPanel(self.prefs.sourcebrowserpane == 0)
			self.currentpage.AppendPanel(self.txtDocument.SourceBrowser, (self.prefs.sourcebrowserpane == 0))
			self.currentpage.OnSize(None)
		else:
			self.txtDocument.SourceBrowser.Browse()
			if (self.prefs.sourcebrowserpane == 0):
				if not self.currentpage.LeftIsVisible: 
					self.currentpage.LeftIsVisible = True
			else:
				if not self.currentpage.RightIsVisible:
					self.currentpage.RightIsVisible = True
			self.currentpage.OnSize(None)
			
	def ShowPrompt(self, Visible = True):
		if Visible:
			if self.currentpage.PromptIsVisible:
				return
			self.currentpage.PromptIsVisible = True
			if self.hasToolBar:
				self.toolbar.ToggleTool(self.ID_TOGGLE_PROMPT,  True)		
			self.currentpage.OnSize(None)
			self.txtPrompt.SetFocus()
		else:
			if not self.currentpage.PromptIsVisible:
				return
			self.currentpage.PromptIsVisible = False
			if self.hasToolBar:
				self.toolbar.ToggleTool(self.ID_TOGGLE_PROMPT,  False)
			self.currentpage.OnSize(None)
			self.txtDocument.SetFocus()			
	
	def UpdateMenuAndToolbar(self):
		thereisafile = (len(self.txtDocument.filename) > 0)
		
		#Toolbar
		if (self.hasToolBar):
			self.toolbar.EnableTool(self.ID_RELOAD, thereisafile)
			
			if (self.txtPrompt.pid is -1):
				self.toolbar.EnableTool(self.ID_PYTHON, True)
				self.toolbar.EnableTool(self.ID_END, False)
				self.toolbar.EnableTool(self.ID_RUN, thereisafile)
				self.toolbar.EnableTool(self.ID_SET_ARGS, thereisafile)			
			else:
				self.toolbar.EnableTool(self.ID_PYTHON, False)
				self.toolbar.EnableTool(self.ID_END, True)
				if thereisafile:
					self.toolbar.EnableTool(self.ID_RUN, False)
					self.toolbar.EnableTool(self.ID_SET_ARGS, False)
		
		#Menus
		self.filemenu.Enable(self.ID_RELOAD, thereisafile)
		self.filemenu.Enable(self.ID_RESTORE_FROM_BACKUP, thereisafile)
		
		if (self.txtPrompt.pid is -1):
			self.programmenu.Enable(self.ID_PYTHON, True)
			self.programmenu.Enable(self.ID_END, False)
			self.programmenu.Enable(self.ID_RUN, thereisafile)
			self.programmenu.Enable(self.ID_SET_ARGS, thereisafile)				
		else:
			self.programmenu.Enable(self.ID_PYTHON, False)
			self.programmenu.Enable(self.ID_END, True)
			if thereisafile:
				self.programmenu.Enable(self.ID_RUN, False)
				self.programmenu.Enable(self.ID_SET_ARGS, False)	
	
	#franz: oldprefs not used.
	def updatePrefsMDI(self):	
		#Determine What is showing
		self.currentpage.OnSize(None)
		
		self.Layout()
		
		self.UpdateMenuAndToolbar()

		#Styling:
		self.txtDocument.StyleResetDefault()
		self.txtDocument.StyleClearAll()
		
		self.txtPrompt.StyleResetDefault()
		self.txtPrompt.StyleClearAll()
				
		self.txtDocument.SetupPrefsDocument(0)
		self.txtPrompt.SetupPrefsPrompt(0)
		
		#Shortcuts
		drShortcuts.SetSTCShortcuts(self.txtPrompt, self.STCShortcuts, self.STCUseDefault)
		self.STCKeycodeArray = drShortcuts.SetSTCShortcuts(self.txtDocument, self.STCShortcuts, self.STCUseDefault)
			
	
	def updatePrefs(self, oldprefs):		
		if (self.hasToolBar):
			self.bSizer.Remove(self.toolbar)
					
		self.bSizer.Remove(self.mdinotebook)
	
		#Prompt Size:
		self.currentpage.promptsize = self.prefs.promptsize
		
		self.currentpage.OnSize(None)
		
		#Find/Replace:
		if 	(self.prefs.findreplaceregularexpression is not oldprefs.findreplaceregularexpression) or \
		(self.prefs.findreplacematchcase is not oldprefs.findreplacematchcase) or \
		(self.prefs.findreplacefindbackwards is not oldprefs.findreplacefindbackwards) or \
		(self.prefs.findreplacewholeword is not oldprefs.findreplacewholeword) or \
		(self.prefs.findreplaceinselection is not oldprefs.findreplaceinselection) or \
		(self.prefs.findreplacefromcursor is not oldprefs.findreplacefromcursor) or \
		(self.prefs.findreplacepromptonreplace is not oldprefs.findreplacepromptonreplace):
			self.FindOptions = []
			self.ReplaceOptions = []
		
		#SourceBrowser:
		if not (self.prefs.sourcebrowserpane == oldprefs.sourcebrowserpane):
			for document in self.txtDocumentArray:
				if document.SourceBrowser:
					document.SourceBrowser = None
		
		#DrScript:
		if self.prefs.drscriptloadexamples is not oldprefs.drscriptloadexamples:
			self.drscriptmenu.reloadscripts()			
		
		#Toolbar
		if (self.prefs.iconsize > 0):
			if (self.hasToolBar):
				self.DestroyToolBar()
			else:
				self.toolbar = wx.ToolBar(self, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_HORIZONTAL)
			self.ToolBarIdList = self.SetupToolBar()
			self.hasToolBar = 1
		else:
			if (self.hasToolBar):
				self.DestroyToolBar()
				self.toolbar.Show(0)
				self.toolbar.Destroy()
				self.toolbar = None
				self.hasToolBar = 0
		
		#comment limodou 2004/04/13
		self.locale=self.prefs.encoding
		#end limodou
		
		if not (oldprefs.recentfileslimit is self.prefs.recentfileslimit):
			self.DestroyRecentFileMenu()
			self.recentfiles = []

			self.LoadRecentFiles()
			self.CreateRecentFileMenu()		

		#Styling:
		self.txtDocument.StyleResetDefault()
		self.txtDocument.StyleClearAll()
		
		self.txtPrompt.StyleResetDefault()
		self.txtPrompt.StyleClearAll()
		
		self.txtDocument.SetupPrefsDocument()
		self.txtPrompt.SetupPrefsPrompt()

		if (oldprefs.docfolding):
			if (not self.prefs.docfolding):
				self.txtDocument.FoldAll(True)
		
		#Work Around Bug in wx.Python 2.5.1
		try:
			self.Maximize(False)
		except:
			pass
				
		#Add The Stuff to the Sizer
		if (self.prefs.iconsize > 0):
			self.bSizer.Add(self.toolbar, 0, wx.EXPAND)
		
		self.bSizer.Add(self.mdinotebook, 1, wx.EXPAND)
		
		self.txtDocument.OnModified(None)
		
		self.Layout()
		
	def WriteRecentFiles(self):
		try:
			fin = open((self.homedirectory + "/recent_files.log"), 'w')
			x = 0
			length = len(self.recentfiles)
			while (x < self.prefs.recentfileslimit) and (x < length):
				fin.write(self.recentfiles[x] + '\n')
				x = x + 1
			fin.close()
		except IOError:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Error Writing: " + self.homedirectory + "/recent_files.log"), "Recent Files Error")
			d.ShowModal()
			d.Destroy()
			
#*******************************************************************************************************

class DrApp(wx.App):

	def OnInit(self):
				
		frame = DrFrame(None, 101, "DrPython - Untitled 1")	
		
		frame.Show(True)

		self.SetTopWindow(frame)

		return True
	
if __name__ == '__main__':
	app = DrApp(0)
	app.MainLoop()